"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cursorpagination import CursorPagination, CursorPaginationTypedDict
from .event import Event, EventTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceWithCursorPaginationEventTypedDict(TypedDict):
    items: List[EventTypedDict]
    pagination: CursorPaginationTypedDict


class ListResourceWithCursorPaginationEvent(BaseModel):
    items: List[Event]

    pagination: CursorPagination
