"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class CustomerPaymentMethodCreateRequiresActionResponseTypedDict(TypedDict):
    client_secret: str
    status: Literal["requires_action"]


class CustomerPaymentMethodCreateRequiresActionResponse(BaseModel):
    client_secret: str

    STATUS: Annotated[
        Annotated[
            Literal["requires_action"],
            AfterValidator(validate_const("requires_action")),
        ],
        pydantic.Field(alias="status"),
    ] = "requires_action"
