"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class OrderRefundedMetadataTypedDict(TypedDict):
    order_id: str
    refunded_amount: int
    currency: str


class OrderRefundedMetadata(BaseModel):
    order_id: str

    refunded_amount: int

    currency: str
