"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .costmetadata_output import CostMetadataOutput, CostMetadataOutputTypedDict
from .llmmetadata import LLMMetadata, LLMMetadataTypedDict
from typing import Union
from typing_extensions import TypeAliasType


EventMetadataOutputTypedDict = TypeAliasType(
    "EventMetadataOutputTypedDict",
    Union[CostMetadataOutputTypedDict, LLMMetadataTypedDict, str, int, float, bool],
)


EventMetadataOutput = TypeAliasType(
    "EventMetadataOutput", Union[CostMetadataOutput, LLMMetadata, str, int, float, bool]
)
