"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SubscriptionBillingPeriodUpdatedMetadataTypedDict(TypedDict):
    subscription_id: str
    old_period_end: str
    new_period_end: str


class SubscriptionBillingPeriodUpdatedMetadata(BaseModel):
    subscription_id: str

    old_period_end: str

    new_period_end: str
