"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CheckoutCreatedMetadataTypedDict(TypedDict):
    checkout_id: str
    checkout_status: str
    product_id: NotRequired[str]


class CheckoutCreatedMetadata(BaseModel):
    checkout_id: str

    checkout_status: str

    product_id: Optional[str] = None
