"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SeatClaimTypedDict(TypedDict):
    invitation_token: str
    r"""Invitation token to claim the seat"""


class SeatClaim(BaseModel):
    invitation_token: str
    r"""Invitation token to claim the seat"""
