"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpriceseattiers_input import (
    ProductPriceSeatTiersInput,
    ProductPriceSeatTiersInputTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ProductPriceSeatBasedCreateTypedDict(TypedDict):
    r"""Schema to create a seat-based price with volume-based tiers."""

    seat_tiers: ProductPriceSeatTiersInputTypedDict
    r"""List of pricing tiers for seat-based pricing.

    The minimum and maximum seat limits are derived from the tiers:
    - minimum_seats = first tier's min_seats
    - maximum_seats = last tier's max_seats (None for unlimited)
    """
    amount_type: Literal["seat_based"]
    price_currency: NotRequired[str]
    r"""The currency. Currently, only `usd` is supported."""


class ProductPriceSeatBasedCreate(BaseModel):
    r"""Schema to create a seat-based price with volume-based tiers."""

    seat_tiers: ProductPriceSeatTiersInput
    r"""List of pricing tiers for seat-based pricing.

    The minimum and maximum seat limits are derived from the tiers:
    - minimum_seats = first tier's min_seats
    - maximum_seats = last tier's max_seats (None for unlimited)
    """

    AMOUNT_TYPE: Annotated[
        Annotated[Literal["seat_based"], AfterValidator(validate_const("seat_based"))],
        pydantic.Field(alias="amount_type"),
    ] = "seat_based"

    price_currency: Optional[str] = "usd"
    r"""The currency. Currently, only `usd` is supported."""
