"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .authorizeuser import AuthorizeUser, AuthorizeUserTypedDict
from .oauth2clientpublic import OAuth2ClientPublic, OAuth2ClientPublicTypedDict
from .scope import Scope
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Dict, List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AuthorizeResponseUserTypedDict(TypedDict):
    client: OAuth2ClientPublicTypedDict
    sub: Nullable[AuthorizeUserTypedDict]
    scopes: List[Scope]
    sub_type: Literal["user"]
    scope_display_names: NotRequired[Dict[str, str]]


class AuthorizeResponseUser(BaseModel):
    client: OAuth2ClientPublic

    sub: Nullable[AuthorizeUser]

    scopes: List[Scope]

    SUB_TYPE: Annotated[
        Annotated[Literal["user"], AfterValidator(validate_const("user"))],
        pydantic.Field(alias="sub_type"),
    ] = "user"

    scope_display_names: Optional[Dict[str, str]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["scope_display_names"]
        nullable_fields = ["sub"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
