"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .costmetadata_input import CostMetadataInput, CostMetadataInputTypedDict
from .llmmetadata import LLMMetadata, LLMMetadataTypedDict
from typing import Union
from typing_extensions import TypeAliasType


EventMetadataInputTypedDict = TypeAliasType(
    "EventMetadataInputTypedDict",
    Union[CostMetadataInputTypedDict, LLMMetadataTypedDict, str, int, float, bool],
)


EventMetadataInput = TypeAliasType(
    "EventMetadataInput", Union[CostMetadataInput, LLMMetadata, str, int, float, bool]
)
