"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SeatStatus(str, Enum):
    PENDING = "pending"
    CLAIMED = "claimed"
    REVOKED = "revoked"
