"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerseat import CustomerSeat, CustomerSeatTypedDict
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomerSeatClaimResponseTypedDict(TypedDict):
    r"""Response after successfully claiming a seat."""

    seat: CustomerSeatTypedDict
    customer_session_token: str
    r"""Session token for immediate customer portal access"""


class CustomerSeatClaimResponse(BaseModel):
    r"""Response after successfully claiming a seat."""

    seat: CustomerSeat

    customer_session_token: str
    r"""Session token for immediate customer portal access"""
