"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SubscriptionSeatsUpdatedMetadataTypedDict(TypedDict):
    subscription_id: str
    old_seats: int
    new_seats: int
    proration_behavior: str


class SubscriptionSeatsUpdatedMetadata(BaseModel):
    subscription_id: str

    old_seats: int

    new_seats: int

    proration_behavior: str
