"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .eventtypewithstats import EventTypeWithStats, EventTypeWithStatsTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceEventTypeWithStatsTypedDict(TypedDict):
    items: List[EventTypeWithStatsTypedDict]
    pagination: PaginationTypedDict


class ListResourceEventTypeWithStats(BaseModel):
    items: List[EventTypeWithStats]

    pagination: Pagination
