"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PaymentSortProperty(str, Enum):
    CREATED_AT = "created_at"
    MINUS_CREATED_AT = "-created_at"
    STATUS = "status"
    MINUS_STATUS = "-status"
    AMOUNT = "amount"
    MINUS_AMOUNT = "-amount"
    METHOD = "method"
    MINUS_METHOD = "-method"
