"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .member import Member, MemberTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceMemberTypedDict(TypedDict):
    items: List[MemberTypedDict]
    pagination: PaginationTypedDict


class ListResourceMember(BaseModel):
    items: List[Member]

    pagination: Pagination
