"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CostMetadataOutputTypedDict(TypedDict):
    amount: str
    r"""The amount in cents."""
    currency: str
    r"""The currency. Currently, only `usd` is supported."""


class CostMetadataOutput(BaseModel):
    amount: str
    r"""The amount in cents."""

    currency: str
    r"""The currency. Currently, only `usd` is supported."""
