"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomerPortalSubscriptionSettingsTypedDict(TypedDict):
    update_seats: bool
    update_plan: bool


class CustomerPortalSubscriptionSettings(BaseModel):
    update_seats: bool

    update_plan: bool
