"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CursorPaginationTypedDict(TypedDict):
    has_next_page: bool


class CursorPagination(BaseModel):
    has_next_page: bool
