"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerbenefitgrantdiscordpropertiesupdate import (
    CustomerBenefitGrantDiscordPropertiesUpdate,
    CustomerBenefitGrantDiscordPropertiesUpdateTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class CustomerBenefitGrantDiscordUpdateTypedDict(TypedDict):
    properties: CustomerBenefitGrantDiscordPropertiesUpdateTypedDict
    benefit_type: Literal["discord"]


class CustomerBenefitGrantDiscordUpdate(BaseModel):
    properties: CustomerBenefitGrantDiscordPropertiesUpdate

    BENEFIT_TYPE: Annotated[
        Annotated[Literal["discord"], AfterValidator(validate_const("discord"))],
        pydantic.Field(alias="benefit_type"),
    ] = "discord"
