"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpriceseattier import ProductPriceSeatTier, ProductPriceSeatTierTypedDict
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List
from typing_extensions import TypedDict


class ProductPriceSeatTiersOutputTypedDict(TypedDict):
    r"""List of pricing tiers for seat-based pricing.

    The minimum and maximum seat limits are derived from the tiers:
    - minimum_seats = first tier's min_seats
    - maximum_seats = last tier's max_seats (None for unlimited)
    """

    tiers: List[ProductPriceSeatTierTypedDict]
    r"""List of pricing tiers"""
    minimum_seats: int
    r"""Minimum number of seats required for purchase, derived from first tier."""
    maximum_seats: Nullable[int]
    r"""Maximum number of seats allowed for purchase, derived from last tier. None for unlimited."""


class ProductPriceSeatTiersOutput(BaseModel):
    r"""List of pricing tiers for seat-based pricing.

    The minimum and maximum seat limits are derived from the tiers:
    - minimum_seats = first tier's min_seats
    - maximum_seats = last tier's max_seats (None for unlimited)
    """

    tiers: List[ProductPriceSeatTier]
    r"""List of pricing tiers"""

    minimum_seats: int
    r"""Minimum number of seats required for purchase, derived from first tier."""

    maximum_seats: Nullable[int]
    r"""Maximum number of seats allowed for purchase, derived from last tier. None for unlimited."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["maximum_seats"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
