"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitgrantcustomwebhook import (
    BenefitGrantCustomWebhook,
    BenefitGrantCustomWebhookTypedDict,
)
from .benefitgrantdiscordwebhook import (
    BenefitGrantDiscordWebhook,
    BenefitGrantDiscordWebhookTypedDict,
)
from .benefitgrantdownloadableswebhook import (
    BenefitGrantDownloadablesWebhook,
    BenefitGrantDownloadablesWebhookTypedDict,
)
from .benefitgrantgithubrepositorywebhook import (
    BenefitGrantGitHubRepositoryWebhook,
    BenefitGrantGitHubRepositoryWebhookTypedDict,
)
from .benefitgrantlicensekeyswebhook import (
    BenefitGrantLicenseKeysWebhook,
    BenefitGrantLicenseKeysWebhookTypedDict,
)
from .benefitgrantmetercreditwebhook import (
    BenefitGrantMeterCreditWebhook,
    BenefitGrantMeterCreditWebhookTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


BenefitGrantWebhookTypedDict = TypeAliasType(
    "BenefitGrantWebhookTypedDict",
    Union[
        BenefitGrantDiscordWebhookTypedDict,
        BenefitGrantCustomWebhookTypedDict,
        BenefitGrantGitHubRepositoryWebhookTypedDict,
        BenefitGrantDownloadablesWebhookTypedDict,
        BenefitGrantLicenseKeysWebhookTypedDict,
        BenefitGrantMeterCreditWebhookTypedDict,
    ],
)


BenefitGrantWebhook = TypeAliasType(
    "BenefitGrantWebhook",
    Union[
        BenefitGrantDiscordWebhook,
        BenefitGrantCustomWebhook,
        BenefitGrantGitHubRepositoryWebhook,
        BenefitGrantDownloadablesWebhook,
        BenefitGrantLicenseKeysWebhook,
        BenefitGrantMeterCreditWebhook,
    ],
)
