"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerorganization import CustomerOrganization, CustomerOrganizationTypedDict
from .customerproduct import CustomerProduct, CustomerProductTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class CustomerOrganizationDataTypedDict(TypedDict):
    r"""Schema of an organization and related data for customer portal."""

    organization: CustomerOrganizationTypedDict
    products: List[CustomerProductTypedDict]


class CustomerOrganizationData(BaseModel):
    r"""Schema of an organization and related data for customer portal."""

    organization: CustomerOrganization

    products: List[CustomerProduct]
