"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class BenefitGrantSortProperty(str, Enum):
    CREATED_AT = "created_at"
    MINUS_CREATED_AT = "-created_at"
    GRANTED_AT = "granted_at"
    MINUS_GRANTED_AT = "-granted_at"
    REVOKED_AT = "revoked_at"
    MINUS_REVOKED_AT = "-revoked_at"
