"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerportalsubscriptionsettings import (
    CustomerPortalSubscriptionSettings,
    CustomerPortalSubscriptionSettingsTypedDict,
)
from .customerportalusagesettings import (
    CustomerPortalUsageSettings,
    CustomerPortalUsageSettingsTypedDict,
)
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class OrganizationCustomerPortalSettingsTypedDict(TypedDict):
    usage: CustomerPortalUsageSettingsTypedDict
    subscription: CustomerPortalSubscriptionSettingsTypedDict


class OrganizationCustomerPortalSettings(BaseModel):
    usage: CustomerPortalUsageSettings

    subscription: CustomerPortalSubscriptionSettings
