"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SubscriptionUpdateBillingPeriodTypedDict(TypedDict):
    current_billing_period_end: datetime
    r"""Set a new date for the end of the current billing period. The subscription will renew on this date. Needs to be later than the current value.

    It is not possible to update the current billing period on a canceled subscription.
    """


class SubscriptionUpdateBillingPeriod(BaseModel):
    current_billing_period_end: datetime
    r"""Set a new date for the end of the current billing period. The subscription will renew on this date. Needs to be later than the current value.

    It is not possible to update the current billing period on a canceled subscription.
    """
