"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dispute import Dispute, DisputeTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceDisputeTypedDict(TypedDict):
    items: List[DisputeTypedDict]
    pagination: PaginationTypedDict


class ListResourceDispute(BaseModel):
    items: List[Dispute]

    pagination: Pagination
