"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DisputeStatus(str, Enum):
    PREVENTED = "prevented"
    EARLY_WARNING = "early_warning"
    NEEDS_RESPONSE = "needs_response"
    UNDER_REVIEW = "under_review"
    LOST = "lost"
    WON = "won"
