"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class OrganizationSortProperty(str, Enum):
    CREATED_AT = "created_at"
    MINUS_CREATED_AT = "-created_at"
    SLUG = "slug"
    MINUS_SLUG = "-slug"
    NAME = "name"
    MINUS_NAME = "-name"
    NEXT_REVIEW_THRESHOLD = "next_review_threshold"
    MINUS_NEXT_REVIEW_THRESHOLD = "-next_review_threshold"
    DAYS_IN_STATUS = "days_in_status"
    MINUS_DAYS_IN_STATUS = "-days_in_status"
