"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ProductPriceSource(str, Enum):
    CATALOG = "catalog"
    AD_HOC = "ad_hoc"
