"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class BalanceOrderMetadataTypedDict(TypedDict):
    transaction_id: str
    order_id: str
    amount: int
    currency: str
    presentment_amount: int
    presentment_currency: str
    tax_amount: int
    fee: int
    product_id: NotRequired[str]
    subscription_id: NotRequired[str]
    tax_state: NotRequired[Nullable[str]]
    tax_country: NotRequired[Nullable[str]]


class BalanceOrderMetadata(BaseModel):
    transaction_id: str

    order_id: str

    amount: int

    currency: str

    presentment_amount: int

    presentment_currency: str

    tax_amount: int

    fee: int

    product_id: Optional[str] = None

    subscription_id: Optional[str] = None

    tax_state: OptionalNullable[str] = UNSET

    tax_country: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["product_id", "subscription_id", "tax_state", "tax_country"]
        nullable_fields = ["tax_state", "tax_country"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
