"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .memberrole import MemberRole
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class MemberCreateTypedDict(TypedDict):
    r"""Schema for creating a new member."""

    customer_id: str
    r"""The ID of the customer this member belongs to."""
    email: str
    r"""The email address of the member."""
    name: NotRequired[Nullable[str]]
    external_id: NotRequired[Nullable[str]]
    r"""The ID of the member in your system. This must be unique within the customer."""
    role: NotRequired[MemberRole]


class MemberCreate(BaseModel):
    r"""Schema for creating a new member."""

    customer_id: str
    r"""The ID of the customer this member belongs to."""

    email: str
    r"""The email address of the member."""

    name: OptionalNullable[str] = UNSET

    external_id: OptionalNullable[str] = UNSET
    r"""The ID of the member in your system. This must be unique within the customer."""

    role: Optional[MemberRole] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["name", "external_id", "role"]
        nullable_fields = ["name", "external_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
