"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WebhookFormat(str, Enum):
    RAW = "raw"
    DISCORD = "discord"
    SLACK = "slack"
