"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptioncreatecustomer import (
    SubscriptionCreateCustomer,
    SubscriptionCreateCustomerTypedDict,
)
from .subscriptioncreateexternalcustomer import (
    SubscriptionCreateExternalCustomer,
    SubscriptionCreateExternalCustomerTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


SubscriptionsCreateSubscriptionCreateTypedDict = TypeAliasType(
    "SubscriptionsCreateSubscriptionCreateTypedDict",
    Union[
        SubscriptionCreateCustomerTypedDict, SubscriptionCreateExternalCustomerTypedDict
    ],
)


SubscriptionsCreateSubscriptionCreate = TypeAliasType(
    "SubscriptionsCreateSubscriptionCreate",
    Union[SubscriptionCreateCustomer, SubscriptionCreateExternalCustomer],
)
