"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpriceseattier import ProductPriceSeatTier, ProductPriceSeatTierTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ProductPriceSeatTiersInputTypedDict(TypedDict):
    r"""List of pricing tiers for seat-based pricing.

    The minimum and maximum seat limits are derived from the tiers:
    - minimum_seats = first tier's min_seats
    - maximum_seats = last tier's max_seats (None for unlimited)
    """

    tiers: List[ProductPriceSeatTierTypedDict]
    r"""List of pricing tiers"""


class ProductPriceSeatTiersInput(BaseModel):
    r"""List of pricing tiers for seat-based pricing.

    The minimum and maximum seat limits are derived from the tiers:
    - minimum_seats = first tier's min_seats
    - maximum_seats = last tier's max_seats (None for unlimited)
    """

    tiers: List[ProductPriceSeatTier]
    r"""List of pricing tiers"""
