"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WebhookEventType(str, Enum):
    CHECKOUT_CREATED = "checkout.created"
    CHECKOUT_UPDATED = "checkout.updated"
    CUSTOMER_CREATED = "customer.created"
    CUSTOMER_UPDATED = "customer.updated"
    CUSTOMER_DELETED = "customer.deleted"
    CUSTOMER_STATE_CHANGED = "customer.state_changed"
    CUSTOMER_SEAT_ASSIGNED = "customer_seat.assigned"
    CUSTOMER_SEAT_CLAIMED = "customer_seat.claimed"
    CUSTOMER_SEAT_REVOKED = "customer_seat.revoked"
    ORDER_CREATED = "order.created"
    ORDER_UPDATED = "order.updated"
    ORDER_PAID = "order.paid"
    ORDER_REFUNDED = "order.refunded"
    SUBSCRIPTION_CREATED = "subscription.created"
    SUBSCRIPTION_UPDATED = "subscription.updated"
    SUBSCRIPTION_ACTIVE = "subscription.active"
    SUBSCRIPTION_CANCELED = "subscription.canceled"
    SUBSCRIPTION_UNCANCELED = "subscription.uncanceled"
    SUBSCRIPTION_REVOKED = "subscription.revoked"
    SUBSCRIPTION_PAST_DUE = "subscription.past_due"
    REFUND_CREATED = "refund.created"
    REFUND_UPDATED = "refund.updated"
    PRODUCT_CREATED = "product.created"
    PRODUCT_UPDATED = "product.updated"
    BENEFIT_CREATED = "benefit.created"
    BENEFIT_UPDATED = "benefit.updated"
    BENEFIT_GRANT_CREATED = "benefit_grant.created"
    BENEFIT_GRANT_CYCLED = "benefit_grant.cycled"
    BENEFIT_GRANT_UPDATED = "benefit_grant.updated"
    BENEFIT_GRANT_REVOKED = "benefit_grant.revoked"
    ORGANIZATION_UPDATED = "organization.updated"
