"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SeatClaimInfoTypedDict(TypedDict):
    r"""Read-only information about a seat claim invitation.
    Safe for email scanners - no side effects when fetched.
    """

    product_name: str
    r"""Name of the product"""
    product_id: str
    r"""ID of the product"""
    organization_name: str
    r"""Name of the organization"""
    organization_slug: str
    r"""Slug of the organization"""
    customer_email: str
    r"""Email of the customer assigned to this seat"""
    can_claim: bool
    r"""Whether the seat can be claimed"""


class SeatClaimInfo(BaseModel):
    r"""Read-only information about a seat claim invitation.
    Safe for email scanners - no side effects when fetched.
    """

    product_name: str
    r"""Name of the product"""

    product_id: str
    r"""ID of the product"""

    organization_name: str
    r"""Name of the organization"""

    organization_slug: str
    r"""Slug of the organization"""

    customer_email: str
    r"""Email of the customer assigned to this seat"""

    can_claim: bool
    r"""Whether the seat can be claimed"""
