"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class OrganizationStatus(str, Enum):
    CREATED = "created"
    ONBOARDING_STARTED = "onboarding_started"
    INITIAL_REVIEW = "initial_review"
    ONGOING_REVIEW = "ongoing_review"
    DENIED = "denied"
    ACTIVE = "active"
