"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class AddressDictTypedDict(TypedDict):
    country: str
    line1: NotRequired[str]
    line2: NotRequired[str]
    postal_code: NotRequired[str]
    city: NotRequired[str]
    state: NotRequired[str]


class AddressDict(BaseModel):
    country: str

    line1: Optional[str] = None

    line2: Optional[str] = None

    postal_code: Optional[str] = None

    city: Optional[str] = None

    state: Optional[str] = None
