"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SubscriptionCreatedMetadataTypedDict(TypedDict):
    subscription_id: str
    product_id: str
    amount: int
    currency: str
    recurring_interval: str
    recurring_interval_count: int
    started_at: str


class SubscriptionCreatedMetadata(BaseModel):
    subscription_id: str

    product_id: str

    amount: int

    currency: str

    recurring_interval: str

    recurring_interval_count: int

    started_at: str
