"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .disputestatus import DisputeStatus
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import TypedDict


class RefundDisputeTypedDict(TypedDict):
    r"""Dispute associated with a refund,
    in case we prevented a dispute by issuing a refund.
    """

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    status: DisputeStatus
    resolved: bool
    r"""Whether the dispute has been resolved (won or lost)."""
    closed: bool
    r"""Whether the dispute is closed (prevented, won, or lost)."""
    amount: int
    r"""Amount in cents disputed."""
    tax_amount: int
    r"""Tax amount in cents disputed."""
    currency: str
    r"""Currency code of the dispute."""
    order_id: str
    r"""The ID of the order associated with the dispute."""
    payment_id: str
    r"""The ID of the payment associated with the dispute."""


class RefundDispute(BaseModel):
    r"""Dispute associated with a refund,
    in case we prevented a dispute by issuing a refund.
    """

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    status: DisputeStatus

    resolved: bool
    r"""Whether the dispute has been resolved (won or lost)."""

    closed: bool
    r"""Whether the dispute is closed (prevented, won, or lost)."""

    amount: int
    r"""Amount in cents disputed."""

    tax_amount: int
    r"""Tax amount in cents disputed."""

    currency: str
    r"""Currency code of the dispute."""

    order_id: str
    r"""The ID of the order associated with the dispute."""

    payment_id: str
    r"""The ID of the payment associated with the dispute."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
