"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Union
from typing_extensions import TypeAliasType, TypedDict


AmountTypedDict = TypeAliasType("AmountTypedDict", Union[float, str])
r"""The amount in cents."""


Amount = TypeAliasType("Amount", Union[float, str])
r"""The amount in cents."""


class CostMetadataInputTypedDict(TypedDict):
    amount: AmountTypedDict
    r"""The amount in cents."""
    currency: str
    r"""The currency. Currently, only `usd` is supported."""


class CostMetadataInput(BaseModel):
    amount: Amount
    r"""The amount in cents."""

    currency: str
    r"""The currency. Currently, only `usd` is supported."""
