"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class MemberRole(str, Enum):
    OWNER = "owner"
    BILLING_MANAGER = "billing_manager"
    MEMBER = "member"
