"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .filter_ import Clauses, ClausesTypedDict, Filter, FilterTypedDict
from .polarerror import PolarError
from typing import TYPE_CHECKING
from importlib import import_module
import builtins
import sys

if TYPE_CHECKING:
    from .address import Address, AddressTypedDict, CountryAlpha2
    from .addressdict import AddressDict, AddressDictTypedDict
    from .addressinput import AddressInput, AddressInputTypedDict, CountryAlpha2Input
    from .aggregationfunction import AggregationFunction
    from .alreadyactivesubscriptionerror import (
        AlreadyActiveSubscriptionError,
        AlreadyActiveSubscriptionErrorData,
    )
    from .alreadycanceledsubscription import (
        AlreadyCanceledSubscription,
        AlreadyCanceledSubscriptionData,
    )
    from .attachedcustomfield import AttachedCustomField, AttachedCustomFieldTypedDict
    from .attachedcustomfieldcreate import (
        AttachedCustomFieldCreate,
        AttachedCustomFieldCreateTypedDict,
    )
    from .authorizationcodetokenrequest import (
        AuthorizationCodeTokenRequest,
        AuthorizationCodeTokenRequestTypedDict,
    )
    from .authorizeorganization import (
        AuthorizeOrganization,
        AuthorizeOrganizationTypedDict,
    )
    from .authorizeresponseorganization import (
        AuthorizeResponseOrganization,
        AuthorizeResponseOrganizationTypedDict,
    )
    from .authorizeresponseuser import (
        AuthorizeResponseUser,
        AuthorizeResponseUserTypedDict,
    )
    from .authorizeuser import AuthorizeUser, AuthorizeUserTypedDict
    from .balancedisputeevent import BalanceDisputeEvent, BalanceDisputeEventTypedDict
    from .balancedisputemetadata import (
        BalanceDisputeMetadata,
        BalanceDisputeMetadataTypedDict,
    )
    from .balancedisputereversalevent import (
        BalanceDisputeReversalEvent,
        BalanceDisputeReversalEventTypedDict,
    )
    from .balanceorderevent import BalanceOrderEvent, BalanceOrderEventTypedDict
    from .balanceordermetadata import (
        BalanceOrderMetadata,
        BalanceOrderMetadataTypedDict,
    )
    from .balancerefundevent import BalanceRefundEvent, BalanceRefundEventTypedDict
    from .balancerefundmetadata import (
        BalanceRefundMetadata,
        BalanceRefundMetadataTypedDict,
    )
    from .balancerefundreversalevent import (
        BalanceRefundReversalEvent,
        BalanceRefundReversalEventTypedDict,
    )
    from .benefit import Benefit, BenefitTypedDict
    from .benefit_grants_listop import (
        BenefitGrantsListQueryParamCustomerIDFilter,
        BenefitGrantsListQueryParamCustomerIDFilterTypedDict,
        BenefitGrantsListQueryParamOrganizationIDFilter,
        BenefitGrantsListQueryParamOrganizationIDFilterTypedDict,
        BenefitGrantsListRequest,
        BenefitGrantsListRequestTypedDict,
        BenefitGrantsListResponse,
        BenefitGrantsListResponseTypedDict,
    )
    from .benefitcreate import BenefitCreate, BenefitCreateTypedDict
    from .benefitcustom import BenefitCustom, BenefitCustomTypedDict
    from .benefitcustomcreate import (
        BenefitCustomCreate,
        BenefitCustomCreateMetadata,
        BenefitCustomCreateMetadataTypedDict,
        BenefitCustomCreateTypedDict,
    )
    from .benefitcustomcreateproperties import (
        BenefitCustomCreateProperties,
        BenefitCustomCreatePropertiesTypedDict,
    )
    from .benefitcustomproperties import (
        BenefitCustomProperties,
        BenefitCustomPropertiesTypedDict,
    )
    from .benefitcustomsubscriber import (
        BenefitCustomSubscriber,
        BenefitCustomSubscriberTypedDict,
    )
    from .benefitcustomsubscriberproperties import (
        BenefitCustomSubscriberProperties,
        BenefitCustomSubscriberPropertiesTypedDict,
    )
    from .benefitcustomupdate import (
        BenefitCustomUpdate,
        BenefitCustomUpdateMetadata,
        BenefitCustomUpdateMetadataTypedDict,
        BenefitCustomUpdateTypedDict,
    )
    from .benefitcycledevent import BenefitCycledEvent, BenefitCycledEventTypedDict
    from .benefitdiscord import BenefitDiscord, BenefitDiscordTypedDict
    from .benefitdiscordcreate import (
        BenefitDiscordCreate,
        BenefitDiscordCreateMetadata,
        BenefitDiscordCreateMetadataTypedDict,
        BenefitDiscordCreateTypedDict,
    )
    from .benefitdiscordcreateproperties import (
        BenefitDiscordCreateProperties,
        BenefitDiscordCreatePropertiesTypedDict,
    )
    from .benefitdiscordproperties import (
        BenefitDiscordProperties,
        BenefitDiscordPropertiesTypedDict,
    )
    from .benefitdiscordsubscriber import (
        BenefitDiscordSubscriber,
        BenefitDiscordSubscriberTypedDict,
    )
    from .benefitdiscordsubscriberproperties import (
        BenefitDiscordSubscriberProperties,
        BenefitDiscordSubscriberPropertiesTypedDict,
    )
    from .benefitdiscordupdate import (
        BenefitDiscordUpdate,
        BenefitDiscordUpdateMetadata,
        BenefitDiscordUpdateMetadataTypedDict,
        BenefitDiscordUpdateTypedDict,
    )
    from .benefitdownloadables import (
        BenefitDownloadables,
        BenefitDownloadablesTypedDict,
    )
    from .benefitdownloadablescreate import (
        BenefitDownloadablesCreate,
        BenefitDownloadablesCreateMetadata,
        BenefitDownloadablesCreateMetadataTypedDict,
        BenefitDownloadablesCreateTypedDict,
    )
    from .benefitdownloadablescreateproperties import (
        BenefitDownloadablesCreateProperties,
        BenefitDownloadablesCreatePropertiesTypedDict,
    )
    from .benefitdownloadablesproperties import (
        BenefitDownloadablesProperties,
        BenefitDownloadablesPropertiesTypedDict,
    )
    from .benefitdownloadablessubscriber import (
        BenefitDownloadablesSubscriber,
        BenefitDownloadablesSubscriberTypedDict,
    )
    from .benefitdownloadablessubscriberproperties import (
        BenefitDownloadablesSubscriberProperties,
        BenefitDownloadablesSubscriberPropertiesTypedDict,
    )
    from .benefitdownloadablesupdate import (
        BenefitDownloadablesUpdate,
        BenefitDownloadablesUpdateMetadata,
        BenefitDownloadablesUpdateMetadataTypedDict,
        BenefitDownloadablesUpdateTypedDict,
    )
    from .benefitgithubrepository import (
        BenefitGitHubRepository,
        BenefitGitHubRepositoryTypedDict,
    )
    from .benefitgithubrepositorycreate import (
        BenefitGitHubRepositoryCreate,
        BenefitGitHubRepositoryCreateMetadata,
        BenefitGitHubRepositoryCreateMetadataTypedDict,
        BenefitGitHubRepositoryCreateTypedDict,
    )
    from .benefitgithubrepositorycreateproperties import (
        BenefitGitHubRepositoryCreateProperties,
        BenefitGitHubRepositoryCreatePropertiesPermission,
        BenefitGitHubRepositoryCreatePropertiesTypedDict,
    )
    from .benefitgithubrepositoryproperties import (
        BenefitGitHubRepositoryProperties,
        BenefitGitHubRepositoryPropertiesTypedDict,
        Permission,
    )
    from .benefitgithubrepositorysubscriber import (
        BenefitGitHubRepositorySubscriber,
        BenefitGitHubRepositorySubscriberTypedDict,
    )
    from .benefitgithubrepositorysubscriberproperties import (
        BenefitGitHubRepositorySubscriberProperties,
        BenefitGitHubRepositorySubscriberPropertiesTypedDict,
    )
    from .benefitgithubrepositoryupdate import (
        BenefitGitHubRepositoryUpdate,
        BenefitGitHubRepositoryUpdateMetadata,
        BenefitGitHubRepositoryUpdateMetadataTypedDict,
        BenefitGitHubRepositoryUpdateTypedDict,
    )
    from .benefitgrant import (
        BenefitGrant,
        BenefitGrantTypedDict,
        Properties,
        PropertiesTypedDict,
    )
    from .benefitgrantcustomproperties import (
        BenefitGrantCustomProperties,
        BenefitGrantCustomPropertiesTypedDict,
    )
    from .benefitgrantcustomwebhook import (
        BenefitGrantCustomWebhook,
        BenefitGrantCustomWebhookTypedDict,
    )
    from .benefitgrantdiscordproperties import (
        BenefitGrantDiscordProperties,
        BenefitGrantDiscordPropertiesTypedDict,
    )
    from .benefitgrantdiscordwebhook import (
        BenefitGrantDiscordWebhook,
        BenefitGrantDiscordWebhookTypedDict,
    )
    from .benefitgrantdownloadablesproperties import (
        BenefitGrantDownloadablesProperties,
        BenefitGrantDownloadablesPropertiesTypedDict,
    )
    from .benefitgrantdownloadableswebhook import (
        BenefitGrantDownloadablesWebhook,
        BenefitGrantDownloadablesWebhookTypedDict,
    )
    from .benefitgrantedevent import BenefitGrantedEvent, BenefitGrantedEventTypedDict
    from .benefitgranterror import BenefitGrantError, BenefitGrantErrorTypedDict
    from .benefitgrantgithubrepositoryproperties import (
        BenefitGrantGitHubRepositoryProperties,
        BenefitGrantGitHubRepositoryPropertiesPermission,
        BenefitGrantGitHubRepositoryPropertiesTypedDict,
    )
    from .benefitgrantgithubrepositorywebhook import (
        BenefitGrantGitHubRepositoryWebhook,
        BenefitGrantGitHubRepositoryWebhookTypedDict,
    )
    from .benefitgrantlicensekeysproperties import (
        BenefitGrantLicenseKeysProperties,
        BenefitGrantLicenseKeysPropertiesTypedDict,
    )
    from .benefitgrantlicensekeyswebhook import (
        BenefitGrantLicenseKeysWebhook,
        BenefitGrantLicenseKeysWebhookTypedDict,
    )
    from .benefitgrantmetadata import (
        BenefitGrantMetadata,
        BenefitGrantMetadataTypedDict,
    )
    from .benefitgrantmetercreditproperties import (
        BenefitGrantMeterCreditProperties,
        BenefitGrantMeterCreditPropertiesTypedDict,
    )
    from .benefitgrantmetercreditwebhook import (
        BenefitGrantMeterCreditWebhook,
        BenefitGrantMeterCreditWebhookTypedDict,
    )
    from .benefitgrantsortproperty import BenefitGrantSortProperty
    from .benefitgrantwebhook import BenefitGrantWebhook, BenefitGrantWebhookTypedDict
    from .benefitlicensekeyactivationcreateproperties import (
        BenefitLicenseKeyActivationCreateProperties,
        BenefitLicenseKeyActivationCreatePropertiesTypedDict,
    )
    from .benefitlicensekeyactivationproperties import (
        BenefitLicenseKeyActivationProperties,
        BenefitLicenseKeyActivationPropertiesTypedDict,
    )
    from .benefitlicensekeyexpirationproperties import (
        BenefitLicenseKeyExpirationProperties,
        BenefitLicenseKeyExpirationPropertiesTypedDict,
        Timeframe,
    )
    from .benefitlicensekeys import BenefitLicenseKeys, BenefitLicenseKeysTypedDict
    from .benefitlicensekeyscreate import (
        BenefitLicenseKeysCreate,
        BenefitLicenseKeysCreateMetadata,
        BenefitLicenseKeysCreateMetadataTypedDict,
        BenefitLicenseKeysCreateTypedDict,
    )
    from .benefitlicensekeyscreateproperties import (
        BenefitLicenseKeysCreateProperties,
        BenefitLicenseKeysCreatePropertiesTypedDict,
    )
    from .benefitlicensekeysproperties import (
        BenefitLicenseKeysProperties,
        BenefitLicenseKeysPropertiesTypedDict,
    )
    from .benefitlicensekeyssubscriber import (
        BenefitLicenseKeysSubscriber,
        BenefitLicenseKeysSubscriberTypedDict,
    )
    from .benefitlicensekeyssubscriberproperties import (
        BenefitLicenseKeysSubscriberProperties,
        BenefitLicenseKeysSubscriberPropertiesTypedDict,
    )
    from .benefitlicensekeysupdate import (
        BenefitLicenseKeysUpdate,
        BenefitLicenseKeysUpdateMetadata,
        BenefitLicenseKeysUpdateMetadataTypedDict,
        BenefitLicenseKeysUpdateTypedDict,
    )
    from .benefitmetercredit import BenefitMeterCredit, BenefitMeterCreditTypedDict
    from .benefitmetercreditcreate import (
        BenefitMeterCreditCreate,
        BenefitMeterCreditCreateMetadata,
        BenefitMeterCreditCreateMetadataTypedDict,
        BenefitMeterCreditCreateTypedDict,
    )
    from .benefitmetercreditcreateproperties import (
        BenefitMeterCreditCreateProperties,
        BenefitMeterCreditCreatePropertiesTypedDict,
    )
    from .benefitmetercreditproperties import (
        BenefitMeterCreditProperties,
        BenefitMeterCreditPropertiesTypedDict,
    )
    from .benefitmetercreditsubscriber import (
        BenefitMeterCreditSubscriber,
        BenefitMeterCreditSubscriberTypedDict,
    )
    from .benefitmetercreditsubscriberproperties import (
        BenefitMeterCreditSubscriberProperties,
        BenefitMeterCreditSubscriberPropertiesTypedDict,
    )
    from .benefitmetercreditupdate import (
        BenefitMeterCreditUpdate,
        BenefitMeterCreditUpdateMetadata,
        BenefitMeterCreditUpdateMetadataTypedDict,
        BenefitMeterCreditUpdateTypedDict,
    )
    from .benefitpublic import BenefitPublic, BenefitPublicTypedDict
    from .benefitrevokedevent import BenefitRevokedEvent, BenefitRevokedEventTypedDict
    from .benefits_deleteop import BenefitsDeleteRequest, BenefitsDeleteRequestTypedDict
    from .benefits_getop import BenefitsGetRequest, BenefitsGetRequestTypedDict
    from .benefits_grantsop import (
        BenefitsGrantsRequest,
        BenefitsGrantsRequestTypedDict,
        BenefitsGrantsResponse,
        BenefitsGrantsResponseTypedDict,
        MemberIDFilter,
        MemberIDFilterTypedDict,
        QueryParamCustomerIDFilter,
        QueryParamCustomerIDFilterTypedDict,
    )
    from .benefits_updateop import (
        BenefitsUpdateBenefitUpdate,
        BenefitsUpdateBenefitUpdateTypedDict,
        BenefitsUpdateRequest,
        BenefitsUpdateRequestTypedDict,
    )
    from .benefitsortproperty import BenefitSortProperty
    from .benefitsubscriberorganization import (
        BenefitSubscriberOrganization,
        BenefitSubscriberOrganizationTypedDict,
    )
    from .benefittype import BenefitType
    from .benefitupdatedevent import BenefitUpdatedEvent, BenefitUpdatedEventTypedDict
    from .billingaddressfieldmode import BillingAddressFieldMode
    from .cardpayment import CardPayment, CardPaymentTypedDict
    from .cardpaymentmetadata import CardPaymentMetadata, CardPaymentMetadataTypedDict
    from .checkout import (
        Checkout,
        CheckoutCustomFieldData,
        CheckoutCustomFieldDataTypedDict,
        CheckoutDiscount,
        CheckoutDiscountTypedDict,
        CheckoutPrices,
        CheckoutPricesTypedDict,
        CheckoutProductPrice,
        CheckoutProductPriceTypedDict,
        CheckoutTypedDict,
        CustomerMetadata,
        CustomerMetadataTypedDict,
    )
    from .checkout_links_deleteop import (
        CheckoutLinksDeleteRequest,
        CheckoutLinksDeleteRequestTypedDict,
    )
    from .checkout_links_getop import (
        CheckoutLinksGetRequest,
        CheckoutLinksGetRequestTypedDict,
    )
    from .checkout_links_listop import (
        CheckoutLinksListQueryParamOrganizationIDFilter,
        CheckoutLinksListQueryParamOrganizationIDFilterTypedDict,
        CheckoutLinksListQueryParamProductIDFilter,
        CheckoutLinksListQueryParamProductIDFilterTypedDict,
        CheckoutLinksListRequest,
        CheckoutLinksListRequestTypedDict,
        CheckoutLinksListResponse,
        CheckoutLinksListResponseTypedDict,
    )
    from .checkout_links_updateop import (
        CheckoutLinksUpdateRequest,
        CheckoutLinksUpdateRequestTypedDict,
    )
    from .checkoutbillingaddressfields import (
        CheckoutBillingAddressFields,
        CheckoutBillingAddressFieldsTypedDict,
    )
    from .checkoutconfirmstripe import (
        CheckoutConfirmStripe,
        CheckoutConfirmStripeCustomFieldData,
        CheckoutConfirmStripeCustomFieldDataTypedDict,
        CheckoutConfirmStripeTypedDict,
    )
    from .checkoutcreate import (
        CheckoutCreate,
        CheckoutCreateCustomFieldData,
        CheckoutCreateCustomFieldDataTypedDict,
        CheckoutCreateCustomerMetadata,
        CheckoutCreateCustomerMetadataTypedDict,
        CheckoutCreateMetadata,
        CheckoutCreateMetadataTypedDict,
        CheckoutCreatePrices,
        CheckoutCreatePricesTypedDict,
        CheckoutCreateTypedDict,
    )
    from .checkoutcreatedevent import (
        CheckoutCreatedEvent,
        CheckoutCreatedEventTypedDict,
    )
    from .checkoutcreatedmetadata import (
        CheckoutCreatedMetadata,
        CheckoutCreatedMetadataTypedDict,
    )
    from .checkoutdiscountfixedonceforeverduration import (
        CheckoutDiscountFixedOnceForeverDuration,
        CheckoutDiscountFixedOnceForeverDurationTypedDict,
    )
    from .checkoutdiscountfixedrepeatduration import (
        CheckoutDiscountFixedRepeatDuration,
        CheckoutDiscountFixedRepeatDurationTypedDict,
    )
    from .checkoutdiscountpercentageonceforeverduration import (
        CheckoutDiscountPercentageOnceForeverDuration,
        CheckoutDiscountPercentageOnceForeverDurationTypedDict,
    )
    from .checkoutdiscountpercentagerepeatduration import (
        CheckoutDiscountPercentageRepeatDuration,
        CheckoutDiscountPercentageRepeatDurationTypedDict,
    )
    from .checkoutforbiddenerror import (
        CheckoutForbiddenError,
        CheckoutForbiddenErrorUnion,
    )
    from .checkoutlink import (
        CheckoutLink,
        CheckoutLinkDiscount,
        CheckoutLinkDiscountTypedDict,
        CheckoutLinkTypedDict,
    )
    from .checkoutlinkcreate import CheckoutLinkCreate, CheckoutLinkCreateTypedDict
    from .checkoutlinkcreateproduct import (
        CheckoutLinkCreateProduct,
        CheckoutLinkCreateProductMetadata,
        CheckoutLinkCreateProductMetadataTypedDict,
        CheckoutLinkCreateProductTypedDict,
    )
    from .checkoutlinkcreateproductprice import (
        CheckoutLinkCreateProductPrice,
        CheckoutLinkCreateProductPriceMetadata,
        CheckoutLinkCreateProductPriceMetadataTypedDict,
        CheckoutLinkCreateProductPriceTypedDict,
    )
    from .checkoutlinkcreateproducts import (
        CheckoutLinkCreateProducts,
        CheckoutLinkCreateProductsMetadata,
        CheckoutLinkCreateProductsMetadataTypedDict,
        CheckoutLinkCreateProductsTypedDict,
    )
    from .checkoutlinkproduct import (
        CheckoutLinkProduct,
        CheckoutLinkProductPrices,
        CheckoutLinkProductPricesTypedDict,
        CheckoutLinkProductTypedDict,
    )
    from .checkoutlinksortproperty import CheckoutLinkSortProperty
    from .checkoutlinkupdate import (
        CheckoutLinkUpdate,
        CheckoutLinkUpdateMetadata,
        CheckoutLinkUpdateMetadataTypedDict,
        CheckoutLinkUpdateTypedDict,
    )
    from .checkoutorganization import (
        CheckoutOrganization,
        CheckoutOrganizationTypedDict,
    )
    from .checkoutproduct import (
        CheckoutProduct,
        CheckoutProductPrices,
        CheckoutProductPricesTypedDict,
        CheckoutProductTypedDict,
    )
    from .checkoutpublic import (
        CheckoutPublic,
        CheckoutPublicCustomFieldData,
        CheckoutPublicCustomFieldDataTypedDict,
        CheckoutPublicDiscount,
        CheckoutPublicDiscountTypedDict,
        CheckoutPublicPrices,
        CheckoutPublicPricesTypedDict,
        CheckoutPublicProductPrice,
        CheckoutPublicProductPriceTypedDict,
        CheckoutPublicTypedDict,
    )
    from .checkoutpublicconfirmed import (
        CheckoutPublicConfirmed,
        CheckoutPublicConfirmedCustomFieldData,
        CheckoutPublicConfirmedCustomFieldDataTypedDict,
        CheckoutPublicConfirmedDiscount,
        CheckoutPublicConfirmedDiscountTypedDict,
        CheckoutPublicConfirmedPrices,
        CheckoutPublicConfirmedPricesTypedDict,
        CheckoutPublicConfirmedProductPrice,
        CheckoutPublicConfirmedProductPriceTypedDict,
        CheckoutPublicConfirmedTypedDict,
    )
    from .checkouts_client_confirmop import (
        CheckoutsClientConfirmRequest,
        CheckoutsClientConfirmRequestTypedDict,
    )
    from .checkouts_client_getop import (
        CheckoutsClientGetRequest,
        CheckoutsClientGetRequestTypedDict,
    )
    from .checkouts_client_updateop import (
        CheckoutsClientUpdateRequest,
        CheckoutsClientUpdateRequestTypedDict,
    )
    from .checkouts_getop import CheckoutsGetRequest, CheckoutsGetRequestTypedDict
    from .checkouts_listop import (
        CheckoutsListQueryParamCustomerIDFilter,
        CheckoutsListQueryParamCustomerIDFilterTypedDict,
        CheckoutsListQueryParamOrganizationIDFilter,
        CheckoutsListQueryParamOrganizationIDFilterTypedDict,
        CheckoutsListQueryParamProductIDFilter,
        CheckoutsListQueryParamProductIDFilterTypedDict,
        CheckoutsListRequest,
        CheckoutsListRequestTypedDict,
        CheckoutsListResponse,
        CheckoutsListResponseTypedDict,
        QueryParamStatusFilter,
        QueryParamStatusFilterTypedDict,
    )
    from .checkouts_updateop import (
        CheckoutsUpdateRequest,
        CheckoutsUpdateRequestTypedDict,
    )
    from .checkoutsortproperty import CheckoutSortProperty
    from .checkoutstatus import CheckoutStatus
    from .checkoutupdate import (
        CheckoutUpdate,
        CheckoutUpdateCustomFieldData,
        CheckoutUpdateCustomFieldDataTypedDict,
        CheckoutUpdateCustomerMetadata,
        CheckoutUpdateCustomerMetadataTypedDict,
        CheckoutUpdateMetadata,
        CheckoutUpdateMetadataTypedDict,
        CheckoutUpdateTypedDict,
    )
    from .checkoutupdatepublic import (
        CheckoutUpdatePublic,
        CheckoutUpdatePublicCustomFieldData,
        CheckoutUpdatePublicCustomFieldDataTypedDict,
        CheckoutUpdatePublicTypedDict,
    )
    from .costmetadata_input import (
        Amount,
        AmountTypedDict,
        CostMetadataInput,
        CostMetadataInputTypedDict,
    )
    from .costmetadata_output import CostMetadataOutput, CostMetadataOutputTypedDict
    from .countaggregation import CountAggregation, CountAggregationTypedDict
    from .cursorpagination import CursorPagination, CursorPaginationTypedDict
    from .custom_fields_deleteop import (
        CustomFieldsDeleteRequest,
        CustomFieldsDeleteRequestTypedDict,
    )
    from .custom_fields_getop import (
        CustomFieldsGetRequest,
        CustomFieldsGetRequestTypedDict,
    )
    from .custom_fields_listop import (
        CustomFieldTypeFilter,
        CustomFieldTypeFilterTypedDict,
        CustomFieldsListQueryParamOrganizationIDFilter,
        CustomFieldsListQueryParamOrganizationIDFilterTypedDict,
        CustomFieldsListRequest,
        CustomFieldsListRequestTypedDict,
        CustomFieldsListResponse,
        CustomFieldsListResponseTypedDict,
    )
    from .custom_fields_updateop import (
        CustomFieldsUpdateRequest,
        CustomFieldsUpdateRequestTypedDict,
    )
    from .customer import (
        Customer,
        CustomerTaxID,
        CustomerTaxIDTypedDict,
        CustomerTypedDict,
    )
    from .customer_meters_getop import (
        CustomerMetersGetRequest,
        CustomerMetersGetRequestTypedDict,
    )
    from .customer_meters_listop import (
        CustomerMetersListQueryParamCustomerIDFilter,
        CustomerMetersListQueryParamCustomerIDFilterTypedDict,
        CustomerMetersListQueryParamExternalCustomerIDFilter,
        CustomerMetersListQueryParamExternalCustomerIDFilterTypedDict,
        CustomerMetersListQueryParamOrganizationIDFilter,
        CustomerMetersListQueryParamOrganizationIDFilterTypedDict,
        CustomerMetersListRequest,
        CustomerMetersListRequestTypedDict,
        CustomerMetersListResponse,
        CustomerMetersListResponseTypedDict,
        QueryParamMeterIDFilter,
        QueryParamMeterIDFilterTypedDict,
    )
    from .customer_portal_benefit_grants_getop import (
        CustomerPortalBenefitGrantsGetRequest,
        CustomerPortalBenefitGrantsGetRequestTypedDict,
        CustomerPortalBenefitGrantsGetSecurity,
        CustomerPortalBenefitGrantsGetSecurityTypedDict,
    )
    from .customer_portal_benefit_grants_listop import (
        CustomerPortalBenefitGrantsListQueryParamBenefitIDFilter,
        CustomerPortalBenefitGrantsListQueryParamBenefitIDFilterTypedDict,
        CustomerPortalBenefitGrantsListQueryParamOrderIDFilter,
        CustomerPortalBenefitGrantsListQueryParamOrderIDFilterTypedDict,
        CustomerPortalBenefitGrantsListRequest,
        CustomerPortalBenefitGrantsListRequestTypedDict,
        CustomerPortalBenefitGrantsListResponse,
        CustomerPortalBenefitGrantsListResponseTypedDict,
        CustomerPortalBenefitGrantsListSecurity,
        CustomerPortalBenefitGrantsListSecurityTypedDict,
        QueryParamBenefitTypeFilter,
        QueryParamBenefitTypeFilterTypedDict,
        QueryParamCheckoutIDFilter,
        QueryParamCheckoutIDFilterTypedDict,
        QueryParamMemberIDFilter,
        QueryParamMemberIDFilterTypedDict,
        QueryParamSubscriptionIDFilter,
        QueryParamSubscriptionIDFilterTypedDict,
    )
    from .customer_portal_benefit_grants_updateop import (
        CustomerPortalBenefitGrantsUpdateRequest,
        CustomerPortalBenefitGrantsUpdateRequestTypedDict,
        CustomerPortalBenefitGrantsUpdateSecurity,
        CustomerPortalBenefitGrantsUpdateSecurityTypedDict,
    )
    from .customer_portal_customer_meters_getop import (
        CustomerPortalCustomerMetersGetRequest,
        CustomerPortalCustomerMetersGetRequestTypedDict,
        CustomerPortalCustomerMetersGetSecurity,
        CustomerPortalCustomerMetersGetSecurityTypedDict,
    )
    from .customer_portal_customer_meters_listop import (
        CustomerPortalCustomerMetersListRequest,
        CustomerPortalCustomerMetersListRequestTypedDict,
        CustomerPortalCustomerMetersListResponse,
        CustomerPortalCustomerMetersListResponseTypedDict,
        CustomerPortalCustomerMetersListSecurity,
        CustomerPortalCustomerMetersListSecurityTypedDict,
        MeterIDFilter,
        MeterIDFilterTypedDict,
    )
    from .customer_portal_customer_session_get_authenticated_userop import (
        CustomerPortalCustomerSessionGetAuthenticatedUserSecurity,
        CustomerPortalCustomerSessionGetAuthenticatedUserSecurityTypedDict,
    )
    from .customer_portal_customer_session_introspectop import (
        CustomerPortalCustomerSessionIntrospectSecurity,
        CustomerPortalCustomerSessionIntrospectSecurityTypedDict,
    )
    from .customer_portal_customers_add_payment_methodop import (
        CustomerPortalCustomersAddPaymentMethodSecurity,
        CustomerPortalCustomersAddPaymentMethodSecurityTypedDict,
    )
    from .customer_portal_customers_confirm_payment_methodop import (
        CustomerPortalCustomersConfirmPaymentMethodSecurity,
        CustomerPortalCustomersConfirmPaymentMethodSecurityTypedDict,
    )
    from .customer_portal_customers_delete_payment_methodop import (
        CustomerPortalCustomersDeletePaymentMethodRequest,
        CustomerPortalCustomersDeletePaymentMethodRequestTypedDict,
        CustomerPortalCustomersDeletePaymentMethodSecurity,
        CustomerPortalCustomersDeletePaymentMethodSecurityTypedDict,
    )
    from .customer_portal_customers_getop import (
        CustomerPortalCustomersGetSecurity,
        CustomerPortalCustomersGetSecurityTypedDict,
    )
    from .customer_portal_customers_list_payment_methodsop import (
        CustomerPortalCustomersListPaymentMethodsRequest,
        CustomerPortalCustomersListPaymentMethodsRequestTypedDict,
        CustomerPortalCustomersListPaymentMethodsResponse,
        CustomerPortalCustomersListPaymentMethodsResponseTypedDict,
        CustomerPortalCustomersListPaymentMethodsSecurity,
        CustomerPortalCustomersListPaymentMethodsSecurityTypedDict,
    )
    from .customer_portal_customers_updateop import (
        CustomerPortalCustomersUpdateSecurity,
        CustomerPortalCustomersUpdateSecurityTypedDict,
    )
    from .customer_portal_downloadables_listop import (
        CustomerPortalDownloadablesListQueryParamBenefitIDFilter,
        CustomerPortalDownloadablesListQueryParamBenefitIDFilterTypedDict,
        CustomerPortalDownloadablesListRequest,
        CustomerPortalDownloadablesListRequestTypedDict,
        CustomerPortalDownloadablesListResponse,
        CustomerPortalDownloadablesListResponseTypedDict,
        CustomerPortalDownloadablesListSecurity,
        CustomerPortalDownloadablesListSecurityTypedDict,
    )
    from .customer_portal_license_keys_getop import (
        CustomerPortalLicenseKeysGetRequest,
        CustomerPortalLicenseKeysGetRequestTypedDict,
        CustomerPortalLicenseKeysGetSecurity,
        CustomerPortalLicenseKeysGetSecurityTypedDict,
    )
    from .customer_portal_license_keys_listop import (
        CustomerPortalLicenseKeysListRequest,
        CustomerPortalLicenseKeysListRequestTypedDict,
        CustomerPortalLicenseKeysListResponse,
        CustomerPortalLicenseKeysListResponseTypedDict,
        CustomerPortalLicenseKeysListSecurity,
        CustomerPortalLicenseKeysListSecurityTypedDict,
    )
    from .customer_portal_orders_confirm_retry_paymentop import (
        CustomerPortalOrdersConfirmRetryPaymentRequest,
        CustomerPortalOrdersConfirmRetryPaymentRequestTypedDict,
        CustomerPortalOrdersConfirmRetryPaymentSecurity,
        CustomerPortalOrdersConfirmRetryPaymentSecurityTypedDict,
    )
    from .customer_portal_orders_generate_invoiceop import (
        CustomerPortalOrdersGenerateInvoiceRequest,
        CustomerPortalOrdersGenerateInvoiceRequestTypedDict,
        CustomerPortalOrdersGenerateInvoiceResponse422CustomerPortalOrdersGenerateInvoice,
        CustomerPortalOrdersGenerateInvoiceResponse422CustomerPortalOrdersGenerateInvoiceUnion,
        CustomerPortalOrdersGenerateInvoiceSecurity,
        CustomerPortalOrdersGenerateInvoiceSecurityTypedDict,
    )
    from .customer_portal_orders_get_payment_statusop import (
        CustomerPortalOrdersGetPaymentStatusRequest,
        CustomerPortalOrdersGetPaymentStatusRequestTypedDict,
        CustomerPortalOrdersGetPaymentStatusSecurity,
        CustomerPortalOrdersGetPaymentStatusSecurityTypedDict,
    )
    from .customer_portal_orders_getop import (
        CustomerPortalOrdersGetRequest,
        CustomerPortalOrdersGetRequestTypedDict,
        CustomerPortalOrdersGetSecurity,
        CustomerPortalOrdersGetSecurityTypedDict,
    )
    from .customer_portal_orders_invoiceop import (
        CustomerPortalOrdersInvoiceRequest,
        CustomerPortalOrdersInvoiceRequestTypedDict,
        CustomerPortalOrdersInvoiceSecurity,
        CustomerPortalOrdersInvoiceSecurityTypedDict,
    )
    from .customer_portal_orders_listop import (
        CustomerPortalOrdersListQueryParamProductBillingTypeFilter,
        CustomerPortalOrdersListQueryParamProductBillingTypeFilterTypedDict,
        CustomerPortalOrdersListQueryParamProductIDFilter,
        CustomerPortalOrdersListQueryParamProductIDFilterTypedDict,
        CustomerPortalOrdersListQueryParamSubscriptionIDFilter,
        CustomerPortalOrdersListQueryParamSubscriptionIDFilterTypedDict,
        CustomerPortalOrdersListRequest,
        CustomerPortalOrdersListRequestTypedDict,
        CustomerPortalOrdersListResponse,
        CustomerPortalOrdersListResponseTypedDict,
        CustomerPortalOrdersListSecurity,
        CustomerPortalOrdersListSecurityTypedDict,
    )
    from .customer_portal_orders_updateop import (
        CustomerPortalOrdersUpdateRequest,
        CustomerPortalOrdersUpdateRequestTypedDict,
        CustomerPortalOrdersUpdateSecurity,
        CustomerPortalOrdersUpdateSecurityTypedDict,
    )
    from .customer_portal_organizations_getop import (
        CustomerPortalOrganizationsGetRequest,
        CustomerPortalOrganizationsGetRequestTypedDict,
    )
    from .customer_portal_seats_assign_seatop import (
        CustomerPortalSeatsAssignSeatSecurity,
        CustomerPortalSeatsAssignSeatSecurityTypedDict,
    )
    from .customer_portal_seats_list_claimed_subscriptionsop import (
        CustomerPortalSeatsListClaimedSubscriptionsSecurity,
        CustomerPortalSeatsListClaimedSubscriptionsSecurityTypedDict,
    )
    from .customer_portal_seats_list_seatsop import (
        CustomerPortalSeatsListSeatsRequest,
        CustomerPortalSeatsListSeatsRequestTypedDict,
        CustomerPortalSeatsListSeatsSecurity,
        CustomerPortalSeatsListSeatsSecurityTypedDict,
    )
    from .customer_portal_seats_resend_invitationop import (
        CustomerPortalSeatsResendInvitationRequest,
        CustomerPortalSeatsResendInvitationRequestTypedDict,
        CustomerPortalSeatsResendInvitationSecurity,
        CustomerPortalSeatsResendInvitationSecurityTypedDict,
    )
    from .customer_portal_seats_revoke_seatop import (
        CustomerPortalSeatsRevokeSeatRequest,
        CustomerPortalSeatsRevokeSeatRequestTypedDict,
        CustomerPortalSeatsRevokeSeatSecurity,
        CustomerPortalSeatsRevokeSeatSecurityTypedDict,
    )
    from .customer_portal_subscriptions_cancelop import (
        CustomerPortalSubscriptionsCancelRequest,
        CustomerPortalSubscriptionsCancelRequestTypedDict,
        CustomerPortalSubscriptionsCancelSecurity,
        CustomerPortalSubscriptionsCancelSecurityTypedDict,
    )
    from .customer_portal_subscriptions_getop import (
        CustomerPortalSubscriptionsGetRequest,
        CustomerPortalSubscriptionsGetRequestTypedDict,
        CustomerPortalSubscriptionsGetSecurity,
        CustomerPortalSubscriptionsGetSecurityTypedDict,
    )
    from .customer_portal_subscriptions_listop import (
        CustomerPortalSubscriptionsListQueryParamProductIDFilter,
        CustomerPortalSubscriptionsListQueryParamProductIDFilterTypedDict,
        CustomerPortalSubscriptionsListRequest,
        CustomerPortalSubscriptionsListRequestTypedDict,
        CustomerPortalSubscriptionsListResponse,
        CustomerPortalSubscriptionsListResponseTypedDict,
        CustomerPortalSubscriptionsListSecurity,
        CustomerPortalSubscriptionsListSecurityTypedDict,
    )
    from .customer_portal_subscriptions_updateop import (
        CustomerPortalSubscriptionsUpdateRequest,
        CustomerPortalSubscriptionsUpdateRequestTypedDict,
        CustomerPortalSubscriptionsUpdateSecurity,
        CustomerPortalSubscriptionsUpdateSecurityTypedDict,
    )
    from .customer_portal_wallets_getop import (
        CustomerPortalWalletsGetRequest,
        CustomerPortalWalletsGetRequestTypedDict,
        CustomerPortalWalletsGetSecurity,
        CustomerPortalWalletsGetSecurityTypedDict,
    )
    from .customer_portal_wallets_listop import (
        CustomerPortalWalletsListRequest,
        CustomerPortalWalletsListRequestTypedDict,
        CustomerPortalWalletsListResponse,
        CustomerPortalWalletsListResponseTypedDict,
        CustomerPortalWalletsListSecurity,
        CustomerPortalWalletsListSecurityTypedDict,
    )
    from .customer_seats_get_claim_infoop import (
        CustomerSeatsGetClaimInfoRequest,
        CustomerSeatsGetClaimInfoRequestTypedDict,
    )
    from .customer_seats_list_seatsop import (
        CustomerSeatsListSeatsRequest,
        CustomerSeatsListSeatsRequestTypedDict,
    )
    from .customer_seats_resend_invitationop import (
        CustomerSeatsResendInvitationRequest,
        CustomerSeatsResendInvitationRequestTypedDict,
    )
    from .customer_seats_revoke_seatop import (
        CustomerSeatsRevokeSeatRequest,
        CustomerSeatsRevokeSeatRequestTypedDict,
    )
    from .customer_sessions_createop import (
        CustomerSessionsCreateCustomerSessionCreate,
        CustomerSessionsCreateCustomerSessionCreateTypedDict,
    )
    from .customerbenefitgrant import (
        CustomerBenefitGrant,
        CustomerBenefitGrantTypedDict,
    )
    from .customerbenefitgrantcustom import (
        CustomerBenefitGrantCustom,
        CustomerBenefitGrantCustomTypedDict,
    )
    from .customerbenefitgrantcustomupdate import (
        CustomerBenefitGrantCustomUpdate,
        CustomerBenefitGrantCustomUpdateTypedDict,
    )
    from .customerbenefitgrantdiscord import (
        CustomerBenefitGrantDiscord,
        CustomerBenefitGrantDiscordTypedDict,
    )
    from .customerbenefitgrantdiscordpropertiesupdate import (
        CustomerBenefitGrantDiscordPropertiesUpdate,
        CustomerBenefitGrantDiscordPropertiesUpdateTypedDict,
    )
    from .customerbenefitgrantdiscordupdate import (
        CustomerBenefitGrantDiscordUpdate,
        CustomerBenefitGrantDiscordUpdateTypedDict,
    )
    from .customerbenefitgrantdownloadables import (
        CustomerBenefitGrantDownloadables,
        CustomerBenefitGrantDownloadablesTypedDict,
    )
    from .customerbenefitgrantdownloadablesupdate import (
        CustomerBenefitGrantDownloadablesUpdate,
        CustomerBenefitGrantDownloadablesUpdateTypedDict,
    )
    from .customerbenefitgrantgithubrepository import (
        CustomerBenefitGrantGitHubRepository,
        CustomerBenefitGrantGitHubRepositoryTypedDict,
    )
    from .customerbenefitgrantgithubrepositorypropertiesupdate import (
        CustomerBenefitGrantGitHubRepositoryPropertiesUpdate,
        CustomerBenefitGrantGitHubRepositoryPropertiesUpdateTypedDict,
    )
    from .customerbenefitgrantgithubrepositoryupdate import (
        CustomerBenefitGrantGitHubRepositoryUpdate,
        CustomerBenefitGrantGitHubRepositoryUpdateTypedDict,
    )
    from .customerbenefitgrantlicensekeys import (
        CustomerBenefitGrantLicenseKeys,
        CustomerBenefitGrantLicenseKeysTypedDict,
    )
    from .customerbenefitgrantlicensekeysupdate import (
        CustomerBenefitGrantLicenseKeysUpdate,
        CustomerBenefitGrantLicenseKeysUpdateTypedDict,
    )
    from .customerbenefitgrantmetercredit import (
        CustomerBenefitGrantMeterCredit,
        CustomerBenefitGrantMeterCreditTypedDict,
    )
    from .customerbenefitgrantmetercreditupdate import (
        CustomerBenefitGrantMeterCreditUpdate,
        CustomerBenefitGrantMeterCreditUpdateTypedDict,
    )
    from .customerbenefitgrantsortproperty import CustomerBenefitGrantSortProperty
    from .customerbenefitgrantupdate import (
        CustomerBenefitGrantUpdate,
        CustomerBenefitGrantUpdateTypedDict,
    )
    from .customercancellationreason import CustomerCancellationReason
    from .customercreate import (
        CustomerCreate,
        CustomerCreateMetadata,
        CustomerCreateMetadataTypedDict,
        CustomerCreateTaxID,
        CustomerCreateTaxIDTypedDict,
        CustomerCreateTypedDict,
    )
    from .customercreatedevent import (
        CustomerCreatedEvent,
        CustomerCreatedEventTypedDict,
    )
    from .customercreatedmetadata import (
        CustomerCreatedMetadata,
        CustomerCreatedMetadataTypedDict,
    )
    from .customercustomermeter import (
        CustomerCustomerMeter,
        CustomerCustomerMeterTypedDict,
    )
    from .customercustomermetermeter import (
        CustomerCustomerMeterMeter,
        CustomerCustomerMeterMeterTypedDict,
    )
    from .customercustomermetersortproperty import CustomerCustomerMeterSortProperty
    from .customercustomersession import (
        CustomerCustomerSession,
        CustomerCustomerSessionTypedDict,
    )
    from .customerdeletedevent import (
        CustomerDeletedEvent,
        CustomerDeletedEventTypedDict,
    )
    from .customerdeletedmetadata import (
        CustomerDeletedMetadata,
        CustomerDeletedMetadataTypedDict,
    )
    from .customermeter import CustomerMeter, CustomerMeterTypedDict
    from .customermetersortproperty import CustomerMeterSortProperty
    from .customernotready import CustomerNotReady, CustomerNotReadyData
    from .customerorder import CustomerOrder, CustomerOrderTypedDict
    from .customerorderconfirmpayment import (
        CustomerOrderConfirmPayment,
        CustomerOrderConfirmPaymentTypedDict,
    )
    from .customerorderinvoice import (
        CustomerOrderInvoice,
        CustomerOrderInvoiceTypedDict,
    )
    from .customerorderpaymentconfirmation import (
        CustomerOrderPaymentConfirmation,
        CustomerOrderPaymentConfirmationTypedDict,
    )
    from .customerorderpaymentstatus import (
        CustomerOrderPaymentStatus,
        CustomerOrderPaymentStatusTypedDict,
    )
    from .customerorderproduct import (
        CustomerOrderProduct,
        CustomerOrderProductPrices,
        CustomerOrderProductPricesTypedDict,
        CustomerOrderProductTypedDict,
    )
    from .customerordersortproperty import CustomerOrderSortProperty
    from .customerordersubscription import (
        CustomerOrderSubscription,
        CustomerOrderSubscriptionTypedDict,
    )
    from .customerorderupdate import CustomerOrderUpdate, CustomerOrderUpdateTypedDict
    from .customerorganization import (
        CustomerOrganization,
        CustomerOrganizationTypedDict,
    )
    from .customerorganizationdata import (
        CustomerOrganizationData,
        CustomerOrganizationDataTypedDict,
    )
    from .customerpaymentmethod import (
        CustomerPaymentMethod,
        CustomerPaymentMethodTypedDict,
    )
    from .customerpaymentmethodconfirm import (
        CustomerPaymentMethodConfirm,
        CustomerPaymentMethodConfirmTypedDict,
    )
    from .customerpaymentmethodcreate import (
        CustomerPaymentMethodCreate,
        CustomerPaymentMethodCreateTypedDict,
    )
    from .customerpaymentmethodcreaterequiresactionresponse import (
        CustomerPaymentMethodCreateRequiresActionResponse,
        CustomerPaymentMethodCreateRequiresActionResponseTypedDict,
    )
    from .customerpaymentmethodcreateresponse import (
        CustomerPaymentMethodCreateResponse,
        CustomerPaymentMethodCreateResponseTypedDict,
    )
    from .customerpaymentmethodcreatesucceededresponse import (
        CustomerPaymentMethodCreateSucceededResponse,
        CustomerPaymentMethodCreateSucceededResponseTypedDict,
    )
    from .customerportalcustomer import (
        CustomerPortalCustomer,
        CustomerPortalCustomerTaxID,
        CustomerPortalCustomerTaxIDTypedDict,
        CustomerPortalCustomerTypedDict,
    )
    from .customerportalcustomerupdate import (
        CustomerPortalCustomerUpdate,
        CustomerPortalCustomerUpdateTypedDict,
    )
    from .customerportaloauthaccount import (
        CustomerPortalOAuthAccount,
        CustomerPortalOAuthAccountTypedDict,
    )
    from .customerportalsubscriptionsettings import (
        CustomerPortalSubscriptionSettings,
        CustomerPortalSubscriptionSettingsTypedDict,
    )
    from .customerportalusagesettings import (
        CustomerPortalUsageSettings,
        CustomerPortalUsageSettingsTypedDict,
    )
    from .customerproduct import (
        CustomerProduct,
        CustomerProductPrices,
        CustomerProductPricesTypedDict,
        CustomerProductTypedDict,
    )
    from .customers_delete_externalop import (
        CustomersDeleteExternalRequest,
        CustomersDeleteExternalRequestTypedDict,
    )
    from .customers_deleteop import (
        CustomersDeleteRequest,
        CustomersDeleteRequestTypedDict,
    )
    from .customers_exportop import (
        CustomersExportQueryParamOrganizationID,
        CustomersExportQueryParamOrganizationIDTypedDict,
        CustomersExportRequest,
        CustomersExportRequestTypedDict,
    )
    from .customers_get_externalop import (
        CustomersGetExternalRequest,
        CustomersGetExternalRequestTypedDict,
    )
    from .customers_get_state_externalop import (
        CustomersGetStateExternalRequest,
        CustomersGetStateExternalRequestTypedDict,
    )
    from .customers_get_stateop import (
        CustomersGetStateRequest,
        CustomersGetStateRequestTypedDict,
    )
    from .customers_getop import CustomersGetRequest, CustomersGetRequestTypedDict
    from .customers_update_externalop import (
        CustomersUpdateExternalRequest,
        CustomersUpdateExternalRequestTypedDict,
    )
    from .customers_updateop import (
        CustomersUpdateRequest,
        CustomersUpdateRequestTypedDict,
    )
    from .customerseat import CustomerSeat, CustomerSeatTypedDict
    from .customerseatclaimresponse import (
        CustomerSeatClaimResponse,
        CustomerSeatClaimResponseTypedDict,
    )
    from .customersession import CustomerSession, CustomerSessionTypedDict
    from .customersessioncustomerexternalidcreate import (
        CustomerSessionCustomerExternalIDCreate,
        CustomerSessionCustomerExternalIDCreateTypedDict,
    )
    from .customersessioncustomeridcreate import (
        CustomerSessionCustomerIDCreate,
        CustomerSessionCustomerIDCreateTypedDict,
    )
    from .customersortproperty import CustomerSortProperty
    from .customerstate import (
        CustomerState,
        CustomerStateTaxID,
        CustomerStateTaxIDTypedDict,
        CustomerStateTypedDict,
    )
    from .customerstatebenefitgrant import (
        CustomerStateBenefitGrant,
        CustomerStateBenefitGrantProperties,
        CustomerStateBenefitGrantPropertiesTypedDict,
        CustomerStateBenefitGrantTypedDict,
    )
    from .customerstatemeter import CustomerStateMeter, CustomerStateMeterTypedDict
    from .customerstatesubscription import (
        CustomerStateSubscription,
        CustomerStateSubscriptionCustomFieldData,
        CustomerStateSubscriptionCustomFieldDataTypedDict,
        CustomerStateSubscriptionTypedDict,
        Status,
    )
    from .customerstatesubscriptionmeter import (
        CustomerStateSubscriptionMeter,
        CustomerStateSubscriptionMeterTypedDict,
    )
    from .customersubscription import (
        CustomerSubscription,
        CustomerSubscriptionPrices,
        CustomerSubscriptionPricesTypedDict,
        CustomerSubscriptionTypedDict,
    )
    from .customersubscriptioncancel import (
        CustomerSubscriptionCancel,
        CustomerSubscriptionCancelTypedDict,
    )
    from .customersubscriptionmeter import (
        CustomerSubscriptionMeter,
        CustomerSubscriptionMeterTypedDict,
    )
    from .customersubscriptionmetermeter import (
        CustomerSubscriptionMeterMeter,
        CustomerSubscriptionMeterMeterTypedDict,
    )
    from .customersubscriptionproduct import (
        CustomerSubscriptionProduct,
        CustomerSubscriptionProductPrices,
        CustomerSubscriptionProductPricesTypedDict,
        CustomerSubscriptionProductTypedDict,
    )
    from .customersubscriptionsortproperty import CustomerSubscriptionSortProperty
    from .customersubscriptionupdate import (
        CustomerSubscriptionUpdate,
        CustomerSubscriptionUpdateTypedDict,
    )
    from .customersubscriptionupdateproduct import (
        CustomerSubscriptionUpdateProduct,
        CustomerSubscriptionUpdateProductTypedDict,
    )
    from .customersubscriptionupdateseats import (
        CustomerSubscriptionUpdateSeats,
        CustomerSubscriptionUpdateSeatsTypedDict,
    )
    from .customerupdate import (
        CustomerUpdate,
        CustomerUpdateMetadata,
        CustomerUpdateMetadataTypedDict,
        CustomerUpdateTaxID,
        CustomerUpdateTaxIDTypedDict,
        CustomerUpdateTypedDict,
    )
    from .customerupdatedevent import (
        CustomerUpdatedEvent,
        CustomerUpdatedEventTypedDict,
    )
    from .customerupdatedfields import (
        CustomerUpdatedFields,
        CustomerUpdatedFieldsMetadata,
        CustomerUpdatedFieldsMetadataTypedDict,
        CustomerUpdatedFieldsTypedDict,
    )
    from .customerupdatedmetadata import (
        CustomerUpdatedMetadata,
        CustomerUpdatedMetadataTypedDict,
    )
    from .customerupdateexternalid import (
        CustomerUpdateExternalID,
        CustomerUpdateExternalIDMetadata,
        CustomerUpdateExternalIDMetadataTypedDict,
        CustomerUpdateExternalIDTaxID,
        CustomerUpdateExternalIDTaxIDTypedDict,
        CustomerUpdateExternalIDTypedDict,
    )
    from .customerwallet import CustomerWallet, CustomerWalletTypedDict
    from .customerwalletsortproperty import CustomerWalletSortProperty
    from .customerwithmembers import (
        CustomerWithMembers,
        CustomerWithMembersTaxID,
        CustomerWithMembersTaxIDTypedDict,
        CustomerWithMembersTypedDict,
    )
    from .customfield import CustomField, CustomFieldTypedDict
    from .customfieldcheckbox import CustomFieldCheckbox, CustomFieldCheckboxTypedDict
    from .customfieldcheckboxproperties import (
        CustomFieldCheckboxProperties,
        CustomFieldCheckboxPropertiesTypedDict,
    )
    from .customfieldcreate import CustomFieldCreate, CustomFieldCreateTypedDict
    from .customfieldcreatecheckbox import (
        CustomFieldCreateCheckbox,
        CustomFieldCreateCheckboxMetadata,
        CustomFieldCreateCheckboxMetadataTypedDict,
        CustomFieldCreateCheckboxTypedDict,
    )
    from .customfieldcreatedate import (
        CustomFieldCreateDate,
        CustomFieldCreateDateMetadata,
        CustomFieldCreateDateMetadataTypedDict,
        CustomFieldCreateDateTypedDict,
    )
    from .customfieldcreatenumber import (
        CustomFieldCreateNumber,
        CustomFieldCreateNumberMetadata,
        CustomFieldCreateNumberMetadataTypedDict,
        CustomFieldCreateNumberTypedDict,
    )
    from .customfieldcreateselect import (
        CustomFieldCreateSelect,
        CustomFieldCreateSelectMetadata,
        CustomFieldCreateSelectMetadataTypedDict,
        CustomFieldCreateSelectTypedDict,
    )
    from .customfieldcreatetext import (
        CustomFieldCreateText,
        CustomFieldCreateTextMetadata,
        CustomFieldCreateTextMetadataTypedDict,
        CustomFieldCreateTextTypedDict,
    )
    from .customfielddate import CustomFieldDate, CustomFieldDateTypedDict
    from .customfielddateproperties import (
        CustomFieldDateProperties,
        CustomFieldDatePropertiesTypedDict,
    )
    from .customfieldnumber import CustomFieldNumber, CustomFieldNumberTypedDict
    from .customfieldnumberproperties import (
        CustomFieldNumberProperties,
        CustomFieldNumberPropertiesTypedDict,
    )
    from .customfieldselect import CustomFieldSelect, CustomFieldSelectTypedDict
    from .customfieldselectoption import (
        CustomFieldSelectOption,
        CustomFieldSelectOptionTypedDict,
    )
    from .customfieldselectproperties import (
        CustomFieldSelectProperties,
        CustomFieldSelectPropertiesTypedDict,
    )
    from .customfieldsortproperty import CustomFieldSortProperty
    from .customfieldtext import CustomFieldText, CustomFieldTextTypedDict
    from .customfieldtextproperties import (
        CustomFieldTextProperties,
        CustomFieldTextPropertiesTypedDict,
    )
    from .customfieldtype import CustomFieldType
    from .customfieldupdate import CustomFieldUpdate, CustomFieldUpdateTypedDict
    from .customfieldupdatecheckbox import (
        CustomFieldUpdateCheckbox,
        CustomFieldUpdateCheckboxMetadata,
        CustomFieldUpdateCheckboxMetadataTypedDict,
        CustomFieldUpdateCheckboxTypedDict,
    )
    from .customfieldupdatedate import (
        CustomFieldUpdateDate,
        CustomFieldUpdateDateMetadata,
        CustomFieldUpdateDateMetadataTypedDict,
        CustomFieldUpdateDateTypedDict,
    )
    from .customfieldupdatenumber import (
        CustomFieldUpdateNumber,
        CustomFieldUpdateNumberMetadata,
        CustomFieldUpdateNumberMetadataTypedDict,
        CustomFieldUpdateNumberTypedDict,
    )
    from .customfieldupdateselect import (
        CustomFieldUpdateSelect,
        CustomFieldUpdateSelectMetadata,
        CustomFieldUpdateSelectMetadataTypedDict,
        CustomFieldUpdateSelectTypedDict,
    )
    from .customfieldupdatetext import (
        CustomFieldUpdateText,
        CustomFieldUpdateTextMetadata,
        CustomFieldUpdateTextMetadataTypedDict,
        CustomFieldUpdateTextTypedDict,
    )
    from .discount import Discount, DiscountTypedDict
    from .discountcreate import DiscountCreate, DiscountCreateTypedDict
    from .discountduration import DiscountDuration
    from .discountfixedonceforeverduration import (
        DiscountFixedOnceForeverDuration,
        DiscountFixedOnceForeverDurationTypedDict,
    )
    from .discountfixedonceforeverdurationbase import (
        DiscountFixedOnceForeverDurationBase,
        DiscountFixedOnceForeverDurationBaseTypedDict,
    )
    from .discountfixedonceforeverdurationcreate import (
        DiscountFixedOnceForeverDurationCreate,
        DiscountFixedOnceForeverDurationCreateMetadata,
        DiscountFixedOnceForeverDurationCreateMetadataTypedDict,
        DiscountFixedOnceForeverDurationCreateTypedDict,
    )
    from .discountfixedrepeatduration import (
        DiscountFixedRepeatDuration,
        DiscountFixedRepeatDurationTypedDict,
    )
    from .discountfixedrepeatdurationbase import (
        DiscountFixedRepeatDurationBase,
        DiscountFixedRepeatDurationBaseTypedDict,
    )
    from .discountfixedrepeatdurationcreate import (
        DiscountFixedRepeatDurationCreate,
        DiscountFixedRepeatDurationCreateMetadata,
        DiscountFixedRepeatDurationCreateMetadataTypedDict,
        DiscountFixedRepeatDurationCreateTypedDict,
    )
    from .discountpercentageonceforeverduration import (
        DiscountPercentageOnceForeverDuration,
        DiscountPercentageOnceForeverDurationTypedDict,
    )
    from .discountpercentageonceforeverdurationbase import (
        DiscountPercentageOnceForeverDurationBase,
        DiscountPercentageOnceForeverDurationBaseTypedDict,
    )
    from .discountpercentageonceforeverdurationcreate import (
        DiscountPercentageOnceForeverDurationCreate,
        DiscountPercentageOnceForeverDurationCreateMetadata,
        DiscountPercentageOnceForeverDurationCreateMetadataTypedDict,
        DiscountPercentageOnceForeverDurationCreateTypedDict,
    )
    from .discountpercentagerepeatduration import (
        DiscountPercentageRepeatDuration,
        DiscountPercentageRepeatDurationTypedDict,
    )
    from .discountpercentagerepeatdurationbase import (
        DiscountPercentageRepeatDurationBase,
        DiscountPercentageRepeatDurationBaseTypedDict,
    )
    from .discountpercentagerepeatdurationcreate import (
        DiscountPercentageRepeatDurationCreate,
        DiscountPercentageRepeatDurationCreateMetadata,
        DiscountPercentageRepeatDurationCreateMetadataTypedDict,
        DiscountPercentageRepeatDurationCreateTypedDict,
    )
    from .discountproduct import DiscountProduct, DiscountProductTypedDict
    from .discounts_deleteop import (
        DiscountsDeleteRequest,
        DiscountsDeleteRequestTypedDict,
    )
    from .discounts_getop import DiscountsGetRequest, DiscountsGetRequestTypedDict
    from .discounts_listop import (
        DiscountsListQueryParamOrganizationIDFilter,
        DiscountsListQueryParamOrganizationIDFilterTypedDict,
        DiscountsListRequest,
        DiscountsListRequestTypedDict,
        DiscountsListResponse,
        DiscountsListResponseTypedDict,
    )
    from .discounts_updateop import (
        DiscountsUpdateRequest,
        DiscountsUpdateRequestTypedDict,
    )
    from .discountsortproperty import DiscountSortProperty
    from .discounttype import DiscountType
    from .discountupdate import (
        DiscountUpdate,
        DiscountUpdateMetadata,
        DiscountUpdateMetadataTypedDict,
        DiscountUpdateTypedDict,
    )
    from .dispute import Dispute, DisputeTypedDict
    from .disputes_getop import DisputesGetRequest, DisputesGetRequestTypedDict
    from .disputes_listop import (
        DisputesListQueryParamOrganizationIDFilter,
        DisputesListQueryParamOrganizationIDFilterTypedDict,
        DisputesListRequest,
        DisputesListRequestTypedDict,
        DisputesListResponse,
        DisputesListResponseTypedDict,
        QueryParamOrderIDFilter,
        QueryParamOrderIDFilterTypedDict,
        StatusFilter,
        StatusFilterTypedDict,
    )
    from .disputesortproperty import DisputeSortProperty
    from .disputestatus import DisputeStatus
    from .downloadablefilecreate import (
        DownloadableFileCreate,
        DownloadableFileCreateTypedDict,
    )
    from .downloadablefileread import (
        DownloadableFileRead,
        DownloadableFileReadTypedDict,
    )
    from .downloadableread import DownloadableRead, DownloadableReadTypedDict
    from .event import Event, EventTypedDict
    from .event_types_listop import (
        EventTypesListQueryParamCustomerIDFilter,
        EventTypesListQueryParamCustomerIDFilterTypedDict,
        EventTypesListQueryParamExternalCustomerIDFilter,
        EventTypesListQueryParamExternalCustomerIDFilterTypedDict,
        EventTypesListQueryParamOrganizationIDFilter,
        EventTypesListQueryParamOrganizationIDFilterTypedDict,
        EventTypesListRequest,
        EventTypesListRequestTypedDict,
        EventTypesListResponse,
        EventTypesListResponseTypedDict,
    )
    from .event_types_updateop import (
        EventTypesUpdateRequest,
        EventTypesUpdateRequestTypedDict,
    )
    from .eventcreatecustomer import EventCreateCustomer, EventCreateCustomerTypedDict
    from .eventcreateexternalcustomer import (
        EventCreateExternalCustomer,
        EventCreateExternalCustomerTypedDict,
    )
    from .eventmetadatainput import EventMetadataInput, EventMetadataInputTypedDict
    from .eventmetadataoutput import EventMetadataOutput, EventMetadataOutputTypedDict
    from .eventname import EventName, EventNameTypedDict
    from .eventnamessortproperty import EventNamesSortProperty
    from .events_getop import EventsGetRequest, EventsGetRequestTypedDict
    from .events_list_namesop import (
        EventsListNamesQueryParamCustomerIDFilter,
        EventsListNamesQueryParamCustomerIDFilterTypedDict,
        EventsListNamesQueryParamExternalCustomerIDFilter,
        EventsListNamesQueryParamExternalCustomerIDFilterTypedDict,
        EventsListNamesQueryParamOrganizationIDFilter,
        EventsListNamesQueryParamOrganizationIDFilterTypedDict,
        EventsListNamesRequest,
        EventsListNamesRequestTypedDict,
        EventsListNamesResponse,
        EventsListNamesResponseTypedDict,
        QueryParamSourceFilter,
        QueryParamSourceFilterTypedDict,
    )
    from .eventsingest import (
        EventsIngest,
        EventsIngestTypedDict,
        EventsModel,
        EventsModelTypedDict,
    )
    from .eventsingestresponse import (
        EventsIngestResponse,
        EventsIngestResponseTypedDict,
    )
    from .eventsortproperty import EventSortProperty
    from .eventsource import EventSource
    from .eventtype import EventType, EventTypeTypedDict
    from .eventtypessortproperty import EventTypesSortProperty
    from .eventtypeupdate import EventTypeUpdate, EventTypeUpdateTypedDict
    from .eventtypewithstats import EventTypeWithStats, EventTypeWithStatsTypedDict
    from .existingproductprice import (
        ExistingProductPrice,
        ExistingProductPriceTypedDict,
    )
    from .expiredcheckouterror import ExpiredCheckoutError, ExpiredCheckoutErrorData
    from .filecreate import FileCreate, FileCreateTypedDict
    from .filedownload import FileDownload, FileDownloadTypedDict
    from .filepatch import FilePatch, FilePatchTypedDict
    from .files_deleteop import FilesDeleteRequest, FilesDeleteRequestTypedDict
    from .files_listop import (
        FileIDFilter,
        FileIDFilterTypedDict,
        FilesListQueryParamOrganizationIDFilter,
        FilesListQueryParamOrganizationIDFilterTypedDict,
        FilesListRequest,
        FilesListRequestTypedDict,
        FilesListResponse,
        FilesListResponseTypedDict,
    )
    from .files_updateop import (
        FilesUpdateRequest,
        FilesUpdateRequestTypedDict,
        FilesUpdateResponseFilesUpdate,
        FilesUpdateResponseFilesUpdateTypedDict,
    )
    from .files_uploadedop import (
        FilesUploadedRequest,
        FilesUploadedRequestTypedDict,
        FilesUploadedResponseFilesUploaded,
        FilesUploadedResponseFilesUploadedTypedDict,
    )
    from .fileservicetypes import FileServiceTypes
    from .fileupload import FileUpload, FileUploadTypedDict
    from .fileuploadcompleted import FileUploadCompleted, FileUploadCompletedTypedDict
    from .filterclause import FilterClause, FilterClauseTypedDict, Value, ValueTypedDict
    from .filterconjunction import FilterConjunction
    from .filteroperator import FilterOperator
    from .genericpayment import GenericPayment, GenericPaymentTypedDict
    from .httpvalidationerror import HTTPValidationError, HTTPValidationErrorData
    from .introspecttokenrequest import (
        IntrospectTokenRequest,
        IntrospectTokenRequestTokenTypeHint,
        IntrospectTokenRequestTypedDict,
    )
    from .introspecttokenresponse import (
        IntrospectTokenResponse,
        IntrospectTokenResponseTypedDict,
        TokenType,
    )
    from .legacyrecurringproductprice import (
        LegacyRecurringProductPrice,
        LegacyRecurringProductPriceTypedDict,
    )
    from .legacyrecurringproductpricecustom import (
        LegacyRecurringProductPriceCustom,
        LegacyRecurringProductPriceCustomTypedDict,
    )
    from .legacyrecurringproductpricefixed import (
        LegacyRecurringProductPriceFixed,
        LegacyRecurringProductPriceFixedTypedDict,
    )
    from .legacyrecurringproductpricefree import (
        LegacyRecurringProductPriceFree,
        LegacyRecurringProductPriceFreeTypedDict,
    )
    from .license_keys_get_activationop import (
        LicenseKeysGetActivationRequest,
        LicenseKeysGetActivationRequestTypedDict,
    )
    from .license_keys_getop import (
        LicenseKeysGetRequest,
        LicenseKeysGetRequestTypedDict,
    )
    from .license_keys_listop import (
        LicenseKeysListQueryParamOrganizationIDFilter,
        LicenseKeysListQueryParamOrganizationIDFilterTypedDict,
        LicenseKeysListRequest,
        LicenseKeysListRequestTypedDict,
        LicenseKeysListResponse,
        LicenseKeysListResponseTypedDict,
        QueryParamBenefitIDFilter,
        QueryParamBenefitIDFilterTypedDict,
    )
    from .license_keys_updateop import (
        LicenseKeysUpdateRequest,
        LicenseKeysUpdateRequestTypedDict,
    )
    from .licensekeyactivate import (
        LicenseKeyActivate,
        LicenseKeyActivateConditions,
        LicenseKeyActivateConditionsTypedDict,
        LicenseKeyActivateMeta,
        LicenseKeyActivateMetaTypedDict,
        LicenseKeyActivateTypedDict,
    )
    from .licensekeyactivationbase import (
        LicenseKeyActivationBase,
        LicenseKeyActivationBaseTypedDict,
        Meta,
        MetaTypedDict,
    )
    from .licensekeyactivationread import (
        LicenseKeyActivationRead,
        LicenseKeyActivationReadMeta,
        LicenseKeyActivationReadMetaTypedDict,
        LicenseKeyActivationReadTypedDict,
    )
    from .licensekeycustomer import (
        LicenseKeyCustomer,
        LicenseKeyCustomerTaxID,
        LicenseKeyCustomerTaxIDTypedDict,
        LicenseKeyCustomerTypedDict,
    )
    from .licensekeydeactivate import (
        LicenseKeyDeactivate,
        LicenseKeyDeactivateTypedDict,
    )
    from .licensekeyread import LicenseKeyRead, LicenseKeyReadTypedDict
    from .licensekeystatus import LicenseKeyStatus
    from .licensekeyupdate import LicenseKeyUpdate, LicenseKeyUpdateTypedDict
    from .licensekeyvalidate import (
        Conditions,
        ConditionsTypedDict,
        LicenseKeyValidate,
        LicenseKeyValidateTypedDict,
    )
    from .licensekeywithactivations import (
        LicenseKeyWithActivations,
        LicenseKeyWithActivationsTypedDict,
    )
    from .listresource_ import ListResource, ListResourceTypedDict
    from .listresource_benefit_ import ListResourceBenefit, ListResourceBenefitTypedDict
    from .listresource_benefitgrant_ import (
        ListResourceBenefitGrant,
        ListResourceBenefitGrantTypedDict,
    )
    from .listresource_checkout_ import (
        ListResourceCheckout,
        ListResourceCheckoutTypedDict,
    )
    from .listresource_checkoutlink_ import (
        ListResourceCheckoutLink,
        ListResourceCheckoutLinkTypedDict,
    )
    from .listresource_customerbenefitgrant_ import (
        ListResourceCustomerBenefitGrant,
        ListResourceCustomerBenefitGrantTypedDict,
    )
    from .listresource_customercustomermeter_ import (
        ListResourceCustomerCustomerMeter,
        ListResourceCustomerCustomerMeterTypedDict,
    )
    from .listresource_customermeter_ import (
        ListResourceCustomerMeter,
        ListResourceCustomerMeterTypedDict,
    )
    from .listresource_customerorder_ import (
        ListResourceCustomerOrder,
        ListResourceCustomerOrderTypedDict,
    )
    from .listresource_customerpaymentmethod_ import (
        ListResourceCustomerPaymentMethod,
        ListResourceCustomerPaymentMethodTypedDict,
    )
    from .listresource_customersubscription_ import (
        ListResourceCustomerSubscription,
        ListResourceCustomerSubscriptionTypedDict,
    )
    from .listresource_customerwallet_ import (
        ListResourceCustomerWallet,
        ListResourceCustomerWalletTypedDict,
    )
    from .listresource_customerwithmembers_ import (
        ListResourceCustomerWithMembers,
        ListResourceCustomerWithMembersTypedDict,
    )
    from .listresource_customfield_ import (
        ListResourceCustomField,
        ListResourceCustomFieldTypedDict,
    )
    from .listresource_discount_ import (
        ListResourceDiscount,
        ListResourceDiscountTypedDict,
    )
    from .listresource_dispute_ import ListResourceDispute, ListResourceDisputeTypedDict
    from .listresource_downloadableread_ import (
        ListResourceDownloadableRead,
        ListResourceDownloadableReadTypedDict,
    )
    from .listresource_event_ import ListResourceEvent, ListResourceEventTypedDict
    from .listresource_eventname_ import (
        ListResourceEventName,
        ListResourceEventNameTypedDict,
    )
    from .listresource_eventtypewithstats_ import (
        ListResourceEventTypeWithStats,
        ListResourceEventTypeWithStatsTypedDict,
    )
    from .listresource_fileread_ import (
        FileRead,
        FileReadTypedDict,
        ListResourceFileRead,
        ListResourceFileReadTypedDict,
    )
    from .listresource_licensekeyread_ import (
        ListResourceLicenseKeyRead,
        ListResourceLicenseKeyReadTypedDict,
    )
    from .listresource_member_ import ListResourceMember, ListResourceMemberTypedDict
    from .listresource_meter_ import ListResourceMeter, ListResourceMeterTypedDict
    from .listresource_order_ import ListResourceOrder, ListResourceOrderTypedDict
    from .listresource_organization_ import (
        ListResourceOrganization,
        ListResourceOrganizationTypedDict,
    )
    from .listresource_product_ import ListResourceProduct, ListResourceProductTypedDict
    from .listresource_refund_ import ListResourceRefund, ListResourceRefundTypedDict
    from .listresource_subscription_ import (
        ListResourceSubscription,
        ListResourceSubscriptionTypedDict,
    )
    from .listresource_webhookdelivery_ import (
        ListResourceWebhookDelivery,
        ListResourceWebhookDeliveryTypedDict,
    )
    from .listresource_webhookendpoint_ import (
        ListResourceWebhookEndpoint,
        ListResourceWebhookEndpointTypedDict,
    )
    from .listresourcewithcursorpagination_event_ import (
        ListResourceWithCursorPaginationEvent,
        ListResourceWithCursorPaginationEventTypedDict,
    )
    from .llmmetadata import LLMMetadata, LLMMetadataTypedDict
    from .member import Member, MemberTypedDict
    from .membercreate import MemberCreate, MemberCreateTypedDict
    from .memberrole import MemberRole
    from .members_delete_memberop import (
        MembersDeleteMemberRequest,
        MembersDeleteMemberRequestTypedDict,
    )
    from .members_get_memberop import (
        MembersGetMemberRequest,
        MembersGetMemberRequestTypedDict,
    )
    from .members_list_membersop import (
        MembersListMembersRequest,
        MembersListMembersRequestTypedDict,
        MembersListMembersResponse,
        MembersListMembersResponseTypedDict,
    )
    from .members_update_memberop import (
        MembersUpdateMemberRequest,
        MembersUpdateMemberRequestTypedDict,
    )
    from .membersession import MemberSession, MemberSessionTypedDict
    from .membersessioncreate import MemberSessionCreate, MemberSessionCreateTypedDict
    from .membersortproperty import MemberSortProperty
    from .memberupdate import MemberUpdate, MemberUpdateTypedDict
    from .metadataoutputtype import MetadataOutputType, MetadataOutputTypeTypedDict
    from .meter import (
        Meter,
        MeterAggregation,
        MeterAggregationTypedDict,
        MeterTypedDict,
    )
    from .metercreate import (
        MeterCreate,
        MeterCreateAggregation,
        MeterCreateAggregationTypedDict,
        MeterCreateMetadata,
        MeterCreateMetadataTypedDict,
        MeterCreateTypedDict,
    )
    from .metercreditedmetadata import (
        MeterCreditedMetadata,
        MeterCreditedMetadataTypedDict,
    )
    from .metercreditevent import MeterCreditEvent, MeterCreditEventTypedDict
    from .meterquantities import MeterQuantities, MeterQuantitiesTypedDict
    from .meterquantity import MeterQuantity, MeterQuantityTypedDict
    from .meterresetevent import MeterResetEvent, MeterResetEventTypedDict
    from .meterresetmetadata import MeterResetMetadata, MeterResetMetadataTypedDict
    from .meters_getop import MetersGetRequest, MetersGetRequestTypedDict
    from .meters_updateop import MetersUpdateRequest, MetersUpdateRequestTypedDict
    from .metersortproperty import MeterSortProperty
    from .meterupdate import (
        Aggregation,
        AggregationTypedDict,
        MeterUpdate,
        MeterUpdateMetadata,
        MeterUpdateMetadataTypedDict,
        MeterUpdateTypedDict,
    )
    from .metric import Metric, MetricTypedDict
    from .metricperiod import (
        ActiveSubscriptions,
        ActiveSubscriptionsTypedDict,
        ActiveUserByEvent,
        ActiveUserByEventTypedDict,
        AverageOrderValue,
        AverageOrderValueTypedDict,
        AverageRevenuePerUser,
        AverageRevenuePerUserTypedDict,
        CanceledSubscriptions,
        CanceledSubscriptionsCustomerService,
        CanceledSubscriptionsCustomerServiceTypedDict,
        CanceledSubscriptionsLowQuality,
        CanceledSubscriptionsLowQualityTypedDict,
        CanceledSubscriptionsMissingFeatures,
        CanceledSubscriptionsMissingFeaturesTypedDict,
        CanceledSubscriptionsOther,
        CanceledSubscriptionsOtherTypedDict,
        CanceledSubscriptionsSwitchedService,
        CanceledSubscriptionsSwitchedServiceTypedDict,
        CanceledSubscriptionsTooComplex,
        CanceledSubscriptionsTooComplexTypedDict,
        CanceledSubscriptionsTooExpensive,
        CanceledSubscriptionsTooExpensiveTypedDict,
        CanceledSubscriptionsTypedDict,
        CanceledSubscriptionsUnused,
        CanceledSubscriptionsUnusedTypedDict,
        Cashflow,
        CashflowTypedDict,
        CheckoutsConversion,
        CheckoutsConversionTypedDict,
        CheckoutsModel,
        CheckoutsModelTypedDict,
        ChurnRate,
        ChurnRateTypedDict,
        ChurnedSubscriptions,
        ChurnedSubscriptionsTypedDict,
        CommittedMonthlyRecurringRevenue,
        CommittedMonthlyRecurringRevenueTypedDict,
        CommittedSubscriptions,
        CommittedSubscriptionsTypedDict,
        CostPerUser,
        CostPerUserTypedDict,
        Costs,
        CostsTypedDict,
        CumulativeCosts,
        CumulativeCostsTypedDict,
        CumulativeRevenue,
        CumulativeRevenueTypedDict,
        GrossMargin,
        GrossMarginPercentage,
        GrossMarginPercentageTypedDict,
        GrossMarginTypedDict,
        Ltv,
        LtvTypedDict,
        MetricPeriod,
        MetricPeriodTypedDict,
        MonthlyRecurringRevenue,
        MonthlyRecurringRevenueTypedDict,
        NetAverageOrderValue,
        NetAverageOrderValueTypedDict,
        NetCumulativeRevenue,
        NetCumulativeRevenueTypedDict,
        NetRevenue,
        NetRevenueTypedDict,
        NewSubscriptions,
        NewSubscriptionsNetRevenue,
        NewSubscriptionsNetRevenueTypedDict,
        NewSubscriptionsRevenue,
        NewSubscriptionsRevenueTypedDict,
        NewSubscriptionsTypedDict,
        OneTimeProducts,
        OneTimeProductsNetRevenue,
        OneTimeProductsNetRevenueTypedDict,
        OneTimeProductsRevenue,
        OneTimeProductsRevenueTypedDict,
        OneTimeProductsTypedDict,
        OrdersModel,
        OrdersModelTypedDict,
        RenewedSubscriptions,
        RenewedSubscriptionsNetRevenue,
        RenewedSubscriptionsNetRevenueTypedDict,
        RenewedSubscriptionsRevenue,
        RenewedSubscriptionsRevenueTypedDict,
        RenewedSubscriptionsTypedDict,
        Revenue,
        RevenueTypedDict,
        SucceededCheckouts,
        SucceededCheckoutsTypedDict,
    )
    from .metrics import Metrics, MetricsTypedDict
    from .metrics_getop import (
        MetricsGetQueryParamCustomerIDFilter,
        MetricsGetQueryParamCustomerIDFilterTypedDict,
        MetricsGetQueryParamOrganizationIDFilter,
        MetricsGetQueryParamOrganizationIDFilterTypedDict,
        MetricsGetQueryParamProductIDFilter,
        MetricsGetQueryParamProductIDFilterTypedDict,
        MetricsGetRequest,
        MetricsGetRequestTypedDict,
        QueryParamProductBillingTypeFilter,
        QueryParamProductBillingTypeFilterTypedDict,
    )
    from .metricsintervallimit import (
        MetricsIntervalLimit,
        MetricsIntervalLimitTypedDict,
    )
    from .metricsintervalslimits import (
        MetricsIntervalsLimits,
        MetricsIntervalsLimitsTypedDict,
    )
    from .metricslimits import MetricsLimits, MetricsLimitsTypedDict
    from .metricsresponse import MetricsResponse, MetricsResponseTypedDict
    from .metricstotals import (
        MetricsTotals,
        MetricsTotalsActiveSubscriptions,
        MetricsTotalsActiveSubscriptionsTypedDict,
        MetricsTotalsActiveUserByEvent,
        MetricsTotalsActiveUserByEventTypedDict,
        MetricsTotalsAverageOrderValue,
        MetricsTotalsAverageOrderValueTypedDict,
        MetricsTotalsAverageRevenuePerUser,
        MetricsTotalsAverageRevenuePerUserTypedDict,
        MetricsTotalsCanceledSubscriptions,
        MetricsTotalsCanceledSubscriptionsCustomerService,
        MetricsTotalsCanceledSubscriptionsCustomerServiceTypedDict,
        MetricsTotalsCanceledSubscriptionsLowQuality,
        MetricsTotalsCanceledSubscriptionsLowQualityTypedDict,
        MetricsTotalsCanceledSubscriptionsMissingFeatures,
        MetricsTotalsCanceledSubscriptionsMissingFeaturesTypedDict,
        MetricsTotalsCanceledSubscriptionsOther,
        MetricsTotalsCanceledSubscriptionsOtherTypedDict,
        MetricsTotalsCanceledSubscriptionsSwitchedService,
        MetricsTotalsCanceledSubscriptionsSwitchedServiceTypedDict,
        MetricsTotalsCanceledSubscriptionsTooComplex,
        MetricsTotalsCanceledSubscriptionsTooComplexTypedDict,
        MetricsTotalsCanceledSubscriptionsTooExpensive,
        MetricsTotalsCanceledSubscriptionsTooExpensiveTypedDict,
        MetricsTotalsCanceledSubscriptionsTypedDict,
        MetricsTotalsCanceledSubscriptionsUnused,
        MetricsTotalsCanceledSubscriptionsUnusedTypedDict,
        MetricsTotalsCashflow,
        MetricsTotalsCashflowTypedDict,
        MetricsTotalsCheckouts,
        MetricsTotalsCheckoutsConversion,
        MetricsTotalsCheckoutsConversionTypedDict,
        MetricsTotalsCheckoutsTypedDict,
        MetricsTotalsChurnRate,
        MetricsTotalsChurnRateTypedDict,
        MetricsTotalsChurnedSubscriptions,
        MetricsTotalsChurnedSubscriptionsTypedDict,
        MetricsTotalsCommittedMonthlyRecurringRevenue,
        MetricsTotalsCommittedMonthlyRecurringRevenueTypedDict,
        MetricsTotalsCommittedSubscriptions,
        MetricsTotalsCommittedSubscriptionsTypedDict,
        MetricsTotalsCostPerUser,
        MetricsTotalsCostPerUserTypedDict,
        MetricsTotalsCosts,
        MetricsTotalsCostsTypedDict,
        MetricsTotalsCumulativeCosts,
        MetricsTotalsCumulativeCostsTypedDict,
        MetricsTotalsCumulativeRevenue,
        MetricsTotalsCumulativeRevenueTypedDict,
        MetricsTotalsGrossMargin,
        MetricsTotalsGrossMarginPercentage,
        MetricsTotalsGrossMarginPercentageTypedDict,
        MetricsTotalsGrossMarginTypedDict,
        MetricsTotalsLtv,
        MetricsTotalsLtvTypedDict,
        MetricsTotalsMonthlyRecurringRevenue,
        MetricsTotalsMonthlyRecurringRevenueTypedDict,
        MetricsTotalsNetAverageOrderValue,
        MetricsTotalsNetAverageOrderValueTypedDict,
        MetricsTotalsNetCumulativeRevenue,
        MetricsTotalsNetCumulativeRevenueTypedDict,
        MetricsTotalsNetRevenue,
        MetricsTotalsNetRevenueTypedDict,
        MetricsTotalsNewSubscriptions,
        MetricsTotalsNewSubscriptionsNetRevenue,
        MetricsTotalsNewSubscriptionsNetRevenueTypedDict,
        MetricsTotalsNewSubscriptionsRevenue,
        MetricsTotalsNewSubscriptionsRevenueTypedDict,
        MetricsTotalsNewSubscriptionsTypedDict,
        MetricsTotalsOneTimeProducts,
        MetricsTotalsOneTimeProductsNetRevenue,
        MetricsTotalsOneTimeProductsNetRevenueTypedDict,
        MetricsTotalsOneTimeProductsRevenue,
        MetricsTotalsOneTimeProductsRevenueTypedDict,
        MetricsTotalsOneTimeProductsTypedDict,
        MetricsTotalsOrders,
        MetricsTotalsOrdersTypedDict,
        MetricsTotalsRenewedSubscriptions,
        MetricsTotalsRenewedSubscriptionsNetRevenue,
        MetricsTotalsRenewedSubscriptionsNetRevenueTypedDict,
        MetricsTotalsRenewedSubscriptionsRevenue,
        MetricsTotalsRenewedSubscriptionsRevenueTypedDict,
        MetricsTotalsRenewedSubscriptionsTypedDict,
        MetricsTotalsRevenue,
        MetricsTotalsRevenueTypedDict,
        MetricsTotalsSucceededCheckouts,
        MetricsTotalsSucceededCheckoutsTypedDict,
        MetricsTotalsTypedDict,
    )
    from .metrictype import MetricType
    from .missinginvoicebillingdetails import (
        MissingInvoiceBillingDetails,
        MissingInvoiceBillingDetailsData,
    )
    from .no_response_error import NoResponseError
    from .notopencheckout import NotOpenCheckout, NotOpenCheckoutData
    from .notpaidorder import NotPaidOrder, NotPaidOrderData
    from .notpermitted import NotPermitted, NotPermittedData
    from .oauth2_authorizeop import (
        Oauth2AuthorizeResponseOauth2Authorize,
        Oauth2AuthorizeResponseOauth2AuthorizeTypedDict,
    )
    from .oauth2_clients_oauth2_delete_clientop import (
        Oauth2ClientsOauth2DeleteClientRequest,
        Oauth2ClientsOauth2DeleteClientRequestTypedDict,
    )
    from .oauth2_clients_oauth2_get_clientop import (
        Oauth2ClientsOauth2GetClientRequest,
        Oauth2ClientsOauth2GetClientRequestTypedDict,
    )
    from .oauth2_clients_oauth2_update_clientop import (
        Oauth2ClientsOauth2UpdateClientRequest,
        Oauth2ClientsOauth2UpdateClientRequestTypedDict,
    )
    from .oauth2_request_tokenop import (
        Oauth2RequestTokenRequestBody,
        Oauth2RequestTokenRequestBodyTypedDict,
    )
    from .oauth2_userinfoop import (
        Oauth2UserinfoResponseOauth2Userinfo,
        Oauth2UserinfoResponseOauth2UserinfoTypedDict,
    )
    from .oauth2clientconfiguration import (
        GrantTypes,
        OAuth2ClientConfiguration,
        OAuth2ClientConfigurationTypedDict,
        TokenEndpointAuthMethod,
    )
    from .oauth2clientconfigurationupdate import (
        OAuth2ClientConfigurationUpdate,
        OAuth2ClientConfigurationUpdateGrantTypes,
        OAuth2ClientConfigurationUpdateTokenEndpointAuthMethod,
        OAuth2ClientConfigurationUpdateTypedDict,
    )
    from .oauth2clientpublic import OAuth2ClientPublic, OAuth2ClientPublicTypedDict
    from .order import (
        Order,
        OrderCustomFieldData,
        OrderCustomFieldDataTypedDict,
        OrderDiscount,
        OrderDiscountTypedDict,
        OrderTypedDict,
    )
    from .orderbillingreason import OrderBillingReason
    from .ordercustomer import (
        OrderCustomer,
        OrderCustomerTaxID,
        OrderCustomerTaxIDTypedDict,
        OrderCustomerTypedDict,
    )
    from .orderinvoice import OrderInvoice, OrderInvoiceTypedDict
    from .orderitemschema import OrderItemSchema, OrderItemSchemaTypedDict
    from .ordernoteligibleforretry import (
        OrderNotEligibleForRetry,
        OrderNotEligibleForRetryData,
    )
    from .orderpaidevent import OrderPaidEvent, OrderPaidEventTypedDict
    from .orderpaidmetadata import OrderPaidMetadata, OrderPaidMetadataTypedDict
    from .orderproduct import OrderProduct, OrderProductTypedDict
    from .orderrefundedevent import OrderRefundedEvent, OrderRefundedEventTypedDict
    from .orderrefundedmetadata import (
        OrderRefundedMetadata,
        OrderRefundedMetadataTypedDict,
    )
    from .orders_exportop import (
        OrdersExportQueryParamOrganizationIDFilter,
        OrdersExportQueryParamOrganizationIDFilterTypedDict,
        OrdersExportQueryParamProductIDFilter,
        OrdersExportQueryParamProductIDFilterTypedDict,
        OrdersExportRequest,
        OrdersExportRequestTypedDict,
    )
    from .orders_generate_invoiceop import (
        OrdersGenerateInvoiceRequest,
        OrdersGenerateInvoiceRequestTypedDict,
        OrdersGenerateInvoiceResponse422OrdersGenerateInvoice,
        OrdersGenerateInvoiceResponse422OrdersGenerateInvoiceUnion,
    )
    from .orders_getop import OrdersGetRequest, OrdersGetRequestTypedDict
    from .orders_invoiceop import OrdersInvoiceRequest, OrdersInvoiceRequestTypedDict
    from .orders_updateop import OrdersUpdateRequest, OrdersUpdateRequestTypedDict
    from .ordersortproperty import OrderSortProperty
    from .orderstatus import OrderStatus
    from .ordersubscription import OrderSubscription, OrderSubscriptionTypedDict
    from .orderupdate import OrderUpdate, OrderUpdateTypedDict
    from .organization import Organization, OrganizationTypedDict
    from .organizationavatarfilecreate import (
        OrganizationAvatarFileCreate,
        OrganizationAvatarFileCreateTypedDict,
    )
    from .organizationavatarfileread import (
        OrganizationAvatarFileRead,
        OrganizationAvatarFileReadTypedDict,
    )
    from .organizationcreate import OrganizationCreate, OrganizationCreateTypedDict
    from .organizationcustomeremailsettings import (
        OrganizationCustomerEmailSettings,
        OrganizationCustomerEmailSettingsTypedDict,
    )
    from .organizationcustomerportalsettings import (
        OrganizationCustomerPortalSettings,
        OrganizationCustomerPortalSettingsTypedDict,
    )
    from .organizationdetails import (
        OrganizationDetails,
        OrganizationDetailsTypedDict,
        SwitchingFrom,
    )
    from .organizationfeaturesettings import (
        OrganizationFeatureSettings,
        OrganizationFeatureSettingsTypedDict,
    )
    from .organizationnotificationsettings import (
        OrganizationNotificationSettings,
        OrganizationNotificationSettingsTypedDict,
    )
    from .organizations_getop import (
        OrganizationsGetRequest,
        OrganizationsGetRequestTypedDict,
    )
    from .organizations_listop import (
        OrganizationsListRequest,
        OrganizationsListRequestTypedDict,
        OrganizationsListResponse,
        OrganizationsListResponseTypedDict,
    )
    from .organizations_updateop import (
        OrganizationsUpdateRequest,
        OrganizationsUpdateRequestTypedDict,
    )
    from .organizationsociallink import (
        OrganizationSocialLink,
        OrganizationSocialLinkTypedDict,
    )
    from .organizationsocialplatforms import OrganizationSocialPlatforms
    from .organizationsortproperty import OrganizationSortProperty
    from .organizationstatus import OrganizationStatus
    from .organizationsubscriptionsettings import (
        OrganizationSubscriptionSettings,
        OrganizationSubscriptionSettingsTypedDict,
    )
    from .organizationupdate import OrganizationUpdate, OrganizationUpdateTypedDict
    from .ownercreate import OwnerCreate, OwnerCreateTypedDict
    from .pagination import Pagination, PaginationTypedDict
    from .payment import Payment, PaymentTypedDict
    from .paymentalreadyinprogress import (
        PaymentAlreadyInProgress,
        PaymentAlreadyInProgressData,
    )
    from .paymenterror import PaymentError, PaymentErrorData
    from .paymentmethodcard import PaymentMethodCard, PaymentMethodCardTypedDict
    from .paymentmethodcardmetadata import (
        PaymentMethodCardMetadata,
        PaymentMethodCardMetadataTypedDict,
    )
    from .paymentmethodgeneric import (
        PaymentMethodGeneric,
        PaymentMethodGenericTypedDict,
    )
    from .paymentmethodinusebyactivesubscription import (
        PaymentMethodInUseByActiveSubscription,
        PaymentMethodInUseByActiveSubscriptionData,
    )
    from .paymentnotready import PaymentNotReady, PaymentNotReadyData
    from .paymentprocessor import PaymentProcessor
    from .payments_getop import PaymentsGetRequest, PaymentsGetRequestTypedDict
    from .payments_listop import (
        CustomerEmailFilter,
        CustomerEmailFilterTypedDict,
        MethodFilter,
        MethodFilterTypedDict,
        PaymentsListQueryParamCheckoutIDFilter,
        PaymentsListQueryParamCheckoutIDFilterTypedDict,
        PaymentsListQueryParamOrderIDFilter,
        PaymentsListQueryParamOrderIDFilterTypedDict,
        PaymentsListQueryParamOrganizationIDFilter,
        PaymentsListQueryParamOrganizationIDFilterTypedDict,
        PaymentsListQueryParamStatusFilter,
        PaymentsListQueryParamStatusFilterTypedDict,
        PaymentsListRequest,
        PaymentsListRequestTypedDict,
        PaymentsListResponse,
        PaymentsListResponseTypedDict,
    )
    from .paymentsortproperty import PaymentSortProperty
    from .paymentstatus import PaymentStatus
    from .portalauthenticateduser import (
        PortalAuthenticatedUser,
        PortalAuthenticatedUserTypedDict,
    )
    from .presentmentcurrency import PresentmentCurrency
    from .product import Prices, PricesTypedDict, Product, ProductTypedDict
    from .productbenefitsupdate import (
        ProductBenefitsUpdate,
        ProductBenefitsUpdateTypedDict,
    )
    from .productbillingtype import ProductBillingType
    from .productcreate import ProductCreate, ProductCreateTypedDict
    from .productcreateonetime import (
        ProductCreateOneTime,
        ProductCreateOneTimeMetadata,
        ProductCreateOneTimeMetadataTypedDict,
        ProductCreateOneTimePrices,
        ProductCreateOneTimePricesTypedDict,
        ProductCreateOneTimeTypedDict,
    )
    from .productcreaterecurring import (
        ProductCreateRecurring,
        ProductCreateRecurringMetadata,
        ProductCreateRecurringMetadataTypedDict,
        ProductCreateRecurringPrices,
        ProductCreateRecurringPricesTypedDict,
        ProductCreateRecurringTypedDict,
    )
    from .productmediafilecreate import (
        ProductMediaFileCreate,
        ProductMediaFileCreateTypedDict,
    )
    from .productmediafileread import (
        ProductMediaFileRead,
        ProductMediaFileReadTypedDict,
    )
    from .productprice import ProductPrice, ProductPriceTypedDict
    from .productpricecustom import ProductPriceCustom, ProductPriceCustomTypedDict
    from .productpricecustomcreate import (
        ProductPriceCustomCreate,
        ProductPriceCustomCreateTypedDict,
    )
    from .productpricefixed import ProductPriceFixed, ProductPriceFixedTypedDict
    from .productpricefixedcreate import (
        ProductPriceFixedCreate,
        ProductPriceFixedCreateTypedDict,
    )
    from .productpricefree import ProductPriceFree, ProductPriceFreeTypedDict
    from .productpricefreecreate import (
        ProductPriceFreeCreate,
        ProductPriceFreeCreateTypedDict,
    )
    from .productpricemeter import ProductPriceMeter, ProductPriceMeterTypedDict
    from .productpricemeteredunit import (
        ProductPriceMeteredUnit,
        ProductPriceMeteredUnitTypedDict,
    )
    from .productpricemeteredunitcreate import (
        ProductPriceMeteredUnitCreate,
        ProductPriceMeteredUnitCreateTypedDict,
        UnitAmount,
        UnitAmountTypedDict,
    )
    from .productpriceseatbased import (
        ProductPriceSeatBased,
        ProductPriceSeatBasedTypedDict,
    )
    from .productpriceseatbasedcreate import (
        ProductPriceSeatBasedCreate,
        ProductPriceSeatBasedCreateTypedDict,
    )
    from .productpriceseattier import (
        ProductPriceSeatTier,
        ProductPriceSeatTierTypedDict,
    )
    from .productpriceseattiers_input import (
        ProductPriceSeatTiersInput,
        ProductPriceSeatTiersInputTypedDict,
    )
    from .productpriceseattiers_output import (
        ProductPriceSeatTiersOutput,
        ProductPriceSeatTiersOutputTypedDict,
    )
    from .productpricesource import ProductPriceSource
    from .productpricetype import ProductPriceType
    from .products_getop import ProductsGetRequest, ProductsGetRequestTypedDict
    from .products_update_benefitsop import (
        ProductsUpdateBenefitsRequest,
        ProductsUpdateBenefitsRequestTypedDict,
    )
    from .products_updateop import ProductsUpdateRequest, ProductsUpdateRequestTypedDict
    from .productsortproperty import ProductSortProperty
    from .productupdate import (
        ProductUpdate,
        ProductUpdateMetadata,
        ProductUpdateMetadataTypedDict,
        ProductUpdatePrices,
        ProductUpdatePricesTypedDict,
        ProductUpdateTypedDict,
        Two,
        TwoTypedDict,
    )
    from .propertyaggregation import (
        Func,
        PropertyAggregation,
        PropertyAggregationTypedDict,
    )
    from .refreshtokenrequest import RefreshTokenRequest, RefreshTokenRequestTypedDict
    from .refund import Refund, RefundTypedDict
    from .refundcreate import (
        RefundCreate,
        RefundCreateMetadata,
        RefundCreateMetadataTypedDict,
        RefundCreateTypedDict,
    )
    from .refunddispute import RefundDispute, RefundDisputeTypedDict
    from .refundedalready import RefundedAlready, RefundedAlreadyData
    from .refundreason import RefundReason
    from .refunds_listop import (
        OrderIDFilter,
        OrderIDFilterTypedDict,
        RefundIDFilter,
        RefundIDFilterTypedDict,
        RefundsListQueryParamCustomerIDFilter,
        RefundsListQueryParamCustomerIDFilterTypedDict,
        RefundsListQueryParamOrganizationIDFilter,
        RefundsListQueryParamOrganizationIDFilterTypedDict,
        RefundsListRequest,
        RefundsListRequestTypedDict,
        RefundsListResponse,
        RefundsListResponseTypedDict,
        SubscriptionIDFilter,
        SubscriptionIDFilterTypedDict,
    )
    from .refundsortproperty import RefundSortProperty
    from .refundstatus import RefundStatus
    from .resourcenotfound import ResourceNotFound, ResourceNotFoundData
    from .responsevalidationerror import ResponseValidationError
    from .revoketokenrequest import (
        RevokeTokenRequest,
        RevokeTokenRequestTypedDict,
        TokenTypeHint,
    )
    from .revoketokenresponse import RevokeTokenResponse, RevokeTokenResponseTypedDict
    from .s3downloadurl import S3DownloadURL, S3DownloadURLTypedDict
    from .s3filecreatemultipart import (
        S3FileCreateMultipart,
        S3FileCreateMultipartTypedDict,
    )
    from .s3filecreatepart import S3FileCreatePart, S3FileCreatePartTypedDict
    from .s3fileuploadcompletedpart import (
        S3FileUploadCompletedPart,
        S3FileUploadCompletedPartTypedDict,
    )
    from .s3fileuploadmultipart import (
        S3FileUploadMultipart,
        S3FileUploadMultipartTypedDict,
    )
    from .s3fileuploadpart import S3FileUploadPart, S3FileUploadPartTypedDict
    from .scope import Scope
    from .sdkerror import SDKError
    from .seatassign import SeatAssign, SeatAssignTypedDict
    from .seatclaim import SeatClaim, SeatClaimTypedDict
    from .seatclaiminfo import SeatClaimInfo, SeatClaimInfoTypedDict
    from .seatslist import SeatsList, SeatsListTypedDict
    from .seatstatus import SeatStatus
    from .security import Security, SecurityTypedDict
    from .subscription import (
        CustomFieldData,
        CustomFieldDataTypedDict,
        Subscription,
        SubscriptionDiscount,
        SubscriptionDiscountTypedDict,
        SubscriptionPrices,
        SubscriptionPricesTypedDict,
        SubscriptionTypedDict,
    )
    from .subscriptionbillingperiodupdatedevent import (
        SubscriptionBillingPeriodUpdatedEvent,
        SubscriptionBillingPeriodUpdatedEventTypedDict,
    )
    from .subscriptionbillingperiodupdatedmetadata import (
        SubscriptionBillingPeriodUpdatedMetadata,
        SubscriptionBillingPeriodUpdatedMetadataTypedDict,
    )
    from .subscriptioncancel import SubscriptionCancel, SubscriptionCancelTypedDict
    from .subscriptioncanceledevent import (
        SubscriptionCanceledEvent,
        SubscriptionCanceledEventTypedDict,
    )
    from .subscriptioncanceledmetadata import (
        SubscriptionCanceledMetadata,
        SubscriptionCanceledMetadataTypedDict,
    )
    from .subscriptioncreatecustomer import (
        Metadata,
        MetadataTypedDict,
        SubscriptionCreateCustomer,
        SubscriptionCreateCustomerTypedDict,
    )
    from .subscriptioncreatedevent import (
        SubscriptionCreatedEvent,
        SubscriptionCreatedEventTypedDict,
    )
    from .subscriptioncreatedmetadata import (
        SubscriptionCreatedMetadata,
        SubscriptionCreatedMetadataTypedDict,
    )
    from .subscriptioncreateexternalcustomer import (
        SubscriptionCreateExternalCustomer,
        SubscriptionCreateExternalCustomerMetadata,
        SubscriptionCreateExternalCustomerMetadataTypedDict,
        SubscriptionCreateExternalCustomerTypedDict,
    )
    from .subscriptioncustomer import (
        SubscriptionCustomer,
        SubscriptionCustomerTypedDict,
        TaxID,
        TaxIDTypedDict,
    )
    from .subscriptioncycledevent import (
        SubscriptionCycledEvent,
        SubscriptionCycledEventTypedDict,
    )
    from .subscriptioncycledmetadata import (
        SubscriptionCycledMetadata,
        SubscriptionCycledMetadataTypedDict,
    )
    from .subscriptionlocked import SubscriptionLocked, SubscriptionLockedData
    from .subscriptionmeter import SubscriptionMeter, SubscriptionMeterTypedDict
    from .subscriptionproductupdatedevent import (
        SubscriptionProductUpdatedEvent,
        SubscriptionProductUpdatedEventTypedDict,
    )
    from .subscriptionproductupdatedmetadata import (
        SubscriptionProductUpdatedMetadata,
        SubscriptionProductUpdatedMetadataTypedDict,
    )
    from .subscriptionprorationbehavior import SubscriptionProrationBehavior
    from .subscriptionrecurringinterval import SubscriptionRecurringInterval
    from .subscriptionrevoke import SubscriptionRevoke, SubscriptionRevokeTypedDict
    from .subscriptionrevokedevent import (
        SubscriptionRevokedEvent,
        SubscriptionRevokedEventTypedDict,
    )
    from .subscriptionrevokedmetadata import (
        SubscriptionRevokedMetadata,
        SubscriptionRevokedMetadataTypedDict,
    )
    from .subscriptions_createop import (
        SubscriptionsCreateSubscriptionCreate,
        SubscriptionsCreateSubscriptionCreateTypedDict,
    )
    from .subscriptions_exportop import (
        OrganizationID,
        OrganizationIDTypedDict,
        SubscriptionsExportRequest,
        SubscriptionsExportRequestTypedDict,
    )
    from .subscriptions_getop import (
        SubscriptionsGetRequest,
        SubscriptionsGetRequestTypedDict,
    )
    from .subscriptions_listop import (
        BenefitIDFilter,
        BenefitIDFilterTypedDict,
        BenefitTypeFilter,
        BenefitTypeFilterTypedDict,
        BenefitsListRequest,
        BenefitsListRequestTypedDict,
        BenefitsListResponse,
        BenefitsListResponseTypedDict,
        CheckoutIDFilter,
        CheckoutIDFilterTypedDict,
        CustomerIDFilter,
        CustomerIDFilterTypedDict,
        CustomersListQueryParamOrganizationIDFilter,
        CustomersListQueryParamOrganizationIDFilterTypedDict,
        CustomersListRequest,
        CustomersListRequestTypedDict,
        CustomersListResponse,
        CustomersListResponseTypedDict,
        DiscountIDFilter,
        DiscountIDFilterTypedDict,
        EventsListQueryParamCustomerIDFilter,
        EventsListQueryParamCustomerIDFilterTypedDict,
        EventsListQueryParamOrganizationIDFilter,
        EventsListQueryParamOrganizationIDFilterTypedDict,
        EventsListRequest,
        EventsListRequestTypedDict,
        EventsListResponseEventsList,
        EventsListResponseEventsListTypedDict,
        ExcludeIDs,
        ExcludeIDsTypedDict,
        ExternalCustomerIDFilter,
        ExternalCustomerIDFilterTypedDict,
        FilterIDs,
        FilterIDsTypedDict,
        MetadataQuery,
        MetadataQueryTypedDict,
        MetersListQueryParamOrganizationIDFilter,
        MetersListQueryParamOrganizationIDFilterTypedDict,
        MetersListRequest,
        MetersListRequestTypedDict,
        MetersListResponse,
        MetersListResponseTypedDict,
        MetersQuantitiesQueryParamCustomerIDFilter,
        MetersQuantitiesQueryParamCustomerIDFilterTypedDict,
        MetersQuantitiesQueryParamExternalCustomerIDFilter,
        MetersQuantitiesQueryParamExternalCustomerIDFilterTypedDict,
        MetersQuantitiesRequest,
        MetersQuantitiesRequestTypedDict,
        NameFilter,
        NameFilterTypedDict,
        OrdersListQueryParamCustomerIDFilter,
        OrdersListQueryParamCustomerIDFilterTypedDict,
        OrdersListQueryParamOrganizationIDFilter,
        OrdersListQueryParamOrganizationIDFilterTypedDict,
        OrdersListQueryParamProductIDFilter,
        OrdersListQueryParamProductIDFilterTypedDict,
        OrdersListRequest,
        OrdersListRequestTypedDict,
        OrdersListResponse,
        OrdersListResponseTypedDict,
        OrganizationIDFilter,
        OrganizationIDFilterTypedDict,
        ProductBillingTypeFilter,
        ProductBillingTypeFilterTypedDict,
        ProductIDFilter,
        ProductIDFilterTypedDict,
        ProductsListQueryParamOrganizationIDFilter,
        ProductsListQueryParamOrganizationIDFilterTypedDict,
        ProductsListRequest,
        ProductsListRequestTypedDict,
        ProductsListResponse,
        ProductsListResponseTypedDict,
        QueryParamDiscountIDFilter,
        QueryParamDiscountIDFilterTypedDict,
        QueryParamExternalCustomerIDFilter,
        QueryParamExternalCustomerIDFilterTypedDict,
        QueryParamOrganizationIDFilter,
        QueryParamOrganizationIDFilterTypedDict,
        QueryParamProductIDFilter,
        QueryParamProductIDFilterTypedDict,
        SourceFilter,
        SourceFilterTypedDict,
        SubscriptionsListRequest,
        SubscriptionsListRequestTypedDict,
        SubscriptionsListResponse,
        SubscriptionsListResponseTypedDict,
    )
    from .subscriptions_revokeop import (
        SubscriptionsRevokeRequest,
        SubscriptionsRevokeRequestTypedDict,
    )
    from .subscriptions_updateop import (
        SubscriptionsUpdateRequest,
        SubscriptionsUpdateRequestTypedDict,
    )
    from .subscriptionseatsupdatedevent import (
        SubscriptionSeatsUpdatedEvent,
        SubscriptionSeatsUpdatedEventTypedDict,
    )
    from .subscriptionseatsupdatedmetadata import (
        SubscriptionSeatsUpdatedMetadata,
        SubscriptionSeatsUpdatedMetadataTypedDict,
    )
    from .subscriptionsortproperty import SubscriptionSortProperty
    from .subscriptionstatus import SubscriptionStatus
    from .subscriptionuncanceledevent import (
        SubscriptionUncanceledEvent,
        SubscriptionUncanceledEventTypedDict,
    )
    from .subscriptionuncanceledmetadata import (
        SubscriptionUncanceledMetadata,
        SubscriptionUncanceledMetadataTypedDict,
    )
    from .subscriptionupdate import SubscriptionUpdate, SubscriptionUpdateTypedDict
    from .subscriptionupdatebillingperiod import (
        SubscriptionUpdateBillingPeriod,
        SubscriptionUpdateBillingPeriodTypedDict,
    )
    from .subscriptionupdatediscount import (
        SubscriptionUpdateDiscount,
        SubscriptionUpdateDiscountTypedDict,
    )
    from .subscriptionupdateproduct import (
        SubscriptionUpdateProduct,
        SubscriptionUpdateProductTypedDict,
    )
    from .subscriptionupdateseats import (
        SubscriptionUpdateSeats,
        SubscriptionUpdateSeatsTypedDict,
    )
    from .subscriptionupdatetrial import (
        SubscriptionUpdateTrial,
        SubscriptionUpdateTrialTypedDict,
    )
    from .subtype import SubType
    from .systemevent import SystemEvent, SystemEventTypedDict
    from .taxidformat import TaxIDFormat
    from .timeinterval import TimeInterval
    from .tokenresponse import TokenResponse, TokenResponseTypedDict
    from .trialalreadyredeemed import TrialAlreadyRedeemed, TrialAlreadyRedeemedData
    from .trialinterval import TrialInterval
    from .unauthorized import Unauthorized, UnauthorizedData
    from .uniqueaggregation import UniqueAggregation, UniqueAggregationTypedDict
    from .userevent import UserEvent, UserEventTypedDict
    from .userinfoorganization import (
        UserInfoOrganization,
        UserInfoOrganizationTypedDict,
    )
    from .userinfouser import UserInfoUser, UserInfoUserTypedDict
    from .validatedlicensekey import ValidatedLicenseKey, ValidatedLicenseKeyTypedDict
    from .validationerror import (
        Loc,
        LocTypedDict,
        ValidationError,
        ValidationErrorTypedDict,
    )
    from .webhookbenefitcreatedpayload import (
        WebhookBenefitCreatedPayload,
        WebhookBenefitCreatedPayloadTypedDict,
    )
    from .webhookbenefitgrantcreatedpayload import (
        WebhookBenefitGrantCreatedPayload,
        WebhookBenefitGrantCreatedPayloadTypedDict,
    )
    from .webhookbenefitgrantcycledpayload import (
        WebhookBenefitGrantCycledPayload,
        WebhookBenefitGrantCycledPayloadTypedDict,
    )
    from .webhookbenefitgrantrevokedpayload import (
        WebhookBenefitGrantRevokedPayload,
        WebhookBenefitGrantRevokedPayloadTypedDict,
    )
    from .webhookbenefitgrantupdatedpayload import (
        WebhookBenefitGrantUpdatedPayload,
        WebhookBenefitGrantUpdatedPayloadTypedDict,
    )
    from .webhookbenefitupdatedpayload import (
        WebhookBenefitUpdatedPayload,
        WebhookBenefitUpdatedPayloadTypedDict,
    )
    from .webhookcheckoutcreatedpayload import (
        WebhookCheckoutCreatedPayload,
        WebhookCheckoutCreatedPayloadTypedDict,
    )
    from .webhookcheckoutupdatedpayload import (
        WebhookCheckoutUpdatedPayload,
        WebhookCheckoutUpdatedPayloadTypedDict,
    )
    from .webhookcustomercreatedpayload import (
        WebhookCustomerCreatedPayload,
        WebhookCustomerCreatedPayloadTypedDict,
    )
    from .webhookcustomerdeletedpayload import (
        WebhookCustomerDeletedPayload,
        WebhookCustomerDeletedPayloadTypedDict,
    )
    from .webhookcustomerseatassignedpayload import (
        WebhookCustomerSeatAssignedPayload,
        WebhookCustomerSeatAssignedPayloadTypedDict,
    )
    from .webhookcustomerseatclaimedpayload import (
        WebhookCustomerSeatClaimedPayload,
        WebhookCustomerSeatClaimedPayloadTypedDict,
    )
    from .webhookcustomerseatrevokedpayload import (
        WebhookCustomerSeatRevokedPayload,
        WebhookCustomerSeatRevokedPayloadTypedDict,
    )
    from .webhookcustomerstatechangedpayload import (
        WebhookCustomerStateChangedPayload,
        WebhookCustomerStateChangedPayloadTypedDict,
    )
    from .webhookcustomerupdatedpayload import (
        WebhookCustomerUpdatedPayload,
        WebhookCustomerUpdatedPayloadTypedDict,
    )
    from .webhookdelivery import WebhookDelivery, WebhookDeliveryTypedDict
    from .webhookendpoint import WebhookEndpoint, WebhookEndpointTypedDict
    from .webhookendpointcreate import (
        WebhookEndpointCreate,
        WebhookEndpointCreateTypedDict,
    )
    from .webhookendpointupdate import (
        WebhookEndpointUpdate,
        WebhookEndpointUpdateTypedDict,
    )
    from .webhookevent import WebhookEvent, WebhookEventTypedDict
    from .webhookeventtype import WebhookEventType
    from .webhookformat import WebhookFormat
    from .webhookordercreatedpayload import (
        WebhookOrderCreatedPayload,
        WebhookOrderCreatedPayloadTypedDict,
    )
    from .webhookorderpaidpayload import (
        WebhookOrderPaidPayload,
        WebhookOrderPaidPayloadTypedDict,
    )
    from .webhookorderrefundedpayload import (
        WebhookOrderRefundedPayload,
        WebhookOrderRefundedPayloadTypedDict,
    )
    from .webhookorderupdatedpayload import (
        WebhookOrderUpdatedPayload,
        WebhookOrderUpdatedPayloadTypedDict,
    )
    from .webhookorganizationupdatedpayload import (
        WebhookOrganizationUpdatedPayload,
        WebhookOrganizationUpdatedPayloadTypedDict,
    )
    from .webhookproductcreatedpayload import (
        WebhookProductCreatedPayload,
        WebhookProductCreatedPayloadTypedDict,
    )
    from .webhookproductupdatedpayload import (
        WebhookProductUpdatedPayload,
        WebhookProductUpdatedPayloadTypedDict,
    )
    from .webhookrefundcreatedpayload import (
        WebhookRefundCreatedPayload,
        WebhookRefundCreatedPayloadTypedDict,
    )
    from .webhookrefundupdatedpayload import (
        WebhookRefundUpdatedPayload,
        WebhookRefundUpdatedPayloadTypedDict,
    )
    from .webhooks_delete_webhook_endpointop import (
        WebhooksDeleteWebhookEndpointRequest,
        WebhooksDeleteWebhookEndpointRequestTypedDict,
    )
    from .webhooks_get_webhook_endpointop import (
        WebhooksGetWebhookEndpointRequest,
        WebhooksGetWebhookEndpointRequestTypedDict,
    )
    from .webhooks_list_webhook_deliveriesop import (
        EndpointID,
        EndpointIDTypedDict,
        WebhooksListWebhookDeliveriesRequest,
        WebhooksListWebhookDeliveriesRequestTypedDict,
        WebhooksListWebhookDeliveriesResponse,
        WebhooksListWebhookDeliveriesResponseTypedDict,
    )
    from .webhooks_list_webhook_endpointsop import (
        QueryParamOrganizationID,
        QueryParamOrganizationIDTypedDict,
        WebhooksListWebhookEndpointsRequest,
        WebhooksListWebhookEndpointsRequestTypedDict,
        WebhooksListWebhookEndpointsResponse,
        WebhooksListWebhookEndpointsResponseTypedDict,
    )
    from .webhooks_redeliver_webhook_eventop import (
        WebhooksRedeliverWebhookEventRequest,
        WebhooksRedeliverWebhookEventRequestTypedDict,
    )
    from .webhooks_reset_webhook_endpoint_secretop import (
        WebhooksResetWebhookEndpointSecretRequest,
        WebhooksResetWebhookEndpointSecretRequestTypedDict,
    )
    from .webhooks_update_webhook_endpointop import (
        WebhooksUpdateWebhookEndpointRequest,
        WebhooksUpdateWebhookEndpointRequestTypedDict,
    )
    from .webhooksubscriptionactivepayload import (
        WebhookSubscriptionActivePayload,
        WebhookSubscriptionActivePayloadTypedDict,
    )
    from .webhooksubscriptioncanceledpayload import (
        WebhookSubscriptionCanceledPayload,
        WebhookSubscriptionCanceledPayloadTypedDict,
    )
    from .webhooksubscriptioncreatedpayload import (
        WebhookSubscriptionCreatedPayload,
        WebhookSubscriptionCreatedPayloadTypedDict,
    )
    from .webhooksubscriptionpastduepayload import (
        WebhookSubscriptionPastDuePayload,
        WebhookSubscriptionPastDuePayloadTypedDict,
    )
    from .webhooksubscriptionrevokedpayload import (
        WebhookSubscriptionRevokedPayload,
        WebhookSubscriptionRevokedPayloadTypedDict,
    )
    from .webhooksubscriptionuncanceledpayload import (
        WebhookSubscriptionUncanceledPayload,
        WebhookSubscriptionUncanceledPayloadTypedDict,
    )
    from .webhooksubscriptionupdatedpayload import (
        WebhookSubscriptionUpdatedPayload,
        WebhookSubscriptionUpdatedPayloadTypedDict,
    )
    from .webtokenrequest import (
        WebTokenRequest,
        WebTokenRequestSubType,
        WebTokenRequestTypedDict,
    )  # Pydantic models with forward references
Filter.model_rebuild()


__all__ = [
    "ActiveSubscriptions",
    "ActiveSubscriptionsTypedDict",
    "ActiveUserByEvent",
    "ActiveUserByEventTypedDict",
    "Address",
    "AddressDict",
    "AddressDictTypedDict",
    "AddressInput",
    "AddressInputTypedDict",
    "AddressTypedDict",
    "Aggregation",
    "AggregationFunction",
    "AggregationTypedDict",
    "AlreadyActiveSubscriptionError",
    "AlreadyActiveSubscriptionErrorData",
    "AlreadyCanceledSubscription",
    "AlreadyCanceledSubscriptionData",
    "Amount",
    "AmountTypedDict",
    "AttachedCustomField",
    "AttachedCustomFieldCreate",
    "AttachedCustomFieldCreateTypedDict",
    "AttachedCustomFieldTypedDict",
    "AuthorizationCodeTokenRequest",
    "AuthorizationCodeTokenRequestTypedDict",
    "AuthorizeOrganization",
    "AuthorizeOrganizationTypedDict",
    "AuthorizeResponseOrganization",
    "AuthorizeResponseOrganizationTypedDict",
    "AuthorizeResponseUser",
    "AuthorizeResponseUserTypedDict",
    "AuthorizeUser",
    "AuthorizeUserTypedDict",
    "AverageOrderValue",
    "AverageOrderValueTypedDict",
    "AverageRevenuePerUser",
    "AverageRevenuePerUserTypedDict",
    "BalanceDisputeEvent",
    "BalanceDisputeEventTypedDict",
    "BalanceDisputeMetadata",
    "BalanceDisputeMetadataTypedDict",
    "BalanceDisputeReversalEvent",
    "BalanceDisputeReversalEventTypedDict",
    "BalanceOrderEvent",
    "BalanceOrderEventTypedDict",
    "BalanceOrderMetadata",
    "BalanceOrderMetadataTypedDict",
    "BalanceRefundEvent",
    "BalanceRefundEventTypedDict",
    "BalanceRefundMetadata",
    "BalanceRefundMetadataTypedDict",
    "BalanceRefundReversalEvent",
    "BalanceRefundReversalEventTypedDict",
    "Benefit",
    "BenefitCreate",
    "BenefitCreateTypedDict",
    "BenefitCustom",
    "BenefitCustomCreate",
    "BenefitCustomCreateMetadata",
    "BenefitCustomCreateMetadataTypedDict",
    "BenefitCustomCreateProperties",
    "BenefitCustomCreatePropertiesTypedDict",
    "BenefitCustomCreateTypedDict",
    "BenefitCustomProperties",
    "BenefitCustomPropertiesTypedDict",
    "BenefitCustomSubscriber",
    "BenefitCustomSubscriberProperties",
    "BenefitCustomSubscriberPropertiesTypedDict",
    "BenefitCustomSubscriberTypedDict",
    "BenefitCustomTypedDict",
    "BenefitCustomUpdate",
    "BenefitCustomUpdateMetadata",
    "BenefitCustomUpdateMetadataTypedDict",
    "BenefitCustomUpdateTypedDict",
    "BenefitCycledEvent",
    "BenefitCycledEventTypedDict",
    "BenefitDiscord",
    "BenefitDiscordCreate",
    "BenefitDiscordCreateMetadata",
    "BenefitDiscordCreateMetadataTypedDict",
    "BenefitDiscordCreateProperties",
    "BenefitDiscordCreatePropertiesTypedDict",
    "BenefitDiscordCreateTypedDict",
    "BenefitDiscordProperties",
    "BenefitDiscordPropertiesTypedDict",
    "BenefitDiscordSubscriber",
    "BenefitDiscordSubscriberProperties",
    "BenefitDiscordSubscriberPropertiesTypedDict",
    "BenefitDiscordSubscriberTypedDict",
    "BenefitDiscordTypedDict",
    "BenefitDiscordUpdate",
    "BenefitDiscordUpdateMetadata",
    "BenefitDiscordUpdateMetadataTypedDict",
    "BenefitDiscordUpdateTypedDict",
    "BenefitDownloadables",
    "BenefitDownloadablesCreate",
    "BenefitDownloadablesCreateMetadata",
    "BenefitDownloadablesCreateMetadataTypedDict",
    "BenefitDownloadablesCreateProperties",
    "BenefitDownloadablesCreatePropertiesTypedDict",
    "BenefitDownloadablesCreateTypedDict",
    "BenefitDownloadablesProperties",
    "BenefitDownloadablesPropertiesTypedDict",
    "BenefitDownloadablesSubscriber",
    "BenefitDownloadablesSubscriberProperties",
    "BenefitDownloadablesSubscriberPropertiesTypedDict",
    "BenefitDownloadablesSubscriberTypedDict",
    "BenefitDownloadablesTypedDict",
    "BenefitDownloadablesUpdate",
    "BenefitDownloadablesUpdateMetadata",
    "BenefitDownloadablesUpdateMetadataTypedDict",
    "BenefitDownloadablesUpdateTypedDict",
    "BenefitGitHubRepository",
    "BenefitGitHubRepositoryCreate",
    "BenefitGitHubRepositoryCreateMetadata",
    "BenefitGitHubRepositoryCreateMetadataTypedDict",
    "BenefitGitHubRepositoryCreateProperties",
    "BenefitGitHubRepositoryCreatePropertiesPermission",
    "BenefitGitHubRepositoryCreatePropertiesTypedDict",
    "BenefitGitHubRepositoryCreateTypedDict",
    "BenefitGitHubRepositoryProperties",
    "BenefitGitHubRepositoryPropertiesTypedDict",
    "BenefitGitHubRepositorySubscriber",
    "BenefitGitHubRepositorySubscriberProperties",
    "BenefitGitHubRepositorySubscriberPropertiesTypedDict",
    "BenefitGitHubRepositorySubscriberTypedDict",
    "BenefitGitHubRepositoryTypedDict",
    "BenefitGitHubRepositoryUpdate",
    "BenefitGitHubRepositoryUpdateMetadata",
    "BenefitGitHubRepositoryUpdateMetadataTypedDict",
    "BenefitGitHubRepositoryUpdateTypedDict",
    "BenefitGrant",
    "BenefitGrantCustomProperties",
    "BenefitGrantCustomPropertiesTypedDict",
    "BenefitGrantCustomWebhook",
    "BenefitGrantCustomWebhookTypedDict",
    "BenefitGrantDiscordProperties",
    "BenefitGrantDiscordPropertiesTypedDict",
    "BenefitGrantDiscordWebhook",
    "BenefitGrantDiscordWebhookTypedDict",
    "BenefitGrantDownloadablesProperties",
    "BenefitGrantDownloadablesPropertiesTypedDict",
    "BenefitGrantDownloadablesWebhook",
    "BenefitGrantDownloadablesWebhookTypedDict",
    "BenefitGrantError",
    "BenefitGrantErrorTypedDict",
    "BenefitGrantGitHubRepositoryProperties",
    "BenefitGrantGitHubRepositoryPropertiesPermission",
    "BenefitGrantGitHubRepositoryPropertiesTypedDict",
    "BenefitGrantGitHubRepositoryWebhook",
    "BenefitGrantGitHubRepositoryWebhookTypedDict",
    "BenefitGrantLicenseKeysProperties",
    "BenefitGrantLicenseKeysPropertiesTypedDict",
    "BenefitGrantLicenseKeysWebhook",
    "BenefitGrantLicenseKeysWebhookTypedDict",
    "BenefitGrantMetadata",
    "BenefitGrantMetadataTypedDict",
    "BenefitGrantMeterCreditProperties",
    "BenefitGrantMeterCreditPropertiesTypedDict",
    "BenefitGrantMeterCreditWebhook",
    "BenefitGrantMeterCreditWebhookTypedDict",
    "BenefitGrantSortProperty",
    "BenefitGrantTypedDict",
    "BenefitGrantWebhook",
    "BenefitGrantWebhookTypedDict",
    "BenefitGrantedEvent",
    "BenefitGrantedEventTypedDict",
    "BenefitGrantsListQueryParamCustomerIDFilter",
    "BenefitGrantsListQueryParamCustomerIDFilterTypedDict",
    "BenefitGrantsListQueryParamOrganizationIDFilter",
    "BenefitGrantsListQueryParamOrganizationIDFilterTypedDict",
    "BenefitGrantsListRequest",
    "BenefitGrantsListRequestTypedDict",
    "BenefitGrantsListResponse",
    "BenefitGrantsListResponseTypedDict",
    "BenefitIDFilter",
    "BenefitIDFilterTypedDict",
    "BenefitLicenseKeyActivationCreateProperties",
    "BenefitLicenseKeyActivationCreatePropertiesTypedDict",
    "BenefitLicenseKeyActivationProperties",
    "BenefitLicenseKeyActivationPropertiesTypedDict",
    "BenefitLicenseKeyExpirationProperties",
    "BenefitLicenseKeyExpirationPropertiesTypedDict",
    "BenefitLicenseKeys",
    "BenefitLicenseKeysCreate",
    "BenefitLicenseKeysCreateMetadata",
    "BenefitLicenseKeysCreateMetadataTypedDict",
    "BenefitLicenseKeysCreateProperties",
    "BenefitLicenseKeysCreatePropertiesTypedDict",
    "BenefitLicenseKeysCreateTypedDict",
    "BenefitLicenseKeysProperties",
    "BenefitLicenseKeysPropertiesTypedDict",
    "BenefitLicenseKeysSubscriber",
    "BenefitLicenseKeysSubscriberProperties",
    "BenefitLicenseKeysSubscriberPropertiesTypedDict",
    "BenefitLicenseKeysSubscriberTypedDict",
    "BenefitLicenseKeysTypedDict",
    "BenefitLicenseKeysUpdate",
    "BenefitLicenseKeysUpdateMetadata",
    "BenefitLicenseKeysUpdateMetadataTypedDict",
    "BenefitLicenseKeysUpdateTypedDict",
    "BenefitMeterCredit",
    "BenefitMeterCreditCreate",
    "BenefitMeterCreditCreateMetadata",
    "BenefitMeterCreditCreateMetadataTypedDict",
    "BenefitMeterCreditCreateProperties",
    "BenefitMeterCreditCreatePropertiesTypedDict",
    "BenefitMeterCreditCreateTypedDict",
    "BenefitMeterCreditProperties",
    "BenefitMeterCreditPropertiesTypedDict",
    "BenefitMeterCreditSubscriber",
    "BenefitMeterCreditSubscriberProperties",
    "BenefitMeterCreditSubscriberPropertiesTypedDict",
    "BenefitMeterCreditSubscriberTypedDict",
    "BenefitMeterCreditTypedDict",
    "BenefitMeterCreditUpdate",
    "BenefitMeterCreditUpdateMetadata",
    "BenefitMeterCreditUpdateMetadataTypedDict",
    "BenefitMeterCreditUpdateTypedDict",
    "BenefitPublic",
    "BenefitPublicTypedDict",
    "BenefitRevokedEvent",
    "BenefitRevokedEventTypedDict",
    "BenefitSortProperty",
    "BenefitSubscriberOrganization",
    "BenefitSubscriberOrganizationTypedDict",
    "BenefitType",
    "BenefitTypeFilter",
    "BenefitTypeFilterTypedDict",
    "BenefitTypedDict",
    "BenefitUpdatedEvent",
    "BenefitUpdatedEventTypedDict",
    "BenefitsDeleteRequest",
    "BenefitsDeleteRequestTypedDict",
    "BenefitsGetRequest",
    "BenefitsGetRequestTypedDict",
    "BenefitsGrantsRequest",
    "BenefitsGrantsRequestTypedDict",
    "BenefitsGrantsResponse",
    "BenefitsGrantsResponseTypedDict",
    "BenefitsListRequest",
    "BenefitsListRequestTypedDict",
    "BenefitsListResponse",
    "BenefitsListResponseTypedDict",
    "BenefitsUpdateBenefitUpdate",
    "BenefitsUpdateBenefitUpdateTypedDict",
    "BenefitsUpdateRequest",
    "BenefitsUpdateRequestTypedDict",
    "BillingAddressFieldMode",
    "CanceledSubscriptions",
    "CanceledSubscriptionsCustomerService",
    "CanceledSubscriptionsCustomerServiceTypedDict",
    "CanceledSubscriptionsLowQuality",
    "CanceledSubscriptionsLowQualityTypedDict",
    "CanceledSubscriptionsMissingFeatures",
    "CanceledSubscriptionsMissingFeaturesTypedDict",
    "CanceledSubscriptionsOther",
    "CanceledSubscriptionsOtherTypedDict",
    "CanceledSubscriptionsSwitchedService",
    "CanceledSubscriptionsSwitchedServiceTypedDict",
    "CanceledSubscriptionsTooComplex",
    "CanceledSubscriptionsTooComplexTypedDict",
    "CanceledSubscriptionsTooExpensive",
    "CanceledSubscriptionsTooExpensiveTypedDict",
    "CanceledSubscriptionsTypedDict",
    "CanceledSubscriptionsUnused",
    "CanceledSubscriptionsUnusedTypedDict",
    "CardPayment",
    "CardPaymentMetadata",
    "CardPaymentMetadataTypedDict",
    "CardPaymentTypedDict",
    "Cashflow",
    "CashflowTypedDict",
    "Checkout",
    "CheckoutBillingAddressFields",
    "CheckoutBillingAddressFieldsTypedDict",
    "CheckoutConfirmStripe",
    "CheckoutConfirmStripeCustomFieldData",
    "CheckoutConfirmStripeCustomFieldDataTypedDict",
    "CheckoutConfirmStripeTypedDict",
    "CheckoutCreate",
    "CheckoutCreateCustomFieldData",
    "CheckoutCreateCustomFieldDataTypedDict",
    "CheckoutCreateCustomerMetadata",
    "CheckoutCreateCustomerMetadataTypedDict",
    "CheckoutCreateMetadata",
    "CheckoutCreateMetadataTypedDict",
    "CheckoutCreatePrices",
    "CheckoutCreatePricesTypedDict",
    "CheckoutCreateTypedDict",
    "CheckoutCreatedEvent",
    "CheckoutCreatedEventTypedDict",
    "CheckoutCreatedMetadata",
    "CheckoutCreatedMetadataTypedDict",
    "CheckoutCustomFieldData",
    "CheckoutCustomFieldDataTypedDict",
    "CheckoutDiscount",
    "CheckoutDiscountFixedOnceForeverDuration",
    "CheckoutDiscountFixedOnceForeverDurationTypedDict",
    "CheckoutDiscountFixedRepeatDuration",
    "CheckoutDiscountFixedRepeatDurationTypedDict",
    "CheckoutDiscountPercentageOnceForeverDuration",
    "CheckoutDiscountPercentageOnceForeverDurationTypedDict",
    "CheckoutDiscountPercentageRepeatDuration",
    "CheckoutDiscountPercentageRepeatDurationTypedDict",
    "CheckoutDiscountTypedDict",
    "CheckoutForbiddenError",
    "CheckoutForbiddenErrorUnion",
    "CheckoutIDFilter",
    "CheckoutIDFilterTypedDict",
    "CheckoutLink",
    "CheckoutLinkCreate",
    "CheckoutLinkCreateProduct",
    "CheckoutLinkCreateProductMetadata",
    "CheckoutLinkCreateProductMetadataTypedDict",
    "CheckoutLinkCreateProductPrice",
    "CheckoutLinkCreateProductPriceMetadata",
    "CheckoutLinkCreateProductPriceMetadataTypedDict",
    "CheckoutLinkCreateProductPriceTypedDict",
    "CheckoutLinkCreateProductTypedDict",
    "CheckoutLinkCreateProducts",
    "CheckoutLinkCreateProductsMetadata",
    "CheckoutLinkCreateProductsMetadataTypedDict",
    "CheckoutLinkCreateProductsTypedDict",
    "CheckoutLinkCreateTypedDict",
    "CheckoutLinkDiscount",
    "CheckoutLinkDiscountTypedDict",
    "CheckoutLinkProduct",
    "CheckoutLinkProductPrices",
    "CheckoutLinkProductPricesTypedDict",
    "CheckoutLinkProductTypedDict",
    "CheckoutLinkSortProperty",
    "CheckoutLinkTypedDict",
    "CheckoutLinkUpdate",
    "CheckoutLinkUpdateMetadata",
    "CheckoutLinkUpdateMetadataTypedDict",
    "CheckoutLinkUpdateTypedDict",
    "CheckoutLinksDeleteRequest",
    "CheckoutLinksDeleteRequestTypedDict",
    "CheckoutLinksGetRequest",
    "CheckoutLinksGetRequestTypedDict",
    "CheckoutLinksListQueryParamOrganizationIDFilter",
    "CheckoutLinksListQueryParamOrganizationIDFilterTypedDict",
    "CheckoutLinksListQueryParamProductIDFilter",
    "CheckoutLinksListQueryParamProductIDFilterTypedDict",
    "CheckoutLinksListRequest",
    "CheckoutLinksListRequestTypedDict",
    "CheckoutLinksListResponse",
    "CheckoutLinksListResponseTypedDict",
    "CheckoutLinksUpdateRequest",
    "CheckoutLinksUpdateRequestTypedDict",
    "CheckoutOrganization",
    "CheckoutOrganizationTypedDict",
    "CheckoutPrices",
    "CheckoutPricesTypedDict",
    "CheckoutProduct",
    "CheckoutProductPrice",
    "CheckoutProductPriceTypedDict",
    "CheckoutProductPrices",
    "CheckoutProductPricesTypedDict",
    "CheckoutProductTypedDict",
    "CheckoutPublic",
    "CheckoutPublicConfirmed",
    "CheckoutPublicConfirmedCustomFieldData",
    "CheckoutPublicConfirmedCustomFieldDataTypedDict",
    "CheckoutPublicConfirmedDiscount",
    "CheckoutPublicConfirmedDiscountTypedDict",
    "CheckoutPublicConfirmedPrices",
    "CheckoutPublicConfirmedPricesTypedDict",
    "CheckoutPublicConfirmedProductPrice",
    "CheckoutPublicConfirmedProductPriceTypedDict",
    "CheckoutPublicConfirmedTypedDict",
    "CheckoutPublicCustomFieldData",
    "CheckoutPublicCustomFieldDataTypedDict",
    "CheckoutPublicDiscount",
    "CheckoutPublicDiscountTypedDict",
    "CheckoutPublicPrices",
    "CheckoutPublicPricesTypedDict",
    "CheckoutPublicProductPrice",
    "CheckoutPublicProductPriceTypedDict",
    "CheckoutPublicTypedDict",
    "CheckoutSortProperty",
    "CheckoutStatus",
    "CheckoutTypedDict",
    "CheckoutUpdate",
    "CheckoutUpdateCustomFieldData",
    "CheckoutUpdateCustomFieldDataTypedDict",
    "CheckoutUpdateCustomerMetadata",
    "CheckoutUpdateCustomerMetadataTypedDict",
    "CheckoutUpdateMetadata",
    "CheckoutUpdateMetadataTypedDict",
    "CheckoutUpdatePublic",
    "CheckoutUpdatePublicCustomFieldData",
    "CheckoutUpdatePublicCustomFieldDataTypedDict",
    "CheckoutUpdatePublicTypedDict",
    "CheckoutUpdateTypedDict",
    "CheckoutsClientConfirmRequest",
    "CheckoutsClientConfirmRequestTypedDict",
    "CheckoutsClientGetRequest",
    "CheckoutsClientGetRequestTypedDict",
    "CheckoutsClientUpdateRequest",
    "CheckoutsClientUpdateRequestTypedDict",
    "CheckoutsConversion",
    "CheckoutsConversionTypedDict",
    "CheckoutsGetRequest",
    "CheckoutsGetRequestTypedDict",
    "CheckoutsListQueryParamCustomerIDFilter",
    "CheckoutsListQueryParamCustomerIDFilterTypedDict",
    "CheckoutsListQueryParamOrganizationIDFilter",
    "CheckoutsListQueryParamOrganizationIDFilterTypedDict",
    "CheckoutsListQueryParamProductIDFilter",
    "CheckoutsListQueryParamProductIDFilterTypedDict",
    "CheckoutsListRequest",
    "CheckoutsListRequestTypedDict",
    "CheckoutsListResponse",
    "CheckoutsListResponseTypedDict",
    "CheckoutsModel",
    "CheckoutsModelTypedDict",
    "CheckoutsUpdateRequest",
    "CheckoutsUpdateRequestTypedDict",
    "ChurnRate",
    "ChurnRateTypedDict",
    "ChurnedSubscriptions",
    "ChurnedSubscriptionsTypedDict",
    "Clauses",
    "ClausesTypedDict",
    "CommittedMonthlyRecurringRevenue",
    "CommittedMonthlyRecurringRevenueTypedDict",
    "CommittedSubscriptions",
    "CommittedSubscriptionsTypedDict",
    "Conditions",
    "ConditionsTypedDict",
    "CostMetadataInput",
    "CostMetadataInputTypedDict",
    "CostMetadataOutput",
    "CostMetadataOutputTypedDict",
    "CostPerUser",
    "CostPerUserTypedDict",
    "Costs",
    "CostsTypedDict",
    "CountAggregation",
    "CountAggregationTypedDict",
    "CountryAlpha2",
    "CountryAlpha2Input",
    "CumulativeCosts",
    "CumulativeCostsTypedDict",
    "CumulativeRevenue",
    "CumulativeRevenueTypedDict",
    "CursorPagination",
    "CursorPaginationTypedDict",
    "CustomField",
    "CustomFieldCheckbox",
    "CustomFieldCheckboxProperties",
    "CustomFieldCheckboxPropertiesTypedDict",
    "CustomFieldCheckboxTypedDict",
    "CustomFieldCreate",
    "CustomFieldCreateCheckbox",
    "CustomFieldCreateCheckboxMetadata",
    "CustomFieldCreateCheckboxMetadataTypedDict",
    "CustomFieldCreateCheckboxTypedDict",
    "CustomFieldCreateDate",
    "CustomFieldCreateDateMetadata",
    "CustomFieldCreateDateMetadataTypedDict",
    "CustomFieldCreateDateTypedDict",
    "CustomFieldCreateNumber",
    "CustomFieldCreateNumberMetadata",
    "CustomFieldCreateNumberMetadataTypedDict",
    "CustomFieldCreateNumberTypedDict",
    "CustomFieldCreateSelect",
    "CustomFieldCreateSelectMetadata",
    "CustomFieldCreateSelectMetadataTypedDict",
    "CustomFieldCreateSelectTypedDict",
    "CustomFieldCreateText",
    "CustomFieldCreateTextMetadata",
    "CustomFieldCreateTextMetadataTypedDict",
    "CustomFieldCreateTextTypedDict",
    "CustomFieldCreateTypedDict",
    "CustomFieldData",
    "CustomFieldDataTypedDict",
    "CustomFieldDate",
    "CustomFieldDateProperties",
    "CustomFieldDatePropertiesTypedDict",
    "CustomFieldDateTypedDict",
    "CustomFieldNumber",
    "CustomFieldNumberProperties",
    "CustomFieldNumberPropertiesTypedDict",
    "CustomFieldNumberTypedDict",
    "CustomFieldSelect",
    "CustomFieldSelectOption",
    "CustomFieldSelectOptionTypedDict",
    "CustomFieldSelectProperties",
    "CustomFieldSelectPropertiesTypedDict",
    "CustomFieldSelectTypedDict",
    "CustomFieldSortProperty",
    "CustomFieldText",
    "CustomFieldTextProperties",
    "CustomFieldTextPropertiesTypedDict",
    "CustomFieldTextTypedDict",
    "CustomFieldType",
    "CustomFieldTypeFilter",
    "CustomFieldTypeFilterTypedDict",
    "CustomFieldTypedDict",
    "CustomFieldUpdate",
    "CustomFieldUpdateCheckbox",
    "CustomFieldUpdateCheckboxMetadata",
    "CustomFieldUpdateCheckboxMetadataTypedDict",
    "CustomFieldUpdateCheckboxTypedDict",
    "CustomFieldUpdateDate",
    "CustomFieldUpdateDateMetadata",
    "CustomFieldUpdateDateMetadataTypedDict",
    "CustomFieldUpdateDateTypedDict",
    "CustomFieldUpdateNumber",
    "CustomFieldUpdateNumberMetadata",
    "CustomFieldUpdateNumberMetadataTypedDict",
    "CustomFieldUpdateNumberTypedDict",
    "CustomFieldUpdateSelect",
    "CustomFieldUpdateSelectMetadata",
    "CustomFieldUpdateSelectMetadataTypedDict",
    "CustomFieldUpdateSelectTypedDict",
    "CustomFieldUpdateText",
    "CustomFieldUpdateTextMetadata",
    "CustomFieldUpdateTextMetadataTypedDict",
    "CustomFieldUpdateTextTypedDict",
    "CustomFieldUpdateTypedDict",
    "CustomFieldsDeleteRequest",
    "CustomFieldsDeleteRequestTypedDict",
    "CustomFieldsGetRequest",
    "CustomFieldsGetRequestTypedDict",
    "CustomFieldsListQueryParamOrganizationIDFilter",
    "CustomFieldsListQueryParamOrganizationIDFilterTypedDict",
    "CustomFieldsListRequest",
    "CustomFieldsListRequestTypedDict",
    "CustomFieldsListResponse",
    "CustomFieldsListResponseTypedDict",
    "CustomFieldsUpdateRequest",
    "CustomFieldsUpdateRequestTypedDict",
    "Customer",
    "CustomerBenefitGrant",
    "CustomerBenefitGrantCustom",
    "CustomerBenefitGrantCustomTypedDict",
    "CustomerBenefitGrantCustomUpdate",
    "CustomerBenefitGrantCustomUpdateTypedDict",
    "CustomerBenefitGrantDiscord",
    "CustomerBenefitGrantDiscordPropertiesUpdate",
    "CustomerBenefitGrantDiscordPropertiesUpdateTypedDict",
    "CustomerBenefitGrantDiscordTypedDict",
    "CustomerBenefitGrantDiscordUpdate",
    "CustomerBenefitGrantDiscordUpdateTypedDict",
    "CustomerBenefitGrantDownloadables",
    "CustomerBenefitGrantDownloadablesTypedDict",
    "CustomerBenefitGrantDownloadablesUpdate",
    "CustomerBenefitGrantDownloadablesUpdateTypedDict",
    "CustomerBenefitGrantGitHubRepository",
    "CustomerBenefitGrantGitHubRepositoryPropertiesUpdate",
    "CustomerBenefitGrantGitHubRepositoryPropertiesUpdateTypedDict",
    "CustomerBenefitGrantGitHubRepositoryTypedDict",
    "CustomerBenefitGrantGitHubRepositoryUpdate",
    "CustomerBenefitGrantGitHubRepositoryUpdateTypedDict",
    "CustomerBenefitGrantLicenseKeys",
    "CustomerBenefitGrantLicenseKeysTypedDict",
    "CustomerBenefitGrantLicenseKeysUpdate",
    "CustomerBenefitGrantLicenseKeysUpdateTypedDict",
    "CustomerBenefitGrantMeterCredit",
    "CustomerBenefitGrantMeterCreditTypedDict",
    "CustomerBenefitGrantMeterCreditUpdate",
    "CustomerBenefitGrantMeterCreditUpdateTypedDict",
    "CustomerBenefitGrantSortProperty",
    "CustomerBenefitGrantTypedDict",
    "CustomerBenefitGrantUpdate",
    "CustomerBenefitGrantUpdateTypedDict",
    "CustomerCancellationReason",
    "CustomerCreate",
    "CustomerCreateMetadata",
    "CustomerCreateMetadataTypedDict",
    "CustomerCreateTaxID",
    "CustomerCreateTaxIDTypedDict",
    "CustomerCreateTypedDict",
    "CustomerCreatedEvent",
    "CustomerCreatedEventTypedDict",
    "CustomerCreatedMetadata",
    "CustomerCreatedMetadataTypedDict",
    "CustomerCustomerMeter",
    "CustomerCustomerMeterMeter",
    "CustomerCustomerMeterMeterTypedDict",
    "CustomerCustomerMeterSortProperty",
    "CustomerCustomerMeterTypedDict",
    "CustomerCustomerSession",
    "CustomerCustomerSessionTypedDict",
    "CustomerDeletedEvent",
    "CustomerDeletedEventTypedDict",
    "CustomerDeletedMetadata",
    "CustomerDeletedMetadataTypedDict",
    "CustomerEmailFilter",
    "CustomerEmailFilterTypedDict",
    "CustomerIDFilter",
    "CustomerIDFilterTypedDict",
    "CustomerMetadata",
    "CustomerMetadataTypedDict",
    "CustomerMeter",
    "CustomerMeterSortProperty",
    "CustomerMeterTypedDict",
    "CustomerMetersGetRequest",
    "CustomerMetersGetRequestTypedDict",
    "CustomerMetersListQueryParamCustomerIDFilter",
    "CustomerMetersListQueryParamCustomerIDFilterTypedDict",
    "CustomerMetersListQueryParamExternalCustomerIDFilter",
    "CustomerMetersListQueryParamExternalCustomerIDFilterTypedDict",
    "CustomerMetersListQueryParamOrganizationIDFilter",
    "CustomerMetersListQueryParamOrganizationIDFilterTypedDict",
    "CustomerMetersListRequest",
    "CustomerMetersListRequestTypedDict",
    "CustomerMetersListResponse",
    "CustomerMetersListResponseTypedDict",
    "CustomerNotReady",
    "CustomerNotReadyData",
    "CustomerOrder",
    "CustomerOrderConfirmPayment",
    "CustomerOrderConfirmPaymentTypedDict",
    "CustomerOrderInvoice",
    "CustomerOrderInvoiceTypedDict",
    "CustomerOrderPaymentConfirmation",
    "CustomerOrderPaymentConfirmationTypedDict",
    "CustomerOrderPaymentStatus",
    "CustomerOrderPaymentStatusTypedDict",
    "CustomerOrderProduct",
    "CustomerOrderProductPrices",
    "CustomerOrderProductPricesTypedDict",
    "CustomerOrderProductTypedDict",
    "CustomerOrderSortProperty",
    "CustomerOrderSubscription",
    "CustomerOrderSubscriptionTypedDict",
    "CustomerOrderTypedDict",
    "CustomerOrderUpdate",
    "CustomerOrderUpdateTypedDict",
    "CustomerOrganization",
    "CustomerOrganizationData",
    "CustomerOrganizationDataTypedDict",
    "CustomerOrganizationTypedDict",
    "CustomerPaymentMethod",
    "CustomerPaymentMethodConfirm",
    "CustomerPaymentMethodConfirmTypedDict",
    "CustomerPaymentMethodCreate",
    "CustomerPaymentMethodCreateRequiresActionResponse",
    "CustomerPaymentMethodCreateRequiresActionResponseTypedDict",
    "CustomerPaymentMethodCreateResponse",
    "CustomerPaymentMethodCreateResponseTypedDict",
    "CustomerPaymentMethodCreateSucceededResponse",
    "CustomerPaymentMethodCreateSucceededResponseTypedDict",
    "CustomerPaymentMethodCreateTypedDict",
    "CustomerPaymentMethodTypedDict",
    "CustomerPortalBenefitGrantsGetRequest",
    "CustomerPortalBenefitGrantsGetRequestTypedDict",
    "CustomerPortalBenefitGrantsGetSecurity",
    "CustomerPortalBenefitGrantsGetSecurityTypedDict",
    "CustomerPortalBenefitGrantsListQueryParamBenefitIDFilter",
    "CustomerPortalBenefitGrantsListQueryParamBenefitIDFilterTypedDict",
    "CustomerPortalBenefitGrantsListQueryParamOrderIDFilter",
    "CustomerPortalBenefitGrantsListQueryParamOrderIDFilterTypedDict",
    "CustomerPortalBenefitGrantsListRequest",
    "CustomerPortalBenefitGrantsListRequestTypedDict",
    "CustomerPortalBenefitGrantsListResponse",
    "CustomerPortalBenefitGrantsListResponseTypedDict",
    "CustomerPortalBenefitGrantsListSecurity",
    "CustomerPortalBenefitGrantsListSecurityTypedDict",
    "CustomerPortalBenefitGrantsUpdateRequest",
    "CustomerPortalBenefitGrantsUpdateRequestTypedDict",
    "CustomerPortalBenefitGrantsUpdateSecurity",
    "CustomerPortalBenefitGrantsUpdateSecurityTypedDict",
    "CustomerPortalCustomer",
    "CustomerPortalCustomerMetersGetRequest",
    "CustomerPortalCustomerMetersGetRequestTypedDict",
    "CustomerPortalCustomerMetersGetSecurity",
    "CustomerPortalCustomerMetersGetSecurityTypedDict",
    "CustomerPortalCustomerMetersListRequest",
    "CustomerPortalCustomerMetersListRequestTypedDict",
    "CustomerPortalCustomerMetersListResponse",
    "CustomerPortalCustomerMetersListResponseTypedDict",
    "CustomerPortalCustomerMetersListSecurity",
    "CustomerPortalCustomerMetersListSecurityTypedDict",
    "CustomerPortalCustomerSessionGetAuthenticatedUserSecurity",
    "CustomerPortalCustomerSessionGetAuthenticatedUserSecurityTypedDict",
    "CustomerPortalCustomerSessionIntrospectSecurity",
    "CustomerPortalCustomerSessionIntrospectSecurityTypedDict",
    "CustomerPortalCustomerTaxID",
    "CustomerPortalCustomerTaxIDTypedDict",
    "CustomerPortalCustomerTypedDict",
    "CustomerPortalCustomerUpdate",
    "CustomerPortalCustomerUpdateTypedDict",
    "CustomerPortalCustomersAddPaymentMethodSecurity",
    "CustomerPortalCustomersAddPaymentMethodSecurityTypedDict",
    "CustomerPortalCustomersConfirmPaymentMethodSecurity",
    "CustomerPortalCustomersConfirmPaymentMethodSecurityTypedDict",
    "CustomerPortalCustomersDeletePaymentMethodRequest",
    "CustomerPortalCustomersDeletePaymentMethodRequestTypedDict",
    "CustomerPortalCustomersDeletePaymentMethodSecurity",
    "CustomerPortalCustomersDeletePaymentMethodSecurityTypedDict",
    "CustomerPortalCustomersGetSecurity",
    "CustomerPortalCustomersGetSecurityTypedDict",
    "CustomerPortalCustomersListPaymentMethodsRequest",
    "CustomerPortalCustomersListPaymentMethodsRequestTypedDict",
    "CustomerPortalCustomersListPaymentMethodsResponse",
    "CustomerPortalCustomersListPaymentMethodsResponseTypedDict",
    "CustomerPortalCustomersListPaymentMethodsSecurity",
    "CustomerPortalCustomersListPaymentMethodsSecurityTypedDict",
    "CustomerPortalCustomersUpdateSecurity",
    "CustomerPortalCustomersUpdateSecurityTypedDict",
    "CustomerPortalDownloadablesListQueryParamBenefitIDFilter",
    "CustomerPortalDownloadablesListQueryParamBenefitIDFilterTypedDict",
    "CustomerPortalDownloadablesListRequest",
    "CustomerPortalDownloadablesListRequestTypedDict",
    "CustomerPortalDownloadablesListResponse",
    "CustomerPortalDownloadablesListResponseTypedDict",
    "CustomerPortalDownloadablesListSecurity",
    "CustomerPortalDownloadablesListSecurityTypedDict",
    "CustomerPortalLicenseKeysGetRequest",
    "CustomerPortalLicenseKeysGetRequestTypedDict",
    "CustomerPortalLicenseKeysGetSecurity",
    "CustomerPortalLicenseKeysGetSecurityTypedDict",
    "CustomerPortalLicenseKeysListRequest",
    "CustomerPortalLicenseKeysListRequestTypedDict",
    "CustomerPortalLicenseKeysListResponse",
    "CustomerPortalLicenseKeysListResponseTypedDict",
    "CustomerPortalLicenseKeysListSecurity",
    "CustomerPortalLicenseKeysListSecurityTypedDict",
    "CustomerPortalOAuthAccount",
    "CustomerPortalOAuthAccountTypedDict",
    "CustomerPortalOrdersConfirmRetryPaymentRequest",
    "CustomerPortalOrdersConfirmRetryPaymentRequestTypedDict",
    "CustomerPortalOrdersConfirmRetryPaymentSecurity",
    "CustomerPortalOrdersConfirmRetryPaymentSecurityTypedDict",
    "CustomerPortalOrdersGenerateInvoiceRequest",
    "CustomerPortalOrdersGenerateInvoiceRequestTypedDict",
    "CustomerPortalOrdersGenerateInvoiceResponse422CustomerPortalOrdersGenerateInvoice",
    "CustomerPortalOrdersGenerateInvoiceResponse422CustomerPortalOrdersGenerateInvoiceUnion",
    "CustomerPortalOrdersGenerateInvoiceSecurity",
    "CustomerPortalOrdersGenerateInvoiceSecurityTypedDict",
    "CustomerPortalOrdersGetPaymentStatusRequest",
    "CustomerPortalOrdersGetPaymentStatusRequestTypedDict",
    "CustomerPortalOrdersGetPaymentStatusSecurity",
    "CustomerPortalOrdersGetPaymentStatusSecurityTypedDict",
    "CustomerPortalOrdersGetRequest",
    "CustomerPortalOrdersGetRequestTypedDict",
    "CustomerPortalOrdersGetSecurity",
    "CustomerPortalOrdersGetSecurityTypedDict",
    "CustomerPortalOrdersInvoiceRequest",
    "CustomerPortalOrdersInvoiceRequestTypedDict",
    "CustomerPortalOrdersInvoiceSecurity",
    "CustomerPortalOrdersInvoiceSecurityTypedDict",
    "CustomerPortalOrdersListQueryParamProductBillingTypeFilter",
    "CustomerPortalOrdersListQueryParamProductBillingTypeFilterTypedDict",
    "CustomerPortalOrdersListQueryParamProductIDFilter",
    "CustomerPortalOrdersListQueryParamProductIDFilterTypedDict",
    "CustomerPortalOrdersListQueryParamSubscriptionIDFilter",
    "CustomerPortalOrdersListQueryParamSubscriptionIDFilterTypedDict",
    "CustomerPortalOrdersListRequest",
    "CustomerPortalOrdersListRequestTypedDict",
    "CustomerPortalOrdersListResponse",
    "CustomerPortalOrdersListResponseTypedDict",
    "CustomerPortalOrdersListSecurity",
    "CustomerPortalOrdersListSecurityTypedDict",
    "CustomerPortalOrdersUpdateRequest",
    "CustomerPortalOrdersUpdateRequestTypedDict",
    "CustomerPortalOrdersUpdateSecurity",
    "CustomerPortalOrdersUpdateSecurityTypedDict",
    "CustomerPortalOrganizationsGetRequest",
    "CustomerPortalOrganizationsGetRequestTypedDict",
    "CustomerPortalSeatsAssignSeatSecurity",
    "CustomerPortalSeatsAssignSeatSecurityTypedDict",
    "CustomerPortalSeatsListClaimedSubscriptionsSecurity",
    "CustomerPortalSeatsListClaimedSubscriptionsSecurityTypedDict",
    "CustomerPortalSeatsListSeatsRequest",
    "CustomerPortalSeatsListSeatsRequestTypedDict",
    "CustomerPortalSeatsListSeatsSecurity",
    "CustomerPortalSeatsListSeatsSecurityTypedDict",
    "CustomerPortalSeatsResendInvitationRequest",
    "CustomerPortalSeatsResendInvitationRequestTypedDict",
    "CustomerPortalSeatsResendInvitationSecurity",
    "CustomerPortalSeatsResendInvitationSecurityTypedDict",
    "CustomerPortalSeatsRevokeSeatRequest",
    "CustomerPortalSeatsRevokeSeatRequestTypedDict",
    "CustomerPortalSeatsRevokeSeatSecurity",
    "CustomerPortalSeatsRevokeSeatSecurityTypedDict",
    "CustomerPortalSubscriptionSettings",
    "CustomerPortalSubscriptionSettingsTypedDict",
    "CustomerPortalSubscriptionsCancelRequest",
    "CustomerPortalSubscriptionsCancelRequestTypedDict",
    "CustomerPortalSubscriptionsCancelSecurity",
    "CustomerPortalSubscriptionsCancelSecurityTypedDict",
    "CustomerPortalSubscriptionsGetRequest",
    "CustomerPortalSubscriptionsGetRequestTypedDict",
    "CustomerPortalSubscriptionsGetSecurity",
    "CustomerPortalSubscriptionsGetSecurityTypedDict",
    "CustomerPortalSubscriptionsListQueryParamProductIDFilter",
    "CustomerPortalSubscriptionsListQueryParamProductIDFilterTypedDict",
    "CustomerPortalSubscriptionsListRequest",
    "CustomerPortalSubscriptionsListRequestTypedDict",
    "CustomerPortalSubscriptionsListResponse",
    "CustomerPortalSubscriptionsListResponseTypedDict",
    "CustomerPortalSubscriptionsListSecurity",
    "CustomerPortalSubscriptionsListSecurityTypedDict",
    "CustomerPortalSubscriptionsUpdateRequest",
    "CustomerPortalSubscriptionsUpdateRequestTypedDict",
    "CustomerPortalSubscriptionsUpdateSecurity",
    "CustomerPortalSubscriptionsUpdateSecurityTypedDict",
    "CustomerPortalUsageSettings",
    "CustomerPortalUsageSettingsTypedDict",
    "CustomerPortalWalletsGetRequest",
    "CustomerPortalWalletsGetRequestTypedDict",
    "CustomerPortalWalletsGetSecurity",
    "CustomerPortalWalletsGetSecurityTypedDict",
    "CustomerPortalWalletsListRequest",
    "CustomerPortalWalletsListRequestTypedDict",
    "CustomerPortalWalletsListResponse",
    "CustomerPortalWalletsListResponseTypedDict",
    "CustomerPortalWalletsListSecurity",
    "CustomerPortalWalletsListSecurityTypedDict",
    "CustomerProduct",
    "CustomerProductPrices",
    "CustomerProductPricesTypedDict",
    "CustomerProductTypedDict",
    "CustomerSeat",
    "CustomerSeatClaimResponse",
    "CustomerSeatClaimResponseTypedDict",
    "CustomerSeatTypedDict",
    "CustomerSeatsGetClaimInfoRequest",
    "CustomerSeatsGetClaimInfoRequestTypedDict",
    "CustomerSeatsListSeatsRequest",
    "CustomerSeatsListSeatsRequestTypedDict",
    "CustomerSeatsResendInvitationRequest",
    "CustomerSeatsResendInvitationRequestTypedDict",
    "CustomerSeatsRevokeSeatRequest",
    "CustomerSeatsRevokeSeatRequestTypedDict",
    "CustomerSession",
    "CustomerSessionCustomerExternalIDCreate",
    "CustomerSessionCustomerExternalIDCreateTypedDict",
    "CustomerSessionCustomerIDCreate",
    "CustomerSessionCustomerIDCreateTypedDict",
    "CustomerSessionTypedDict",
    "CustomerSessionsCreateCustomerSessionCreate",
    "CustomerSessionsCreateCustomerSessionCreateTypedDict",
    "CustomerSortProperty",
    "CustomerState",
    "CustomerStateBenefitGrant",
    "CustomerStateBenefitGrantProperties",
    "CustomerStateBenefitGrantPropertiesTypedDict",
    "CustomerStateBenefitGrantTypedDict",
    "CustomerStateMeter",
    "CustomerStateMeterTypedDict",
    "CustomerStateSubscription",
    "CustomerStateSubscriptionCustomFieldData",
    "CustomerStateSubscriptionCustomFieldDataTypedDict",
    "CustomerStateSubscriptionMeter",
    "CustomerStateSubscriptionMeterTypedDict",
    "CustomerStateSubscriptionTypedDict",
    "CustomerStateTaxID",
    "CustomerStateTaxIDTypedDict",
    "CustomerStateTypedDict",
    "CustomerSubscription",
    "CustomerSubscriptionCancel",
    "CustomerSubscriptionCancelTypedDict",
    "CustomerSubscriptionMeter",
    "CustomerSubscriptionMeterMeter",
    "CustomerSubscriptionMeterMeterTypedDict",
    "CustomerSubscriptionMeterTypedDict",
    "CustomerSubscriptionPrices",
    "CustomerSubscriptionPricesTypedDict",
    "CustomerSubscriptionProduct",
    "CustomerSubscriptionProductPrices",
    "CustomerSubscriptionProductPricesTypedDict",
    "CustomerSubscriptionProductTypedDict",
    "CustomerSubscriptionSortProperty",
    "CustomerSubscriptionTypedDict",
    "CustomerSubscriptionUpdate",
    "CustomerSubscriptionUpdateProduct",
    "CustomerSubscriptionUpdateProductTypedDict",
    "CustomerSubscriptionUpdateSeats",
    "CustomerSubscriptionUpdateSeatsTypedDict",
    "CustomerSubscriptionUpdateTypedDict",
    "CustomerTaxID",
    "CustomerTaxIDTypedDict",
    "CustomerTypedDict",
    "CustomerUpdate",
    "CustomerUpdateExternalID",
    "CustomerUpdateExternalIDMetadata",
    "CustomerUpdateExternalIDMetadataTypedDict",
    "CustomerUpdateExternalIDTaxID",
    "CustomerUpdateExternalIDTaxIDTypedDict",
    "CustomerUpdateExternalIDTypedDict",
    "CustomerUpdateMetadata",
    "CustomerUpdateMetadataTypedDict",
    "CustomerUpdateTaxID",
    "CustomerUpdateTaxIDTypedDict",
    "CustomerUpdateTypedDict",
    "CustomerUpdatedEvent",
    "CustomerUpdatedEventTypedDict",
    "CustomerUpdatedFields",
    "CustomerUpdatedFieldsMetadata",
    "CustomerUpdatedFieldsMetadataTypedDict",
    "CustomerUpdatedFieldsTypedDict",
    "CustomerUpdatedMetadata",
    "CustomerUpdatedMetadataTypedDict",
    "CustomerWallet",
    "CustomerWalletSortProperty",
    "CustomerWalletTypedDict",
    "CustomerWithMembers",
    "CustomerWithMembersTaxID",
    "CustomerWithMembersTaxIDTypedDict",
    "CustomerWithMembersTypedDict",
    "CustomersDeleteExternalRequest",
    "CustomersDeleteExternalRequestTypedDict",
    "CustomersDeleteRequest",
    "CustomersDeleteRequestTypedDict",
    "CustomersExportQueryParamOrganizationID",
    "CustomersExportQueryParamOrganizationIDTypedDict",
    "CustomersExportRequest",
    "CustomersExportRequestTypedDict",
    "CustomersGetExternalRequest",
    "CustomersGetExternalRequestTypedDict",
    "CustomersGetRequest",
    "CustomersGetRequestTypedDict",
    "CustomersGetStateExternalRequest",
    "CustomersGetStateExternalRequestTypedDict",
    "CustomersGetStateRequest",
    "CustomersGetStateRequestTypedDict",
    "CustomersListQueryParamOrganizationIDFilter",
    "CustomersListQueryParamOrganizationIDFilterTypedDict",
    "CustomersListRequest",
    "CustomersListRequestTypedDict",
    "CustomersListResponse",
    "CustomersListResponseTypedDict",
    "CustomersUpdateExternalRequest",
    "CustomersUpdateExternalRequestTypedDict",
    "CustomersUpdateRequest",
    "CustomersUpdateRequestTypedDict",
    "Discount",
    "DiscountCreate",
    "DiscountCreateTypedDict",
    "DiscountDuration",
    "DiscountFixedOnceForeverDuration",
    "DiscountFixedOnceForeverDurationBase",
    "DiscountFixedOnceForeverDurationBaseTypedDict",
    "DiscountFixedOnceForeverDurationCreate",
    "DiscountFixedOnceForeverDurationCreateMetadata",
    "DiscountFixedOnceForeverDurationCreateMetadataTypedDict",
    "DiscountFixedOnceForeverDurationCreateTypedDict",
    "DiscountFixedOnceForeverDurationTypedDict",
    "DiscountFixedRepeatDuration",
    "DiscountFixedRepeatDurationBase",
    "DiscountFixedRepeatDurationBaseTypedDict",
    "DiscountFixedRepeatDurationCreate",
    "DiscountFixedRepeatDurationCreateMetadata",
    "DiscountFixedRepeatDurationCreateMetadataTypedDict",
    "DiscountFixedRepeatDurationCreateTypedDict",
    "DiscountFixedRepeatDurationTypedDict",
    "DiscountIDFilter",
    "DiscountIDFilterTypedDict",
    "DiscountPercentageOnceForeverDuration",
    "DiscountPercentageOnceForeverDurationBase",
    "DiscountPercentageOnceForeverDurationBaseTypedDict",
    "DiscountPercentageOnceForeverDurationCreate",
    "DiscountPercentageOnceForeverDurationCreateMetadata",
    "DiscountPercentageOnceForeverDurationCreateMetadataTypedDict",
    "DiscountPercentageOnceForeverDurationCreateTypedDict",
    "DiscountPercentageOnceForeverDurationTypedDict",
    "DiscountPercentageRepeatDuration",
    "DiscountPercentageRepeatDurationBase",
    "DiscountPercentageRepeatDurationBaseTypedDict",
    "DiscountPercentageRepeatDurationCreate",
    "DiscountPercentageRepeatDurationCreateMetadata",
    "DiscountPercentageRepeatDurationCreateMetadataTypedDict",
    "DiscountPercentageRepeatDurationCreateTypedDict",
    "DiscountPercentageRepeatDurationTypedDict",
    "DiscountProduct",
    "DiscountProductTypedDict",
    "DiscountSortProperty",
    "DiscountType",
    "DiscountTypedDict",
    "DiscountUpdate",
    "DiscountUpdateMetadata",
    "DiscountUpdateMetadataTypedDict",
    "DiscountUpdateTypedDict",
    "DiscountsDeleteRequest",
    "DiscountsDeleteRequestTypedDict",
    "DiscountsGetRequest",
    "DiscountsGetRequestTypedDict",
    "DiscountsListQueryParamOrganizationIDFilter",
    "DiscountsListQueryParamOrganizationIDFilterTypedDict",
    "DiscountsListRequest",
    "DiscountsListRequestTypedDict",
    "DiscountsListResponse",
    "DiscountsListResponseTypedDict",
    "DiscountsUpdateRequest",
    "DiscountsUpdateRequestTypedDict",
    "Dispute",
    "DisputeSortProperty",
    "DisputeStatus",
    "DisputeTypedDict",
    "DisputesGetRequest",
    "DisputesGetRequestTypedDict",
    "DisputesListQueryParamOrganizationIDFilter",
    "DisputesListQueryParamOrganizationIDFilterTypedDict",
    "DisputesListRequest",
    "DisputesListRequestTypedDict",
    "DisputesListResponse",
    "DisputesListResponseTypedDict",
    "DownloadableFileCreate",
    "DownloadableFileCreateTypedDict",
    "DownloadableFileRead",
    "DownloadableFileReadTypedDict",
    "DownloadableRead",
    "DownloadableReadTypedDict",
    "EndpointID",
    "EndpointIDTypedDict",
    "Event",
    "EventCreateCustomer",
    "EventCreateCustomerTypedDict",
    "EventCreateExternalCustomer",
    "EventCreateExternalCustomerTypedDict",
    "EventMetadataInput",
    "EventMetadataInputTypedDict",
    "EventMetadataOutput",
    "EventMetadataOutputTypedDict",
    "EventName",
    "EventNameTypedDict",
    "EventNamesSortProperty",
    "EventSortProperty",
    "EventSource",
    "EventType",
    "EventTypeTypedDict",
    "EventTypeUpdate",
    "EventTypeUpdateTypedDict",
    "EventTypeWithStats",
    "EventTypeWithStatsTypedDict",
    "EventTypedDict",
    "EventTypesListQueryParamCustomerIDFilter",
    "EventTypesListQueryParamCustomerIDFilterTypedDict",
    "EventTypesListQueryParamExternalCustomerIDFilter",
    "EventTypesListQueryParamExternalCustomerIDFilterTypedDict",
    "EventTypesListQueryParamOrganizationIDFilter",
    "EventTypesListQueryParamOrganizationIDFilterTypedDict",
    "EventTypesListRequest",
    "EventTypesListRequestTypedDict",
    "EventTypesListResponse",
    "EventTypesListResponseTypedDict",
    "EventTypesSortProperty",
    "EventTypesUpdateRequest",
    "EventTypesUpdateRequestTypedDict",
    "EventsGetRequest",
    "EventsGetRequestTypedDict",
    "EventsIngest",
    "EventsIngestResponse",
    "EventsIngestResponseTypedDict",
    "EventsIngestTypedDict",
    "EventsListNamesQueryParamCustomerIDFilter",
    "EventsListNamesQueryParamCustomerIDFilterTypedDict",
    "EventsListNamesQueryParamExternalCustomerIDFilter",
    "EventsListNamesQueryParamExternalCustomerIDFilterTypedDict",
    "EventsListNamesQueryParamOrganizationIDFilter",
    "EventsListNamesQueryParamOrganizationIDFilterTypedDict",
    "EventsListNamesRequest",
    "EventsListNamesRequestTypedDict",
    "EventsListNamesResponse",
    "EventsListNamesResponseTypedDict",
    "EventsListQueryParamCustomerIDFilter",
    "EventsListQueryParamCustomerIDFilterTypedDict",
    "EventsListQueryParamOrganizationIDFilter",
    "EventsListQueryParamOrganizationIDFilterTypedDict",
    "EventsListRequest",
    "EventsListRequestTypedDict",
    "EventsListResponseEventsList",
    "EventsListResponseEventsListTypedDict",
    "EventsModel",
    "EventsModelTypedDict",
    "ExcludeIDs",
    "ExcludeIDsTypedDict",
    "ExistingProductPrice",
    "ExistingProductPriceTypedDict",
    "ExpiredCheckoutError",
    "ExpiredCheckoutErrorData",
    "ExternalCustomerIDFilter",
    "ExternalCustomerIDFilterTypedDict",
    "FileCreate",
    "FileCreateTypedDict",
    "FileDownload",
    "FileDownloadTypedDict",
    "FileIDFilter",
    "FileIDFilterTypedDict",
    "FilePatch",
    "FilePatchTypedDict",
    "FileRead",
    "FileReadTypedDict",
    "FileServiceTypes",
    "FileUpload",
    "FileUploadCompleted",
    "FileUploadCompletedTypedDict",
    "FileUploadTypedDict",
    "FilesDeleteRequest",
    "FilesDeleteRequestTypedDict",
    "FilesListQueryParamOrganizationIDFilter",
    "FilesListQueryParamOrganizationIDFilterTypedDict",
    "FilesListRequest",
    "FilesListRequestTypedDict",
    "FilesListResponse",
    "FilesListResponseTypedDict",
    "FilesUpdateRequest",
    "FilesUpdateRequestTypedDict",
    "FilesUpdateResponseFilesUpdate",
    "FilesUpdateResponseFilesUpdateTypedDict",
    "FilesUploadedRequest",
    "FilesUploadedRequestTypedDict",
    "FilesUploadedResponseFilesUploaded",
    "FilesUploadedResponseFilesUploadedTypedDict",
    "Filter",
    "FilterClause",
    "FilterClauseTypedDict",
    "FilterConjunction",
    "FilterIDs",
    "FilterIDsTypedDict",
    "FilterOperator",
    "FilterTypedDict",
    "Func",
    "GenericPayment",
    "GenericPaymentTypedDict",
    "GrantTypes",
    "GrossMargin",
    "GrossMarginPercentage",
    "GrossMarginPercentageTypedDict",
    "GrossMarginTypedDict",
    "HTTPValidationError",
    "HTTPValidationErrorData",
    "IntrospectTokenRequest",
    "IntrospectTokenRequestTokenTypeHint",
    "IntrospectTokenRequestTypedDict",
    "IntrospectTokenResponse",
    "IntrospectTokenResponseTypedDict",
    "LLMMetadata",
    "LLMMetadataTypedDict",
    "LegacyRecurringProductPrice",
    "LegacyRecurringProductPriceCustom",
    "LegacyRecurringProductPriceCustomTypedDict",
    "LegacyRecurringProductPriceFixed",
    "LegacyRecurringProductPriceFixedTypedDict",
    "LegacyRecurringProductPriceFree",
    "LegacyRecurringProductPriceFreeTypedDict",
    "LegacyRecurringProductPriceTypedDict",
    "LicenseKeyActivate",
    "LicenseKeyActivateConditions",
    "LicenseKeyActivateConditionsTypedDict",
    "LicenseKeyActivateMeta",
    "LicenseKeyActivateMetaTypedDict",
    "LicenseKeyActivateTypedDict",
    "LicenseKeyActivationBase",
    "LicenseKeyActivationBaseTypedDict",
    "LicenseKeyActivationRead",
    "LicenseKeyActivationReadMeta",
    "LicenseKeyActivationReadMetaTypedDict",
    "LicenseKeyActivationReadTypedDict",
    "LicenseKeyCustomer",
    "LicenseKeyCustomerTaxID",
    "LicenseKeyCustomerTaxIDTypedDict",
    "LicenseKeyCustomerTypedDict",
    "LicenseKeyDeactivate",
    "LicenseKeyDeactivateTypedDict",
    "LicenseKeyRead",
    "LicenseKeyReadTypedDict",
    "LicenseKeyStatus",
    "LicenseKeyUpdate",
    "LicenseKeyUpdateTypedDict",
    "LicenseKeyValidate",
    "LicenseKeyValidateTypedDict",
    "LicenseKeyWithActivations",
    "LicenseKeyWithActivationsTypedDict",
    "LicenseKeysGetActivationRequest",
    "LicenseKeysGetActivationRequestTypedDict",
    "LicenseKeysGetRequest",
    "LicenseKeysGetRequestTypedDict",
    "LicenseKeysListQueryParamOrganizationIDFilter",
    "LicenseKeysListQueryParamOrganizationIDFilterTypedDict",
    "LicenseKeysListRequest",
    "LicenseKeysListRequestTypedDict",
    "LicenseKeysListResponse",
    "LicenseKeysListResponseTypedDict",
    "LicenseKeysUpdateRequest",
    "LicenseKeysUpdateRequestTypedDict",
    "ListResource",
    "ListResourceBenefit",
    "ListResourceBenefitGrant",
    "ListResourceBenefitGrantTypedDict",
    "ListResourceBenefitTypedDict",
    "ListResourceCheckout",
    "ListResourceCheckoutLink",
    "ListResourceCheckoutLinkTypedDict",
    "ListResourceCheckoutTypedDict",
    "ListResourceCustomField",
    "ListResourceCustomFieldTypedDict",
    "ListResourceCustomerBenefitGrant",
    "ListResourceCustomerBenefitGrantTypedDict",
    "ListResourceCustomerCustomerMeter",
    "ListResourceCustomerCustomerMeterTypedDict",
    "ListResourceCustomerMeter",
    "ListResourceCustomerMeterTypedDict",
    "ListResourceCustomerOrder",
    "ListResourceCustomerOrderTypedDict",
    "ListResourceCustomerPaymentMethod",
    "ListResourceCustomerPaymentMethodTypedDict",
    "ListResourceCustomerSubscription",
    "ListResourceCustomerSubscriptionTypedDict",
    "ListResourceCustomerWallet",
    "ListResourceCustomerWalletTypedDict",
    "ListResourceCustomerWithMembers",
    "ListResourceCustomerWithMembersTypedDict",
    "ListResourceDiscount",
    "ListResourceDiscountTypedDict",
    "ListResourceDispute",
    "ListResourceDisputeTypedDict",
    "ListResourceDownloadableRead",
    "ListResourceDownloadableReadTypedDict",
    "ListResourceEvent",
    "ListResourceEventName",
    "ListResourceEventNameTypedDict",
    "ListResourceEventTypeWithStats",
    "ListResourceEventTypeWithStatsTypedDict",
    "ListResourceEventTypedDict",
    "ListResourceFileRead",
    "ListResourceFileReadTypedDict",
    "ListResourceLicenseKeyRead",
    "ListResourceLicenseKeyReadTypedDict",
    "ListResourceMember",
    "ListResourceMemberTypedDict",
    "ListResourceMeter",
    "ListResourceMeterTypedDict",
    "ListResourceOrder",
    "ListResourceOrderTypedDict",
    "ListResourceOrganization",
    "ListResourceOrganizationTypedDict",
    "ListResourceProduct",
    "ListResourceProductTypedDict",
    "ListResourceRefund",
    "ListResourceRefundTypedDict",
    "ListResourceSubscription",
    "ListResourceSubscriptionTypedDict",
    "ListResourceTypedDict",
    "ListResourceWebhookDelivery",
    "ListResourceWebhookDeliveryTypedDict",
    "ListResourceWebhookEndpoint",
    "ListResourceWebhookEndpointTypedDict",
    "ListResourceWithCursorPaginationEvent",
    "ListResourceWithCursorPaginationEventTypedDict",
    "Loc",
    "LocTypedDict",
    "Ltv",
    "LtvTypedDict",
    "Member",
    "MemberCreate",
    "MemberCreateTypedDict",
    "MemberIDFilter",
    "MemberIDFilterTypedDict",
    "MemberRole",
    "MemberSession",
    "MemberSessionCreate",
    "MemberSessionCreateTypedDict",
    "MemberSessionTypedDict",
    "MemberSortProperty",
    "MemberTypedDict",
    "MemberUpdate",
    "MemberUpdateTypedDict",
    "MembersDeleteMemberRequest",
    "MembersDeleteMemberRequestTypedDict",
    "MembersGetMemberRequest",
    "MembersGetMemberRequestTypedDict",
    "MembersListMembersRequest",
    "MembersListMembersRequestTypedDict",
    "MembersListMembersResponse",
    "MembersListMembersResponseTypedDict",
    "MembersUpdateMemberRequest",
    "MembersUpdateMemberRequestTypedDict",
    "Meta",
    "MetaTypedDict",
    "Metadata",
    "MetadataOutputType",
    "MetadataOutputTypeTypedDict",
    "MetadataQuery",
    "MetadataQueryTypedDict",
    "MetadataTypedDict",
    "Meter",
    "MeterAggregation",
    "MeterAggregationTypedDict",
    "MeterCreate",
    "MeterCreateAggregation",
    "MeterCreateAggregationTypedDict",
    "MeterCreateMetadata",
    "MeterCreateMetadataTypedDict",
    "MeterCreateTypedDict",
    "MeterCreditEvent",
    "MeterCreditEventTypedDict",
    "MeterCreditedMetadata",
    "MeterCreditedMetadataTypedDict",
    "MeterIDFilter",
    "MeterIDFilterTypedDict",
    "MeterQuantities",
    "MeterQuantitiesTypedDict",
    "MeterQuantity",
    "MeterQuantityTypedDict",
    "MeterResetEvent",
    "MeterResetEventTypedDict",
    "MeterResetMetadata",
    "MeterResetMetadataTypedDict",
    "MeterSortProperty",
    "MeterTypedDict",
    "MeterUpdate",
    "MeterUpdateMetadata",
    "MeterUpdateMetadataTypedDict",
    "MeterUpdateTypedDict",
    "MetersGetRequest",
    "MetersGetRequestTypedDict",
    "MetersListQueryParamOrganizationIDFilter",
    "MetersListQueryParamOrganizationIDFilterTypedDict",
    "MetersListRequest",
    "MetersListRequestTypedDict",
    "MetersListResponse",
    "MetersListResponseTypedDict",
    "MetersQuantitiesQueryParamCustomerIDFilter",
    "MetersQuantitiesQueryParamCustomerIDFilterTypedDict",
    "MetersQuantitiesQueryParamExternalCustomerIDFilter",
    "MetersQuantitiesQueryParamExternalCustomerIDFilterTypedDict",
    "MetersQuantitiesRequest",
    "MetersQuantitiesRequestTypedDict",
    "MetersUpdateRequest",
    "MetersUpdateRequestTypedDict",
    "MethodFilter",
    "MethodFilterTypedDict",
    "Metric",
    "MetricPeriod",
    "MetricPeriodTypedDict",
    "MetricType",
    "MetricTypedDict",
    "Metrics",
    "MetricsGetQueryParamCustomerIDFilter",
    "MetricsGetQueryParamCustomerIDFilterTypedDict",
    "MetricsGetQueryParamOrganizationIDFilter",
    "MetricsGetQueryParamOrganizationIDFilterTypedDict",
    "MetricsGetQueryParamProductIDFilter",
    "MetricsGetQueryParamProductIDFilterTypedDict",
    "MetricsGetRequest",
    "MetricsGetRequestTypedDict",
    "MetricsIntervalLimit",
    "MetricsIntervalLimitTypedDict",
    "MetricsIntervalsLimits",
    "MetricsIntervalsLimitsTypedDict",
    "MetricsLimits",
    "MetricsLimitsTypedDict",
    "MetricsResponse",
    "MetricsResponseTypedDict",
    "MetricsTotals",
    "MetricsTotalsActiveSubscriptions",
    "MetricsTotalsActiveSubscriptionsTypedDict",
    "MetricsTotalsActiveUserByEvent",
    "MetricsTotalsActiveUserByEventTypedDict",
    "MetricsTotalsAverageOrderValue",
    "MetricsTotalsAverageOrderValueTypedDict",
    "MetricsTotalsAverageRevenuePerUser",
    "MetricsTotalsAverageRevenuePerUserTypedDict",
    "MetricsTotalsCanceledSubscriptions",
    "MetricsTotalsCanceledSubscriptionsCustomerService",
    "MetricsTotalsCanceledSubscriptionsCustomerServiceTypedDict",
    "MetricsTotalsCanceledSubscriptionsLowQuality",
    "MetricsTotalsCanceledSubscriptionsLowQualityTypedDict",
    "MetricsTotalsCanceledSubscriptionsMissingFeatures",
    "MetricsTotalsCanceledSubscriptionsMissingFeaturesTypedDict",
    "MetricsTotalsCanceledSubscriptionsOther",
    "MetricsTotalsCanceledSubscriptionsOtherTypedDict",
    "MetricsTotalsCanceledSubscriptionsSwitchedService",
    "MetricsTotalsCanceledSubscriptionsSwitchedServiceTypedDict",
    "MetricsTotalsCanceledSubscriptionsTooComplex",
    "MetricsTotalsCanceledSubscriptionsTooComplexTypedDict",
    "MetricsTotalsCanceledSubscriptionsTooExpensive",
    "MetricsTotalsCanceledSubscriptionsTooExpensiveTypedDict",
    "MetricsTotalsCanceledSubscriptionsTypedDict",
    "MetricsTotalsCanceledSubscriptionsUnused",
    "MetricsTotalsCanceledSubscriptionsUnusedTypedDict",
    "MetricsTotalsCashflow",
    "MetricsTotalsCashflowTypedDict",
    "MetricsTotalsCheckouts",
    "MetricsTotalsCheckoutsConversion",
    "MetricsTotalsCheckoutsConversionTypedDict",
    "MetricsTotalsCheckoutsTypedDict",
    "MetricsTotalsChurnRate",
    "MetricsTotalsChurnRateTypedDict",
    "MetricsTotalsChurnedSubscriptions",
    "MetricsTotalsChurnedSubscriptionsTypedDict",
    "MetricsTotalsCommittedMonthlyRecurringRevenue",
    "MetricsTotalsCommittedMonthlyRecurringRevenueTypedDict",
    "MetricsTotalsCommittedSubscriptions",
    "MetricsTotalsCommittedSubscriptionsTypedDict",
    "MetricsTotalsCostPerUser",
    "MetricsTotalsCostPerUserTypedDict",
    "MetricsTotalsCosts",
    "MetricsTotalsCostsTypedDict",
    "MetricsTotalsCumulativeCosts",
    "MetricsTotalsCumulativeCostsTypedDict",
    "MetricsTotalsCumulativeRevenue",
    "MetricsTotalsCumulativeRevenueTypedDict",
    "MetricsTotalsGrossMargin",
    "MetricsTotalsGrossMarginPercentage",
    "MetricsTotalsGrossMarginPercentageTypedDict",
    "MetricsTotalsGrossMarginTypedDict",
    "MetricsTotalsLtv",
    "MetricsTotalsLtvTypedDict",
    "MetricsTotalsMonthlyRecurringRevenue",
    "MetricsTotalsMonthlyRecurringRevenueTypedDict",
    "MetricsTotalsNetAverageOrderValue",
    "MetricsTotalsNetAverageOrderValueTypedDict",
    "MetricsTotalsNetCumulativeRevenue",
    "MetricsTotalsNetCumulativeRevenueTypedDict",
    "MetricsTotalsNetRevenue",
    "MetricsTotalsNetRevenueTypedDict",
    "MetricsTotalsNewSubscriptions",
    "MetricsTotalsNewSubscriptionsNetRevenue",
    "MetricsTotalsNewSubscriptionsNetRevenueTypedDict",
    "MetricsTotalsNewSubscriptionsRevenue",
    "MetricsTotalsNewSubscriptionsRevenueTypedDict",
    "MetricsTotalsNewSubscriptionsTypedDict",
    "MetricsTotalsOneTimeProducts",
    "MetricsTotalsOneTimeProductsNetRevenue",
    "MetricsTotalsOneTimeProductsNetRevenueTypedDict",
    "MetricsTotalsOneTimeProductsRevenue",
    "MetricsTotalsOneTimeProductsRevenueTypedDict",
    "MetricsTotalsOneTimeProductsTypedDict",
    "MetricsTotalsOrders",
    "MetricsTotalsOrdersTypedDict",
    "MetricsTotalsRenewedSubscriptions",
    "MetricsTotalsRenewedSubscriptionsNetRevenue",
    "MetricsTotalsRenewedSubscriptionsNetRevenueTypedDict",
    "MetricsTotalsRenewedSubscriptionsRevenue",
    "MetricsTotalsRenewedSubscriptionsRevenueTypedDict",
    "MetricsTotalsRenewedSubscriptionsTypedDict",
    "MetricsTotalsRevenue",
    "MetricsTotalsRevenueTypedDict",
    "MetricsTotalsSucceededCheckouts",
    "MetricsTotalsSucceededCheckoutsTypedDict",
    "MetricsTotalsTypedDict",
    "MetricsTypedDict",
    "MissingInvoiceBillingDetails",
    "MissingInvoiceBillingDetailsData",
    "MonthlyRecurringRevenue",
    "MonthlyRecurringRevenueTypedDict",
    "NameFilter",
    "NameFilterTypedDict",
    "NetAverageOrderValue",
    "NetAverageOrderValueTypedDict",
    "NetCumulativeRevenue",
    "NetCumulativeRevenueTypedDict",
    "NetRevenue",
    "NetRevenueTypedDict",
    "NewSubscriptions",
    "NewSubscriptionsNetRevenue",
    "NewSubscriptionsNetRevenueTypedDict",
    "NewSubscriptionsRevenue",
    "NewSubscriptionsRevenueTypedDict",
    "NewSubscriptionsTypedDict",
    "NoResponseError",
    "NotOpenCheckout",
    "NotOpenCheckoutData",
    "NotPaidOrder",
    "NotPaidOrderData",
    "NotPermitted",
    "NotPermittedData",
    "OAuth2ClientConfiguration",
    "OAuth2ClientConfigurationTypedDict",
    "OAuth2ClientConfigurationUpdate",
    "OAuth2ClientConfigurationUpdateGrantTypes",
    "OAuth2ClientConfigurationUpdateTokenEndpointAuthMethod",
    "OAuth2ClientConfigurationUpdateTypedDict",
    "OAuth2ClientPublic",
    "OAuth2ClientPublicTypedDict",
    "Oauth2AuthorizeResponseOauth2Authorize",
    "Oauth2AuthorizeResponseOauth2AuthorizeTypedDict",
    "Oauth2ClientsOauth2DeleteClientRequest",
    "Oauth2ClientsOauth2DeleteClientRequestTypedDict",
    "Oauth2ClientsOauth2GetClientRequest",
    "Oauth2ClientsOauth2GetClientRequestTypedDict",
    "Oauth2ClientsOauth2UpdateClientRequest",
    "Oauth2ClientsOauth2UpdateClientRequestTypedDict",
    "Oauth2RequestTokenRequestBody",
    "Oauth2RequestTokenRequestBodyTypedDict",
    "Oauth2UserinfoResponseOauth2Userinfo",
    "Oauth2UserinfoResponseOauth2UserinfoTypedDict",
    "OneTimeProducts",
    "OneTimeProductsNetRevenue",
    "OneTimeProductsNetRevenueTypedDict",
    "OneTimeProductsRevenue",
    "OneTimeProductsRevenueTypedDict",
    "OneTimeProductsTypedDict",
    "Order",
    "OrderBillingReason",
    "OrderCustomFieldData",
    "OrderCustomFieldDataTypedDict",
    "OrderCustomer",
    "OrderCustomerTaxID",
    "OrderCustomerTaxIDTypedDict",
    "OrderCustomerTypedDict",
    "OrderDiscount",
    "OrderDiscountTypedDict",
    "OrderIDFilter",
    "OrderIDFilterTypedDict",
    "OrderInvoice",
    "OrderInvoiceTypedDict",
    "OrderItemSchema",
    "OrderItemSchemaTypedDict",
    "OrderNotEligibleForRetry",
    "OrderNotEligibleForRetryData",
    "OrderPaidEvent",
    "OrderPaidEventTypedDict",
    "OrderPaidMetadata",
    "OrderPaidMetadataTypedDict",
    "OrderProduct",
    "OrderProductTypedDict",
    "OrderRefundedEvent",
    "OrderRefundedEventTypedDict",
    "OrderRefundedMetadata",
    "OrderRefundedMetadataTypedDict",
    "OrderSortProperty",
    "OrderStatus",
    "OrderSubscription",
    "OrderSubscriptionTypedDict",
    "OrderTypedDict",
    "OrderUpdate",
    "OrderUpdateTypedDict",
    "OrdersExportQueryParamOrganizationIDFilter",
    "OrdersExportQueryParamOrganizationIDFilterTypedDict",
    "OrdersExportQueryParamProductIDFilter",
    "OrdersExportQueryParamProductIDFilterTypedDict",
    "OrdersExportRequest",
    "OrdersExportRequestTypedDict",
    "OrdersGenerateInvoiceRequest",
    "OrdersGenerateInvoiceRequestTypedDict",
    "OrdersGenerateInvoiceResponse422OrdersGenerateInvoice",
    "OrdersGenerateInvoiceResponse422OrdersGenerateInvoiceUnion",
    "OrdersGetRequest",
    "OrdersGetRequestTypedDict",
    "OrdersInvoiceRequest",
    "OrdersInvoiceRequestTypedDict",
    "OrdersListQueryParamCustomerIDFilter",
    "OrdersListQueryParamCustomerIDFilterTypedDict",
    "OrdersListQueryParamOrganizationIDFilter",
    "OrdersListQueryParamOrganizationIDFilterTypedDict",
    "OrdersListQueryParamProductIDFilter",
    "OrdersListQueryParamProductIDFilterTypedDict",
    "OrdersListRequest",
    "OrdersListRequestTypedDict",
    "OrdersListResponse",
    "OrdersListResponseTypedDict",
    "OrdersModel",
    "OrdersModelTypedDict",
    "OrdersUpdateRequest",
    "OrdersUpdateRequestTypedDict",
    "Organization",
    "OrganizationAvatarFileCreate",
    "OrganizationAvatarFileCreateTypedDict",
    "OrganizationAvatarFileRead",
    "OrganizationAvatarFileReadTypedDict",
    "OrganizationCreate",
    "OrganizationCreateTypedDict",
    "OrganizationCustomerEmailSettings",
    "OrganizationCustomerEmailSettingsTypedDict",
    "OrganizationCustomerPortalSettings",
    "OrganizationCustomerPortalSettingsTypedDict",
    "OrganizationDetails",
    "OrganizationDetailsTypedDict",
    "OrganizationFeatureSettings",
    "OrganizationFeatureSettingsTypedDict",
    "OrganizationID",
    "OrganizationIDFilter",
    "OrganizationIDFilterTypedDict",
    "OrganizationIDTypedDict",
    "OrganizationNotificationSettings",
    "OrganizationNotificationSettingsTypedDict",
    "OrganizationSocialLink",
    "OrganizationSocialLinkTypedDict",
    "OrganizationSocialPlatforms",
    "OrganizationSortProperty",
    "OrganizationStatus",
    "OrganizationSubscriptionSettings",
    "OrganizationSubscriptionSettingsTypedDict",
    "OrganizationTypedDict",
    "OrganizationUpdate",
    "OrganizationUpdateTypedDict",
    "OrganizationsGetRequest",
    "OrganizationsGetRequestTypedDict",
    "OrganizationsListRequest",
    "OrganizationsListRequestTypedDict",
    "OrganizationsListResponse",
    "OrganizationsListResponseTypedDict",
    "OrganizationsUpdateRequest",
    "OrganizationsUpdateRequestTypedDict",
    "OwnerCreate",
    "OwnerCreateTypedDict",
    "Pagination",
    "PaginationTypedDict",
    "Payment",
    "PaymentAlreadyInProgress",
    "PaymentAlreadyInProgressData",
    "PaymentError",
    "PaymentErrorData",
    "PaymentMethodCard",
    "PaymentMethodCardMetadata",
    "PaymentMethodCardMetadataTypedDict",
    "PaymentMethodCardTypedDict",
    "PaymentMethodGeneric",
    "PaymentMethodGenericTypedDict",
    "PaymentMethodInUseByActiveSubscription",
    "PaymentMethodInUseByActiveSubscriptionData",
    "PaymentNotReady",
    "PaymentNotReadyData",
    "PaymentProcessor",
    "PaymentSortProperty",
    "PaymentStatus",
    "PaymentTypedDict",
    "PaymentsGetRequest",
    "PaymentsGetRequestTypedDict",
    "PaymentsListQueryParamCheckoutIDFilter",
    "PaymentsListQueryParamCheckoutIDFilterTypedDict",
    "PaymentsListQueryParamOrderIDFilter",
    "PaymentsListQueryParamOrderIDFilterTypedDict",
    "PaymentsListQueryParamOrganizationIDFilter",
    "PaymentsListQueryParamOrganizationIDFilterTypedDict",
    "PaymentsListQueryParamStatusFilter",
    "PaymentsListQueryParamStatusFilterTypedDict",
    "PaymentsListRequest",
    "PaymentsListRequestTypedDict",
    "PaymentsListResponse",
    "PaymentsListResponseTypedDict",
    "Permission",
    "PolarError",
    "PortalAuthenticatedUser",
    "PortalAuthenticatedUserTypedDict",
    "PresentmentCurrency",
    "Prices",
    "PricesTypedDict",
    "Product",
    "ProductBenefitsUpdate",
    "ProductBenefitsUpdateTypedDict",
    "ProductBillingType",
    "ProductBillingTypeFilter",
    "ProductBillingTypeFilterTypedDict",
    "ProductCreate",
    "ProductCreateOneTime",
    "ProductCreateOneTimeMetadata",
    "ProductCreateOneTimeMetadataTypedDict",
    "ProductCreateOneTimePrices",
    "ProductCreateOneTimePricesTypedDict",
    "ProductCreateOneTimeTypedDict",
    "ProductCreateRecurring",
    "ProductCreateRecurringMetadata",
    "ProductCreateRecurringMetadataTypedDict",
    "ProductCreateRecurringPrices",
    "ProductCreateRecurringPricesTypedDict",
    "ProductCreateRecurringTypedDict",
    "ProductCreateTypedDict",
    "ProductIDFilter",
    "ProductIDFilterTypedDict",
    "ProductMediaFileCreate",
    "ProductMediaFileCreateTypedDict",
    "ProductMediaFileRead",
    "ProductMediaFileReadTypedDict",
    "ProductPrice",
    "ProductPriceCustom",
    "ProductPriceCustomCreate",
    "ProductPriceCustomCreateTypedDict",
    "ProductPriceCustomTypedDict",
    "ProductPriceFixed",
    "ProductPriceFixedCreate",
    "ProductPriceFixedCreateTypedDict",
    "ProductPriceFixedTypedDict",
    "ProductPriceFree",
    "ProductPriceFreeCreate",
    "ProductPriceFreeCreateTypedDict",
    "ProductPriceFreeTypedDict",
    "ProductPriceMeter",
    "ProductPriceMeterTypedDict",
    "ProductPriceMeteredUnit",
    "ProductPriceMeteredUnitCreate",
    "ProductPriceMeteredUnitCreateTypedDict",
    "ProductPriceMeteredUnitTypedDict",
    "ProductPriceSeatBased",
    "ProductPriceSeatBasedCreate",
    "ProductPriceSeatBasedCreateTypedDict",
    "ProductPriceSeatBasedTypedDict",
    "ProductPriceSeatTier",
    "ProductPriceSeatTierTypedDict",
    "ProductPriceSeatTiersInput",
    "ProductPriceSeatTiersInputTypedDict",
    "ProductPriceSeatTiersOutput",
    "ProductPriceSeatTiersOutputTypedDict",
    "ProductPriceSource",
    "ProductPriceType",
    "ProductPriceTypedDict",
    "ProductSortProperty",
    "ProductTypedDict",
    "ProductUpdate",
    "ProductUpdateMetadata",
    "ProductUpdateMetadataTypedDict",
    "ProductUpdatePrices",
    "ProductUpdatePricesTypedDict",
    "ProductUpdateTypedDict",
    "ProductsGetRequest",
    "ProductsGetRequestTypedDict",
    "ProductsListQueryParamOrganizationIDFilter",
    "ProductsListQueryParamOrganizationIDFilterTypedDict",
    "ProductsListRequest",
    "ProductsListRequestTypedDict",
    "ProductsListResponse",
    "ProductsListResponseTypedDict",
    "ProductsUpdateBenefitsRequest",
    "ProductsUpdateBenefitsRequestTypedDict",
    "ProductsUpdateRequest",
    "ProductsUpdateRequestTypedDict",
    "Properties",
    "PropertiesTypedDict",
    "PropertyAggregation",
    "PropertyAggregationTypedDict",
    "QueryParamBenefitIDFilter",
    "QueryParamBenefitIDFilterTypedDict",
    "QueryParamBenefitTypeFilter",
    "QueryParamBenefitTypeFilterTypedDict",
    "QueryParamCheckoutIDFilter",
    "QueryParamCheckoutIDFilterTypedDict",
    "QueryParamCustomerIDFilter",
    "QueryParamCustomerIDFilterTypedDict",
    "QueryParamDiscountIDFilter",
    "QueryParamDiscountIDFilterTypedDict",
    "QueryParamExternalCustomerIDFilter",
    "QueryParamExternalCustomerIDFilterTypedDict",
    "QueryParamMemberIDFilter",
    "QueryParamMemberIDFilterTypedDict",
    "QueryParamMeterIDFilter",
    "QueryParamMeterIDFilterTypedDict",
    "QueryParamOrderIDFilter",
    "QueryParamOrderIDFilterTypedDict",
    "QueryParamOrganizationID",
    "QueryParamOrganizationIDFilter",
    "QueryParamOrganizationIDFilterTypedDict",
    "QueryParamOrganizationIDTypedDict",
    "QueryParamProductBillingTypeFilter",
    "QueryParamProductBillingTypeFilterTypedDict",
    "QueryParamProductIDFilter",
    "QueryParamProductIDFilterTypedDict",
    "QueryParamSourceFilter",
    "QueryParamSourceFilterTypedDict",
    "QueryParamStatusFilter",
    "QueryParamStatusFilterTypedDict",
    "QueryParamSubscriptionIDFilter",
    "QueryParamSubscriptionIDFilterTypedDict",
    "RefreshTokenRequest",
    "RefreshTokenRequestTypedDict",
    "Refund",
    "RefundCreate",
    "RefundCreateMetadata",
    "RefundCreateMetadataTypedDict",
    "RefundCreateTypedDict",
    "RefundDispute",
    "RefundDisputeTypedDict",
    "RefundIDFilter",
    "RefundIDFilterTypedDict",
    "RefundReason",
    "RefundSortProperty",
    "RefundStatus",
    "RefundTypedDict",
    "RefundedAlready",
    "RefundedAlreadyData",
    "RefundsListQueryParamCustomerIDFilter",
    "RefundsListQueryParamCustomerIDFilterTypedDict",
    "RefundsListQueryParamOrganizationIDFilter",
    "RefundsListQueryParamOrganizationIDFilterTypedDict",
    "RefundsListRequest",
    "RefundsListRequestTypedDict",
    "RefundsListResponse",
    "RefundsListResponseTypedDict",
    "RenewedSubscriptions",
    "RenewedSubscriptionsNetRevenue",
    "RenewedSubscriptionsNetRevenueTypedDict",
    "RenewedSubscriptionsRevenue",
    "RenewedSubscriptionsRevenueTypedDict",
    "RenewedSubscriptionsTypedDict",
    "ResourceNotFound",
    "ResourceNotFoundData",
    "ResponseValidationError",
    "Revenue",
    "RevenueTypedDict",
    "RevokeTokenRequest",
    "RevokeTokenRequestTypedDict",
    "RevokeTokenResponse",
    "RevokeTokenResponseTypedDict",
    "S3DownloadURL",
    "S3DownloadURLTypedDict",
    "S3FileCreateMultipart",
    "S3FileCreateMultipartTypedDict",
    "S3FileCreatePart",
    "S3FileCreatePartTypedDict",
    "S3FileUploadCompletedPart",
    "S3FileUploadCompletedPartTypedDict",
    "S3FileUploadMultipart",
    "S3FileUploadMultipartTypedDict",
    "S3FileUploadPart",
    "S3FileUploadPartTypedDict",
    "SDKError",
    "Scope",
    "SeatAssign",
    "SeatAssignTypedDict",
    "SeatClaim",
    "SeatClaimInfo",
    "SeatClaimInfoTypedDict",
    "SeatClaimTypedDict",
    "SeatStatus",
    "SeatsList",
    "SeatsListTypedDict",
    "Security",
    "SecurityTypedDict",
    "SourceFilter",
    "SourceFilterTypedDict",
    "Status",
    "StatusFilter",
    "StatusFilterTypedDict",
    "SubType",
    "Subscription",
    "SubscriptionBillingPeriodUpdatedEvent",
    "SubscriptionBillingPeriodUpdatedEventTypedDict",
    "SubscriptionBillingPeriodUpdatedMetadata",
    "SubscriptionBillingPeriodUpdatedMetadataTypedDict",
    "SubscriptionCancel",
    "SubscriptionCancelTypedDict",
    "SubscriptionCanceledEvent",
    "SubscriptionCanceledEventTypedDict",
    "SubscriptionCanceledMetadata",
    "SubscriptionCanceledMetadataTypedDict",
    "SubscriptionCreateCustomer",
    "SubscriptionCreateCustomerTypedDict",
    "SubscriptionCreateExternalCustomer",
    "SubscriptionCreateExternalCustomerMetadata",
    "SubscriptionCreateExternalCustomerMetadataTypedDict",
    "SubscriptionCreateExternalCustomerTypedDict",
    "SubscriptionCreatedEvent",
    "SubscriptionCreatedEventTypedDict",
    "SubscriptionCreatedMetadata",
    "SubscriptionCreatedMetadataTypedDict",
    "SubscriptionCustomer",
    "SubscriptionCustomerTypedDict",
    "SubscriptionCycledEvent",
    "SubscriptionCycledEventTypedDict",
    "SubscriptionCycledMetadata",
    "SubscriptionCycledMetadataTypedDict",
    "SubscriptionDiscount",
    "SubscriptionDiscountTypedDict",
    "SubscriptionIDFilter",
    "SubscriptionIDFilterTypedDict",
    "SubscriptionLocked",
    "SubscriptionLockedData",
    "SubscriptionMeter",
    "SubscriptionMeterTypedDict",
    "SubscriptionPrices",
    "SubscriptionPricesTypedDict",
    "SubscriptionProductUpdatedEvent",
    "SubscriptionProductUpdatedEventTypedDict",
    "SubscriptionProductUpdatedMetadata",
    "SubscriptionProductUpdatedMetadataTypedDict",
    "SubscriptionProrationBehavior",
    "SubscriptionRecurringInterval",
    "SubscriptionRevoke",
    "SubscriptionRevokeTypedDict",
    "SubscriptionRevokedEvent",
    "SubscriptionRevokedEventTypedDict",
    "SubscriptionRevokedMetadata",
    "SubscriptionRevokedMetadataTypedDict",
    "SubscriptionSeatsUpdatedEvent",
    "SubscriptionSeatsUpdatedEventTypedDict",
    "SubscriptionSeatsUpdatedMetadata",
    "SubscriptionSeatsUpdatedMetadataTypedDict",
    "SubscriptionSortProperty",
    "SubscriptionStatus",
    "SubscriptionTypedDict",
    "SubscriptionUncanceledEvent",
    "SubscriptionUncanceledEventTypedDict",
    "SubscriptionUncanceledMetadata",
    "SubscriptionUncanceledMetadataTypedDict",
    "SubscriptionUpdate",
    "SubscriptionUpdateBillingPeriod",
    "SubscriptionUpdateBillingPeriodTypedDict",
    "SubscriptionUpdateDiscount",
    "SubscriptionUpdateDiscountTypedDict",
    "SubscriptionUpdateProduct",
    "SubscriptionUpdateProductTypedDict",
    "SubscriptionUpdateSeats",
    "SubscriptionUpdateSeatsTypedDict",
    "SubscriptionUpdateTrial",
    "SubscriptionUpdateTrialTypedDict",
    "SubscriptionUpdateTypedDict",
    "SubscriptionsCreateSubscriptionCreate",
    "SubscriptionsCreateSubscriptionCreateTypedDict",
    "SubscriptionsExportRequest",
    "SubscriptionsExportRequestTypedDict",
    "SubscriptionsGetRequest",
    "SubscriptionsGetRequestTypedDict",
    "SubscriptionsListRequest",
    "SubscriptionsListRequestTypedDict",
    "SubscriptionsListResponse",
    "SubscriptionsListResponseTypedDict",
    "SubscriptionsRevokeRequest",
    "SubscriptionsRevokeRequestTypedDict",
    "SubscriptionsUpdateRequest",
    "SubscriptionsUpdateRequestTypedDict",
    "SucceededCheckouts",
    "SucceededCheckoutsTypedDict",
    "SwitchingFrom",
    "SystemEvent",
    "SystemEventTypedDict",
    "TaxID",
    "TaxIDFormat",
    "TaxIDTypedDict",
    "TimeInterval",
    "Timeframe",
    "TokenEndpointAuthMethod",
    "TokenResponse",
    "TokenResponseTypedDict",
    "TokenType",
    "TokenTypeHint",
    "TrialAlreadyRedeemed",
    "TrialAlreadyRedeemedData",
    "TrialInterval",
    "Two",
    "TwoTypedDict",
    "Unauthorized",
    "UnauthorizedData",
    "UniqueAggregation",
    "UniqueAggregationTypedDict",
    "UnitAmount",
    "UnitAmountTypedDict",
    "UserEvent",
    "UserEventTypedDict",
    "UserInfoOrganization",
    "UserInfoOrganizationTypedDict",
    "UserInfoUser",
    "UserInfoUserTypedDict",
    "ValidatedLicenseKey",
    "ValidatedLicenseKeyTypedDict",
    "ValidationError",
    "ValidationErrorTypedDict",
    "Value",
    "ValueTypedDict",
    "WebTokenRequest",
    "WebTokenRequestSubType",
    "WebTokenRequestTypedDict",
    "WebhookBenefitCreatedPayload",
    "WebhookBenefitCreatedPayloadTypedDict",
    "WebhookBenefitGrantCreatedPayload",
    "WebhookBenefitGrantCreatedPayloadTypedDict",
    "WebhookBenefitGrantCycledPayload",
    "WebhookBenefitGrantCycledPayloadTypedDict",
    "WebhookBenefitGrantRevokedPayload",
    "WebhookBenefitGrantRevokedPayloadTypedDict",
    "WebhookBenefitGrantUpdatedPayload",
    "WebhookBenefitGrantUpdatedPayloadTypedDict",
    "WebhookBenefitUpdatedPayload",
    "WebhookBenefitUpdatedPayloadTypedDict",
    "WebhookCheckoutCreatedPayload",
    "WebhookCheckoutCreatedPayloadTypedDict",
    "WebhookCheckoutUpdatedPayload",
    "WebhookCheckoutUpdatedPayloadTypedDict",
    "WebhookCustomerCreatedPayload",
    "WebhookCustomerCreatedPayloadTypedDict",
    "WebhookCustomerDeletedPayload",
    "WebhookCustomerDeletedPayloadTypedDict",
    "WebhookCustomerSeatAssignedPayload",
    "WebhookCustomerSeatAssignedPayloadTypedDict",
    "WebhookCustomerSeatClaimedPayload",
    "WebhookCustomerSeatClaimedPayloadTypedDict",
    "WebhookCustomerSeatRevokedPayload",
    "WebhookCustomerSeatRevokedPayloadTypedDict",
    "WebhookCustomerStateChangedPayload",
    "WebhookCustomerStateChangedPayloadTypedDict",
    "WebhookCustomerUpdatedPayload",
    "WebhookCustomerUpdatedPayloadTypedDict",
    "WebhookDelivery",
    "WebhookDeliveryTypedDict",
    "WebhookEndpoint",
    "WebhookEndpointCreate",
    "WebhookEndpointCreateTypedDict",
    "WebhookEndpointTypedDict",
    "WebhookEndpointUpdate",
    "WebhookEndpointUpdateTypedDict",
    "WebhookEvent",
    "WebhookEventType",
    "WebhookEventTypedDict",
    "WebhookFormat",
    "WebhookOrderCreatedPayload",
    "WebhookOrderCreatedPayloadTypedDict",
    "WebhookOrderPaidPayload",
    "WebhookOrderPaidPayloadTypedDict",
    "WebhookOrderRefundedPayload",
    "WebhookOrderRefundedPayloadTypedDict",
    "WebhookOrderUpdatedPayload",
    "WebhookOrderUpdatedPayloadTypedDict",
    "WebhookOrganizationUpdatedPayload",
    "WebhookOrganizationUpdatedPayloadTypedDict",
    "WebhookProductCreatedPayload",
    "WebhookProductCreatedPayloadTypedDict",
    "WebhookProductUpdatedPayload",
    "WebhookProductUpdatedPayloadTypedDict",
    "WebhookRefundCreatedPayload",
    "WebhookRefundCreatedPayloadTypedDict",
    "WebhookRefundUpdatedPayload",
    "WebhookRefundUpdatedPayloadTypedDict",
    "WebhookSubscriptionActivePayload",
    "WebhookSubscriptionActivePayloadTypedDict",
    "WebhookSubscriptionCanceledPayload",
    "WebhookSubscriptionCanceledPayloadTypedDict",
    "WebhookSubscriptionCreatedPayload",
    "WebhookSubscriptionCreatedPayloadTypedDict",
    "WebhookSubscriptionPastDuePayload",
    "WebhookSubscriptionPastDuePayloadTypedDict",
    "WebhookSubscriptionRevokedPayload",
    "WebhookSubscriptionRevokedPayloadTypedDict",
    "WebhookSubscriptionUncanceledPayload",
    "WebhookSubscriptionUncanceledPayloadTypedDict",
    "WebhookSubscriptionUpdatedPayload",
    "WebhookSubscriptionUpdatedPayloadTypedDict",
    "WebhooksDeleteWebhookEndpointRequest",
    "WebhooksDeleteWebhookEndpointRequestTypedDict",
    "WebhooksGetWebhookEndpointRequest",
    "WebhooksGetWebhookEndpointRequestTypedDict",
    "WebhooksListWebhookDeliveriesRequest",
    "WebhooksListWebhookDeliveriesRequestTypedDict",
    "WebhooksListWebhookDeliveriesResponse",
    "WebhooksListWebhookDeliveriesResponseTypedDict",
    "WebhooksListWebhookEndpointsRequest",
    "WebhooksListWebhookEndpointsRequestTypedDict",
    "WebhooksListWebhookEndpointsResponse",
    "WebhooksListWebhookEndpointsResponseTypedDict",
    "WebhooksRedeliverWebhookEventRequest",
    "WebhooksRedeliverWebhookEventRequestTypedDict",
    "WebhooksResetWebhookEndpointSecretRequest",
    "WebhooksResetWebhookEndpointSecretRequestTypedDict",
    "WebhooksUpdateWebhookEndpointRequest",
    "WebhooksUpdateWebhookEndpointRequestTypedDict",
]

_dynamic_imports: dict[str, str] = {
    "Address": ".address",
    "AddressTypedDict": ".address",
    "CountryAlpha2": ".address",
    "AddressDict": ".addressdict",
    "AddressDictTypedDict": ".addressdict",
    "AddressInput": ".addressinput",
    "AddressInputTypedDict": ".addressinput",
    "CountryAlpha2Input": ".addressinput",
    "AggregationFunction": ".aggregationfunction",
    "AlreadyActiveSubscriptionError": ".alreadyactivesubscriptionerror",
    "AlreadyActiveSubscriptionErrorData": ".alreadyactivesubscriptionerror",
    "AlreadyCanceledSubscription": ".alreadycanceledsubscription",
    "AlreadyCanceledSubscriptionData": ".alreadycanceledsubscription",
    "AttachedCustomField": ".attachedcustomfield",
    "AttachedCustomFieldTypedDict": ".attachedcustomfield",
    "AttachedCustomFieldCreate": ".attachedcustomfieldcreate",
    "AttachedCustomFieldCreateTypedDict": ".attachedcustomfieldcreate",
    "AuthorizationCodeTokenRequest": ".authorizationcodetokenrequest",
    "AuthorizationCodeTokenRequestTypedDict": ".authorizationcodetokenrequest",
    "AuthorizeOrganization": ".authorizeorganization",
    "AuthorizeOrganizationTypedDict": ".authorizeorganization",
    "AuthorizeResponseOrganization": ".authorizeresponseorganization",
    "AuthorizeResponseOrganizationTypedDict": ".authorizeresponseorganization",
    "AuthorizeResponseUser": ".authorizeresponseuser",
    "AuthorizeResponseUserTypedDict": ".authorizeresponseuser",
    "AuthorizeUser": ".authorizeuser",
    "AuthorizeUserTypedDict": ".authorizeuser",
    "BalanceDisputeEvent": ".balancedisputeevent",
    "BalanceDisputeEventTypedDict": ".balancedisputeevent",
    "BalanceDisputeMetadata": ".balancedisputemetadata",
    "BalanceDisputeMetadataTypedDict": ".balancedisputemetadata",
    "BalanceDisputeReversalEvent": ".balancedisputereversalevent",
    "BalanceDisputeReversalEventTypedDict": ".balancedisputereversalevent",
    "BalanceOrderEvent": ".balanceorderevent",
    "BalanceOrderEventTypedDict": ".balanceorderevent",
    "BalanceOrderMetadata": ".balanceordermetadata",
    "BalanceOrderMetadataTypedDict": ".balanceordermetadata",
    "BalanceRefundEvent": ".balancerefundevent",
    "BalanceRefundEventTypedDict": ".balancerefundevent",
    "BalanceRefundMetadata": ".balancerefundmetadata",
    "BalanceRefundMetadataTypedDict": ".balancerefundmetadata",
    "BalanceRefundReversalEvent": ".balancerefundreversalevent",
    "BalanceRefundReversalEventTypedDict": ".balancerefundreversalevent",
    "Benefit": ".benefit",
    "BenefitTypedDict": ".benefit",
    "BenefitGrantsListQueryParamCustomerIDFilter": ".benefit_grants_listop",
    "BenefitGrantsListQueryParamCustomerIDFilterTypedDict": ".benefit_grants_listop",
    "BenefitGrantsListQueryParamOrganizationIDFilter": ".benefit_grants_listop",
    "BenefitGrantsListQueryParamOrganizationIDFilterTypedDict": ".benefit_grants_listop",
    "BenefitGrantsListRequest": ".benefit_grants_listop",
    "BenefitGrantsListRequestTypedDict": ".benefit_grants_listop",
    "BenefitGrantsListResponse": ".benefit_grants_listop",
    "BenefitGrantsListResponseTypedDict": ".benefit_grants_listop",
    "BenefitCreate": ".benefitcreate",
    "BenefitCreateTypedDict": ".benefitcreate",
    "BenefitCustom": ".benefitcustom",
    "BenefitCustomTypedDict": ".benefitcustom",
    "BenefitCustomCreate": ".benefitcustomcreate",
    "BenefitCustomCreateMetadata": ".benefitcustomcreate",
    "BenefitCustomCreateMetadataTypedDict": ".benefitcustomcreate",
    "BenefitCustomCreateTypedDict": ".benefitcustomcreate",
    "BenefitCustomCreateProperties": ".benefitcustomcreateproperties",
    "BenefitCustomCreatePropertiesTypedDict": ".benefitcustomcreateproperties",
    "BenefitCustomProperties": ".benefitcustomproperties",
    "BenefitCustomPropertiesTypedDict": ".benefitcustomproperties",
    "BenefitCustomSubscriber": ".benefitcustomsubscriber",
    "BenefitCustomSubscriberTypedDict": ".benefitcustomsubscriber",
    "BenefitCustomSubscriberProperties": ".benefitcustomsubscriberproperties",
    "BenefitCustomSubscriberPropertiesTypedDict": ".benefitcustomsubscriberproperties",
    "BenefitCustomUpdate": ".benefitcustomupdate",
    "BenefitCustomUpdateMetadata": ".benefitcustomupdate",
    "BenefitCustomUpdateMetadataTypedDict": ".benefitcustomupdate",
    "BenefitCustomUpdateTypedDict": ".benefitcustomupdate",
    "BenefitCycledEvent": ".benefitcycledevent",
    "BenefitCycledEventTypedDict": ".benefitcycledevent",
    "BenefitDiscord": ".benefitdiscord",
    "BenefitDiscordTypedDict": ".benefitdiscord",
    "BenefitDiscordCreate": ".benefitdiscordcreate",
    "BenefitDiscordCreateMetadata": ".benefitdiscordcreate",
    "BenefitDiscordCreateMetadataTypedDict": ".benefitdiscordcreate",
    "BenefitDiscordCreateTypedDict": ".benefitdiscordcreate",
    "BenefitDiscordCreateProperties": ".benefitdiscordcreateproperties",
    "BenefitDiscordCreatePropertiesTypedDict": ".benefitdiscordcreateproperties",
    "BenefitDiscordProperties": ".benefitdiscordproperties",
    "BenefitDiscordPropertiesTypedDict": ".benefitdiscordproperties",
    "BenefitDiscordSubscriber": ".benefitdiscordsubscriber",
    "BenefitDiscordSubscriberTypedDict": ".benefitdiscordsubscriber",
    "BenefitDiscordSubscriberProperties": ".benefitdiscordsubscriberproperties",
    "BenefitDiscordSubscriberPropertiesTypedDict": ".benefitdiscordsubscriberproperties",
    "BenefitDiscordUpdate": ".benefitdiscordupdate",
    "BenefitDiscordUpdateMetadata": ".benefitdiscordupdate",
    "BenefitDiscordUpdateMetadataTypedDict": ".benefitdiscordupdate",
    "BenefitDiscordUpdateTypedDict": ".benefitdiscordupdate",
    "BenefitDownloadables": ".benefitdownloadables",
    "BenefitDownloadablesTypedDict": ".benefitdownloadables",
    "BenefitDownloadablesCreate": ".benefitdownloadablescreate",
    "BenefitDownloadablesCreateMetadata": ".benefitdownloadablescreate",
    "BenefitDownloadablesCreateMetadataTypedDict": ".benefitdownloadablescreate",
    "BenefitDownloadablesCreateTypedDict": ".benefitdownloadablescreate",
    "BenefitDownloadablesCreateProperties": ".benefitdownloadablescreateproperties",
    "BenefitDownloadablesCreatePropertiesTypedDict": ".benefitdownloadablescreateproperties",
    "BenefitDownloadablesProperties": ".benefitdownloadablesproperties",
    "BenefitDownloadablesPropertiesTypedDict": ".benefitdownloadablesproperties",
    "BenefitDownloadablesSubscriber": ".benefitdownloadablessubscriber",
    "BenefitDownloadablesSubscriberTypedDict": ".benefitdownloadablessubscriber",
    "BenefitDownloadablesSubscriberProperties": ".benefitdownloadablessubscriberproperties",
    "BenefitDownloadablesSubscriberPropertiesTypedDict": ".benefitdownloadablessubscriberproperties",
    "BenefitDownloadablesUpdate": ".benefitdownloadablesupdate",
    "BenefitDownloadablesUpdateMetadata": ".benefitdownloadablesupdate",
    "BenefitDownloadablesUpdateMetadataTypedDict": ".benefitdownloadablesupdate",
    "BenefitDownloadablesUpdateTypedDict": ".benefitdownloadablesupdate",
    "BenefitGitHubRepository": ".benefitgithubrepository",
    "BenefitGitHubRepositoryTypedDict": ".benefitgithubrepository",
    "BenefitGitHubRepositoryCreate": ".benefitgithubrepositorycreate",
    "BenefitGitHubRepositoryCreateMetadata": ".benefitgithubrepositorycreate",
    "BenefitGitHubRepositoryCreateMetadataTypedDict": ".benefitgithubrepositorycreate",
    "BenefitGitHubRepositoryCreateTypedDict": ".benefitgithubrepositorycreate",
    "BenefitGitHubRepositoryCreateProperties": ".benefitgithubrepositorycreateproperties",
    "BenefitGitHubRepositoryCreatePropertiesPermission": ".benefitgithubrepositorycreateproperties",
    "BenefitGitHubRepositoryCreatePropertiesTypedDict": ".benefitgithubrepositorycreateproperties",
    "BenefitGitHubRepositoryProperties": ".benefitgithubrepositoryproperties",
    "BenefitGitHubRepositoryPropertiesTypedDict": ".benefitgithubrepositoryproperties",
    "Permission": ".benefitgithubrepositoryproperties",
    "BenefitGitHubRepositorySubscriber": ".benefitgithubrepositorysubscriber",
    "BenefitGitHubRepositorySubscriberTypedDict": ".benefitgithubrepositorysubscriber",
    "BenefitGitHubRepositorySubscriberProperties": ".benefitgithubrepositorysubscriberproperties",
    "BenefitGitHubRepositorySubscriberPropertiesTypedDict": ".benefitgithubrepositorysubscriberproperties",
    "BenefitGitHubRepositoryUpdate": ".benefitgithubrepositoryupdate",
    "BenefitGitHubRepositoryUpdateMetadata": ".benefitgithubrepositoryupdate",
    "BenefitGitHubRepositoryUpdateMetadataTypedDict": ".benefitgithubrepositoryupdate",
    "BenefitGitHubRepositoryUpdateTypedDict": ".benefitgithubrepositoryupdate",
    "BenefitGrant": ".benefitgrant",
    "BenefitGrantTypedDict": ".benefitgrant",
    "Properties": ".benefitgrant",
    "PropertiesTypedDict": ".benefitgrant",
    "BenefitGrantCustomProperties": ".benefitgrantcustomproperties",
    "BenefitGrantCustomPropertiesTypedDict": ".benefitgrantcustomproperties",
    "BenefitGrantCustomWebhook": ".benefitgrantcustomwebhook",
    "BenefitGrantCustomWebhookTypedDict": ".benefitgrantcustomwebhook",
    "BenefitGrantDiscordProperties": ".benefitgrantdiscordproperties",
    "BenefitGrantDiscordPropertiesTypedDict": ".benefitgrantdiscordproperties",
    "BenefitGrantDiscordWebhook": ".benefitgrantdiscordwebhook",
    "BenefitGrantDiscordWebhookTypedDict": ".benefitgrantdiscordwebhook",
    "BenefitGrantDownloadablesProperties": ".benefitgrantdownloadablesproperties",
    "BenefitGrantDownloadablesPropertiesTypedDict": ".benefitgrantdownloadablesproperties",
    "BenefitGrantDownloadablesWebhook": ".benefitgrantdownloadableswebhook",
    "BenefitGrantDownloadablesWebhookTypedDict": ".benefitgrantdownloadableswebhook",
    "BenefitGrantedEvent": ".benefitgrantedevent",
    "BenefitGrantedEventTypedDict": ".benefitgrantedevent",
    "BenefitGrantError": ".benefitgranterror",
    "BenefitGrantErrorTypedDict": ".benefitgranterror",
    "BenefitGrantGitHubRepositoryProperties": ".benefitgrantgithubrepositoryproperties",
    "BenefitGrantGitHubRepositoryPropertiesPermission": ".benefitgrantgithubrepositoryproperties",
    "BenefitGrantGitHubRepositoryPropertiesTypedDict": ".benefitgrantgithubrepositoryproperties",
    "BenefitGrantGitHubRepositoryWebhook": ".benefitgrantgithubrepositorywebhook",
    "BenefitGrantGitHubRepositoryWebhookTypedDict": ".benefitgrantgithubrepositorywebhook",
    "BenefitGrantLicenseKeysProperties": ".benefitgrantlicensekeysproperties",
    "BenefitGrantLicenseKeysPropertiesTypedDict": ".benefitgrantlicensekeysproperties",
    "BenefitGrantLicenseKeysWebhook": ".benefitgrantlicensekeyswebhook",
    "BenefitGrantLicenseKeysWebhookTypedDict": ".benefitgrantlicensekeyswebhook",
    "BenefitGrantMetadata": ".benefitgrantmetadata",
    "BenefitGrantMetadataTypedDict": ".benefitgrantmetadata",
    "BenefitGrantMeterCreditProperties": ".benefitgrantmetercreditproperties",
    "BenefitGrantMeterCreditPropertiesTypedDict": ".benefitgrantmetercreditproperties",
    "BenefitGrantMeterCreditWebhook": ".benefitgrantmetercreditwebhook",
    "BenefitGrantMeterCreditWebhookTypedDict": ".benefitgrantmetercreditwebhook",
    "BenefitGrantSortProperty": ".benefitgrantsortproperty",
    "BenefitGrantWebhook": ".benefitgrantwebhook",
    "BenefitGrantWebhookTypedDict": ".benefitgrantwebhook",
    "BenefitLicenseKeyActivationCreateProperties": ".benefitlicensekeyactivationcreateproperties",
    "BenefitLicenseKeyActivationCreatePropertiesTypedDict": ".benefitlicensekeyactivationcreateproperties",
    "BenefitLicenseKeyActivationProperties": ".benefitlicensekeyactivationproperties",
    "BenefitLicenseKeyActivationPropertiesTypedDict": ".benefitlicensekeyactivationproperties",
    "BenefitLicenseKeyExpirationProperties": ".benefitlicensekeyexpirationproperties",
    "BenefitLicenseKeyExpirationPropertiesTypedDict": ".benefitlicensekeyexpirationproperties",
    "Timeframe": ".benefitlicensekeyexpirationproperties",
    "BenefitLicenseKeys": ".benefitlicensekeys",
    "BenefitLicenseKeysTypedDict": ".benefitlicensekeys",
    "BenefitLicenseKeysCreate": ".benefitlicensekeyscreate",
    "BenefitLicenseKeysCreateMetadata": ".benefitlicensekeyscreate",
    "BenefitLicenseKeysCreateMetadataTypedDict": ".benefitlicensekeyscreate",
    "BenefitLicenseKeysCreateTypedDict": ".benefitlicensekeyscreate",
    "BenefitLicenseKeysCreateProperties": ".benefitlicensekeyscreateproperties",
    "BenefitLicenseKeysCreatePropertiesTypedDict": ".benefitlicensekeyscreateproperties",
    "BenefitLicenseKeysProperties": ".benefitlicensekeysproperties",
    "BenefitLicenseKeysPropertiesTypedDict": ".benefitlicensekeysproperties",
    "BenefitLicenseKeysSubscriber": ".benefitlicensekeyssubscriber",
    "BenefitLicenseKeysSubscriberTypedDict": ".benefitlicensekeyssubscriber",
    "BenefitLicenseKeysSubscriberProperties": ".benefitlicensekeyssubscriberproperties",
    "BenefitLicenseKeysSubscriberPropertiesTypedDict": ".benefitlicensekeyssubscriberproperties",
    "BenefitLicenseKeysUpdate": ".benefitlicensekeysupdate",
    "BenefitLicenseKeysUpdateMetadata": ".benefitlicensekeysupdate",
    "BenefitLicenseKeysUpdateMetadataTypedDict": ".benefitlicensekeysupdate",
    "BenefitLicenseKeysUpdateTypedDict": ".benefitlicensekeysupdate",
    "BenefitMeterCredit": ".benefitmetercredit",
    "BenefitMeterCreditTypedDict": ".benefitmetercredit",
    "BenefitMeterCreditCreate": ".benefitmetercreditcreate",
    "BenefitMeterCreditCreateMetadata": ".benefitmetercreditcreate",
    "BenefitMeterCreditCreateMetadataTypedDict": ".benefitmetercreditcreate",
    "BenefitMeterCreditCreateTypedDict": ".benefitmetercreditcreate",
    "BenefitMeterCreditCreateProperties": ".benefitmetercreditcreateproperties",
    "BenefitMeterCreditCreatePropertiesTypedDict": ".benefitmetercreditcreateproperties",
    "BenefitMeterCreditProperties": ".benefitmetercreditproperties",
    "BenefitMeterCreditPropertiesTypedDict": ".benefitmetercreditproperties",
    "BenefitMeterCreditSubscriber": ".benefitmetercreditsubscriber",
    "BenefitMeterCreditSubscriberTypedDict": ".benefitmetercreditsubscriber",
    "BenefitMeterCreditSubscriberProperties": ".benefitmetercreditsubscriberproperties",
    "BenefitMeterCreditSubscriberPropertiesTypedDict": ".benefitmetercreditsubscriberproperties",
    "BenefitMeterCreditUpdate": ".benefitmetercreditupdate",
    "BenefitMeterCreditUpdateMetadata": ".benefitmetercreditupdate",
    "BenefitMeterCreditUpdateMetadataTypedDict": ".benefitmetercreditupdate",
    "BenefitMeterCreditUpdateTypedDict": ".benefitmetercreditupdate",
    "BenefitPublic": ".benefitpublic",
    "BenefitPublicTypedDict": ".benefitpublic",
    "BenefitRevokedEvent": ".benefitrevokedevent",
    "BenefitRevokedEventTypedDict": ".benefitrevokedevent",
    "BenefitsDeleteRequest": ".benefits_deleteop",
    "BenefitsDeleteRequestTypedDict": ".benefits_deleteop",
    "BenefitsGetRequest": ".benefits_getop",
    "BenefitsGetRequestTypedDict": ".benefits_getop",
    "BenefitsGrantsRequest": ".benefits_grantsop",
    "BenefitsGrantsRequestTypedDict": ".benefits_grantsop",
    "BenefitsGrantsResponse": ".benefits_grantsop",
    "BenefitsGrantsResponseTypedDict": ".benefits_grantsop",
    "MemberIDFilter": ".benefits_grantsop",
    "MemberIDFilterTypedDict": ".benefits_grantsop",
    "QueryParamCustomerIDFilter": ".benefits_grantsop",
    "QueryParamCustomerIDFilterTypedDict": ".benefits_grantsop",
    "BenefitsUpdateBenefitUpdate": ".benefits_updateop",
    "BenefitsUpdateBenefitUpdateTypedDict": ".benefits_updateop",
    "BenefitsUpdateRequest": ".benefits_updateop",
    "BenefitsUpdateRequestTypedDict": ".benefits_updateop",
    "BenefitSortProperty": ".benefitsortproperty",
    "BenefitSubscriberOrganization": ".benefitsubscriberorganization",
    "BenefitSubscriberOrganizationTypedDict": ".benefitsubscriberorganization",
    "BenefitType": ".benefittype",
    "BenefitUpdatedEvent": ".benefitupdatedevent",
    "BenefitUpdatedEventTypedDict": ".benefitupdatedevent",
    "BillingAddressFieldMode": ".billingaddressfieldmode",
    "CardPayment": ".cardpayment",
    "CardPaymentTypedDict": ".cardpayment",
    "CardPaymentMetadata": ".cardpaymentmetadata",
    "CardPaymentMetadataTypedDict": ".cardpaymentmetadata",
    "Checkout": ".checkout",
    "CheckoutCustomFieldData": ".checkout",
    "CheckoutCustomFieldDataTypedDict": ".checkout",
    "CheckoutDiscount": ".checkout",
    "CheckoutDiscountTypedDict": ".checkout",
    "CheckoutPrices": ".checkout",
    "CheckoutPricesTypedDict": ".checkout",
    "CheckoutProductPrice": ".checkout",
    "CheckoutProductPriceTypedDict": ".checkout",
    "CheckoutTypedDict": ".checkout",
    "CustomerMetadata": ".checkout",
    "CustomerMetadataTypedDict": ".checkout",
    "CheckoutLinksDeleteRequest": ".checkout_links_deleteop",
    "CheckoutLinksDeleteRequestTypedDict": ".checkout_links_deleteop",
    "CheckoutLinksGetRequest": ".checkout_links_getop",
    "CheckoutLinksGetRequestTypedDict": ".checkout_links_getop",
    "CheckoutLinksListQueryParamOrganizationIDFilter": ".checkout_links_listop",
    "CheckoutLinksListQueryParamOrganizationIDFilterTypedDict": ".checkout_links_listop",
    "CheckoutLinksListQueryParamProductIDFilter": ".checkout_links_listop",
    "CheckoutLinksListQueryParamProductIDFilterTypedDict": ".checkout_links_listop",
    "CheckoutLinksListRequest": ".checkout_links_listop",
    "CheckoutLinksListRequestTypedDict": ".checkout_links_listop",
    "CheckoutLinksListResponse": ".checkout_links_listop",
    "CheckoutLinksListResponseTypedDict": ".checkout_links_listop",
    "CheckoutLinksUpdateRequest": ".checkout_links_updateop",
    "CheckoutLinksUpdateRequestTypedDict": ".checkout_links_updateop",
    "CheckoutBillingAddressFields": ".checkoutbillingaddressfields",
    "CheckoutBillingAddressFieldsTypedDict": ".checkoutbillingaddressfields",
    "CheckoutConfirmStripe": ".checkoutconfirmstripe",
    "CheckoutConfirmStripeCustomFieldData": ".checkoutconfirmstripe",
    "CheckoutConfirmStripeCustomFieldDataTypedDict": ".checkoutconfirmstripe",
    "CheckoutConfirmStripeTypedDict": ".checkoutconfirmstripe",
    "CheckoutCreate": ".checkoutcreate",
    "CheckoutCreateCustomFieldData": ".checkoutcreate",
    "CheckoutCreateCustomFieldDataTypedDict": ".checkoutcreate",
    "CheckoutCreateCustomerMetadata": ".checkoutcreate",
    "CheckoutCreateCustomerMetadataTypedDict": ".checkoutcreate",
    "CheckoutCreateMetadata": ".checkoutcreate",
    "CheckoutCreateMetadataTypedDict": ".checkoutcreate",
    "CheckoutCreatePrices": ".checkoutcreate",
    "CheckoutCreatePricesTypedDict": ".checkoutcreate",
    "CheckoutCreateTypedDict": ".checkoutcreate",
    "CheckoutCreatedEvent": ".checkoutcreatedevent",
    "CheckoutCreatedEventTypedDict": ".checkoutcreatedevent",
    "CheckoutCreatedMetadata": ".checkoutcreatedmetadata",
    "CheckoutCreatedMetadataTypedDict": ".checkoutcreatedmetadata",
    "CheckoutDiscountFixedOnceForeverDuration": ".checkoutdiscountfixedonceforeverduration",
    "CheckoutDiscountFixedOnceForeverDurationTypedDict": ".checkoutdiscountfixedonceforeverduration",
    "CheckoutDiscountFixedRepeatDuration": ".checkoutdiscountfixedrepeatduration",
    "CheckoutDiscountFixedRepeatDurationTypedDict": ".checkoutdiscountfixedrepeatduration",
    "CheckoutDiscountPercentageOnceForeverDuration": ".checkoutdiscountpercentageonceforeverduration",
    "CheckoutDiscountPercentageOnceForeverDurationTypedDict": ".checkoutdiscountpercentageonceforeverduration",
    "CheckoutDiscountPercentageRepeatDuration": ".checkoutdiscountpercentagerepeatduration",
    "CheckoutDiscountPercentageRepeatDurationTypedDict": ".checkoutdiscountpercentagerepeatduration",
    "CheckoutForbiddenError": ".checkoutforbiddenerror",
    "CheckoutForbiddenErrorUnion": ".checkoutforbiddenerror",
    "CheckoutLink": ".checkoutlink",
    "CheckoutLinkDiscount": ".checkoutlink",
    "CheckoutLinkDiscountTypedDict": ".checkoutlink",
    "CheckoutLinkTypedDict": ".checkoutlink",
    "CheckoutLinkCreate": ".checkoutlinkcreate",
    "CheckoutLinkCreateTypedDict": ".checkoutlinkcreate",
    "CheckoutLinkCreateProduct": ".checkoutlinkcreateproduct",
    "CheckoutLinkCreateProductMetadata": ".checkoutlinkcreateproduct",
    "CheckoutLinkCreateProductMetadataTypedDict": ".checkoutlinkcreateproduct",
    "CheckoutLinkCreateProductTypedDict": ".checkoutlinkcreateproduct",
    "CheckoutLinkCreateProductPrice": ".checkoutlinkcreateproductprice",
    "CheckoutLinkCreateProductPriceMetadata": ".checkoutlinkcreateproductprice",
    "CheckoutLinkCreateProductPriceMetadataTypedDict": ".checkoutlinkcreateproductprice",
    "CheckoutLinkCreateProductPriceTypedDict": ".checkoutlinkcreateproductprice",
    "CheckoutLinkCreateProducts": ".checkoutlinkcreateproducts",
    "CheckoutLinkCreateProductsMetadata": ".checkoutlinkcreateproducts",
    "CheckoutLinkCreateProductsMetadataTypedDict": ".checkoutlinkcreateproducts",
    "CheckoutLinkCreateProductsTypedDict": ".checkoutlinkcreateproducts",
    "CheckoutLinkProduct": ".checkoutlinkproduct",
    "CheckoutLinkProductPrices": ".checkoutlinkproduct",
    "CheckoutLinkProductPricesTypedDict": ".checkoutlinkproduct",
    "CheckoutLinkProductTypedDict": ".checkoutlinkproduct",
    "CheckoutLinkSortProperty": ".checkoutlinksortproperty",
    "CheckoutLinkUpdate": ".checkoutlinkupdate",
    "CheckoutLinkUpdateMetadata": ".checkoutlinkupdate",
    "CheckoutLinkUpdateMetadataTypedDict": ".checkoutlinkupdate",
    "CheckoutLinkUpdateTypedDict": ".checkoutlinkupdate",
    "CheckoutOrganization": ".checkoutorganization",
    "CheckoutOrganizationTypedDict": ".checkoutorganization",
    "CheckoutProduct": ".checkoutproduct",
    "CheckoutProductPrices": ".checkoutproduct",
    "CheckoutProductPricesTypedDict": ".checkoutproduct",
    "CheckoutProductTypedDict": ".checkoutproduct",
    "CheckoutPublic": ".checkoutpublic",
    "CheckoutPublicCustomFieldData": ".checkoutpublic",
    "CheckoutPublicCustomFieldDataTypedDict": ".checkoutpublic",
    "CheckoutPublicDiscount": ".checkoutpublic",
    "CheckoutPublicDiscountTypedDict": ".checkoutpublic",
    "CheckoutPublicPrices": ".checkoutpublic",
    "CheckoutPublicPricesTypedDict": ".checkoutpublic",
    "CheckoutPublicProductPrice": ".checkoutpublic",
    "CheckoutPublicProductPriceTypedDict": ".checkoutpublic",
    "CheckoutPublicTypedDict": ".checkoutpublic",
    "CheckoutPublicConfirmed": ".checkoutpublicconfirmed",
    "CheckoutPublicConfirmedCustomFieldData": ".checkoutpublicconfirmed",
    "CheckoutPublicConfirmedCustomFieldDataTypedDict": ".checkoutpublicconfirmed",
    "CheckoutPublicConfirmedDiscount": ".checkoutpublicconfirmed",
    "CheckoutPublicConfirmedDiscountTypedDict": ".checkoutpublicconfirmed",
    "CheckoutPublicConfirmedPrices": ".checkoutpublicconfirmed",
    "CheckoutPublicConfirmedPricesTypedDict": ".checkoutpublicconfirmed",
    "CheckoutPublicConfirmedProductPrice": ".checkoutpublicconfirmed",
    "CheckoutPublicConfirmedProductPriceTypedDict": ".checkoutpublicconfirmed",
    "CheckoutPublicConfirmedTypedDict": ".checkoutpublicconfirmed",
    "CheckoutsClientConfirmRequest": ".checkouts_client_confirmop",
    "CheckoutsClientConfirmRequestTypedDict": ".checkouts_client_confirmop",
    "CheckoutsClientGetRequest": ".checkouts_client_getop",
    "CheckoutsClientGetRequestTypedDict": ".checkouts_client_getop",
    "CheckoutsClientUpdateRequest": ".checkouts_client_updateop",
    "CheckoutsClientUpdateRequestTypedDict": ".checkouts_client_updateop",
    "CheckoutsGetRequest": ".checkouts_getop",
    "CheckoutsGetRequestTypedDict": ".checkouts_getop",
    "CheckoutsListQueryParamCustomerIDFilter": ".checkouts_listop",
    "CheckoutsListQueryParamCustomerIDFilterTypedDict": ".checkouts_listop",
    "CheckoutsListQueryParamOrganizationIDFilter": ".checkouts_listop",
    "CheckoutsListQueryParamOrganizationIDFilterTypedDict": ".checkouts_listop",
    "CheckoutsListQueryParamProductIDFilter": ".checkouts_listop",
    "CheckoutsListQueryParamProductIDFilterTypedDict": ".checkouts_listop",
    "CheckoutsListRequest": ".checkouts_listop",
    "CheckoutsListRequestTypedDict": ".checkouts_listop",
    "CheckoutsListResponse": ".checkouts_listop",
    "CheckoutsListResponseTypedDict": ".checkouts_listop",
    "QueryParamStatusFilter": ".checkouts_listop",
    "QueryParamStatusFilterTypedDict": ".checkouts_listop",
    "CheckoutsUpdateRequest": ".checkouts_updateop",
    "CheckoutsUpdateRequestTypedDict": ".checkouts_updateop",
    "CheckoutSortProperty": ".checkoutsortproperty",
    "CheckoutStatus": ".checkoutstatus",
    "CheckoutUpdate": ".checkoutupdate",
    "CheckoutUpdateCustomFieldData": ".checkoutupdate",
    "CheckoutUpdateCustomFieldDataTypedDict": ".checkoutupdate",
    "CheckoutUpdateCustomerMetadata": ".checkoutupdate",
    "CheckoutUpdateCustomerMetadataTypedDict": ".checkoutupdate",
    "CheckoutUpdateMetadata": ".checkoutupdate",
    "CheckoutUpdateMetadataTypedDict": ".checkoutupdate",
    "CheckoutUpdateTypedDict": ".checkoutupdate",
    "CheckoutUpdatePublic": ".checkoutupdatepublic",
    "CheckoutUpdatePublicCustomFieldData": ".checkoutupdatepublic",
    "CheckoutUpdatePublicCustomFieldDataTypedDict": ".checkoutupdatepublic",
    "CheckoutUpdatePublicTypedDict": ".checkoutupdatepublic",
    "Amount": ".costmetadata_input",
    "AmountTypedDict": ".costmetadata_input",
    "CostMetadataInput": ".costmetadata_input",
    "CostMetadataInputTypedDict": ".costmetadata_input",
    "CostMetadataOutput": ".costmetadata_output",
    "CostMetadataOutputTypedDict": ".costmetadata_output",
    "CountAggregation": ".countaggregation",
    "CountAggregationTypedDict": ".countaggregation",
    "CursorPagination": ".cursorpagination",
    "CursorPaginationTypedDict": ".cursorpagination",
    "CustomFieldsDeleteRequest": ".custom_fields_deleteop",
    "CustomFieldsDeleteRequestTypedDict": ".custom_fields_deleteop",
    "CustomFieldsGetRequest": ".custom_fields_getop",
    "CustomFieldsGetRequestTypedDict": ".custom_fields_getop",
    "CustomFieldTypeFilter": ".custom_fields_listop",
    "CustomFieldTypeFilterTypedDict": ".custom_fields_listop",
    "CustomFieldsListQueryParamOrganizationIDFilter": ".custom_fields_listop",
    "CustomFieldsListQueryParamOrganizationIDFilterTypedDict": ".custom_fields_listop",
    "CustomFieldsListRequest": ".custom_fields_listop",
    "CustomFieldsListRequestTypedDict": ".custom_fields_listop",
    "CustomFieldsListResponse": ".custom_fields_listop",
    "CustomFieldsListResponseTypedDict": ".custom_fields_listop",
    "CustomFieldsUpdateRequest": ".custom_fields_updateop",
    "CustomFieldsUpdateRequestTypedDict": ".custom_fields_updateop",
    "Customer": ".customer",
    "CustomerTaxID": ".customer",
    "CustomerTaxIDTypedDict": ".customer",
    "CustomerTypedDict": ".customer",
    "CustomerMetersGetRequest": ".customer_meters_getop",
    "CustomerMetersGetRequestTypedDict": ".customer_meters_getop",
    "CustomerMetersListQueryParamCustomerIDFilter": ".customer_meters_listop",
    "CustomerMetersListQueryParamCustomerIDFilterTypedDict": ".customer_meters_listop",
    "CustomerMetersListQueryParamExternalCustomerIDFilter": ".customer_meters_listop",
    "CustomerMetersListQueryParamExternalCustomerIDFilterTypedDict": ".customer_meters_listop",
    "CustomerMetersListQueryParamOrganizationIDFilter": ".customer_meters_listop",
    "CustomerMetersListQueryParamOrganizationIDFilterTypedDict": ".customer_meters_listop",
    "CustomerMetersListRequest": ".customer_meters_listop",
    "CustomerMetersListRequestTypedDict": ".customer_meters_listop",
    "CustomerMetersListResponse": ".customer_meters_listop",
    "CustomerMetersListResponseTypedDict": ".customer_meters_listop",
    "QueryParamMeterIDFilter": ".customer_meters_listop",
    "QueryParamMeterIDFilterTypedDict": ".customer_meters_listop",
    "CustomerPortalBenefitGrantsGetRequest": ".customer_portal_benefit_grants_getop",
    "CustomerPortalBenefitGrantsGetRequestTypedDict": ".customer_portal_benefit_grants_getop",
    "CustomerPortalBenefitGrantsGetSecurity": ".customer_portal_benefit_grants_getop",
    "CustomerPortalBenefitGrantsGetSecurityTypedDict": ".customer_portal_benefit_grants_getop",
    "CustomerPortalBenefitGrantsListQueryParamBenefitIDFilter": ".customer_portal_benefit_grants_listop",
    "CustomerPortalBenefitGrantsListQueryParamBenefitIDFilterTypedDict": ".customer_portal_benefit_grants_listop",
    "CustomerPortalBenefitGrantsListQueryParamOrderIDFilter": ".customer_portal_benefit_grants_listop",
    "CustomerPortalBenefitGrantsListQueryParamOrderIDFilterTypedDict": ".customer_portal_benefit_grants_listop",
    "CustomerPortalBenefitGrantsListRequest": ".customer_portal_benefit_grants_listop",
    "CustomerPortalBenefitGrantsListRequestTypedDict": ".customer_portal_benefit_grants_listop",
    "CustomerPortalBenefitGrantsListResponse": ".customer_portal_benefit_grants_listop",
    "CustomerPortalBenefitGrantsListResponseTypedDict": ".customer_portal_benefit_grants_listop",
    "CustomerPortalBenefitGrantsListSecurity": ".customer_portal_benefit_grants_listop",
    "CustomerPortalBenefitGrantsListSecurityTypedDict": ".customer_portal_benefit_grants_listop",
    "QueryParamBenefitTypeFilter": ".customer_portal_benefit_grants_listop",
    "QueryParamBenefitTypeFilterTypedDict": ".customer_portal_benefit_grants_listop",
    "QueryParamCheckoutIDFilter": ".customer_portal_benefit_grants_listop",
    "QueryParamCheckoutIDFilterTypedDict": ".customer_portal_benefit_grants_listop",
    "QueryParamMemberIDFilter": ".customer_portal_benefit_grants_listop",
    "QueryParamMemberIDFilterTypedDict": ".customer_portal_benefit_grants_listop",
    "QueryParamSubscriptionIDFilter": ".customer_portal_benefit_grants_listop",
    "QueryParamSubscriptionIDFilterTypedDict": ".customer_portal_benefit_grants_listop",
    "CustomerPortalBenefitGrantsUpdateRequest": ".customer_portal_benefit_grants_updateop",
    "CustomerPortalBenefitGrantsUpdateRequestTypedDict": ".customer_portal_benefit_grants_updateop",
    "CustomerPortalBenefitGrantsUpdateSecurity": ".customer_portal_benefit_grants_updateop",
    "CustomerPortalBenefitGrantsUpdateSecurityTypedDict": ".customer_portal_benefit_grants_updateop",
    "CustomerPortalCustomerMetersGetRequest": ".customer_portal_customer_meters_getop",
    "CustomerPortalCustomerMetersGetRequestTypedDict": ".customer_portal_customer_meters_getop",
    "CustomerPortalCustomerMetersGetSecurity": ".customer_portal_customer_meters_getop",
    "CustomerPortalCustomerMetersGetSecurityTypedDict": ".customer_portal_customer_meters_getop",
    "CustomerPortalCustomerMetersListRequest": ".customer_portal_customer_meters_listop",
    "CustomerPortalCustomerMetersListRequestTypedDict": ".customer_portal_customer_meters_listop",
    "CustomerPortalCustomerMetersListResponse": ".customer_portal_customer_meters_listop",
    "CustomerPortalCustomerMetersListResponseTypedDict": ".customer_portal_customer_meters_listop",
    "CustomerPortalCustomerMetersListSecurity": ".customer_portal_customer_meters_listop",
    "CustomerPortalCustomerMetersListSecurityTypedDict": ".customer_portal_customer_meters_listop",
    "MeterIDFilter": ".customer_portal_customer_meters_listop",
    "MeterIDFilterTypedDict": ".customer_portal_customer_meters_listop",
    "CustomerPortalCustomerSessionGetAuthenticatedUserSecurity": ".customer_portal_customer_session_get_authenticated_userop",
    "CustomerPortalCustomerSessionGetAuthenticatedUserSecurityTypedDict": ".customer_portal_customer_session_get_authenticated_userop",
    "CustomerPortalCustomerSessionIntrospectSecurity": ".customer_portal_customer_session_introspectop",
    "CustomerPortalCustomerSessionIntrospectSecurityTypedDict": ".customer_portal_customer_session_introspectop",
    "CustomerPortalCustomersAddPaymentMethodSecurity": ".customer_portal_customers_add_payment_methodop",
    "CustomerPortalCustomersAddPaymentMethodSecurityTypedDict": ".customer_portal_customers_add_payment_methodop",
    "CustomerPortalCustomersConfirmPaymentMethodSecurity": ".customer_portal_customers_confirm_payment_methodop",
    "CustomerPortalCustomersConfirmPaymentMethodSecurityTypedDict": ".customer_portal_customers_confirm_payment_methodop",
    "CustomerPortalCustomersDeletePaymentMethodRequest": ".customer_portal_customers_delete_payment_methodop",
    "CustomerPortalCustomersDeletePaymentMethodRequestTypedDict": ".customer_portal_customers_delete_payment_methodop",
    "CustomerPortalCustomersDeletePaymentMethodSecurity": ".customer_portal_customers_delete_payment_methodop",
    "CustomerPortalCustomersDeletePaymentMethodSecurityTypedDict": ".customer_portal_customers_delete_payment_methodop",
    "CustomerPortalCustomersGetSecurity": ".customer_portal_customers_getop",
    "CustomerPortalCustomersGetSecurityTypedDict": ".customer_portal_customers_getop",
    "CustomerPortalCustomersListPaymentMethodsRequest": ".customer_portal_customers_list_payment_methodsop",
    "CustomerPortalCustomersListPaymentMethodsRequestTypedDict": ".customer_portal_customers_list_payment_methodsop",
    "CustomerPortalCustomersListPaymentMethodsResponse": ".customer_portal_customers_list_payment_methodsop",
    "CustomerPortalCustomersListPaymentMethodsResponseTypedDict": ".customer_portal_customers_list_payment_methodsop",
    "CustomerPortalCustomersListPaymentMethodsSecurity": ".customer_portal_customers_list_payment_methodsop",
    "CustomerPortalCustomersListPaymentMethodsSecurityTypedDict": ".customer_portal_customers_list_payment_methodsop",
    "CustomerPortalCustomersUpdateSecurity": ".customer_portal_customers_updateop",
    "CustomerPortalCustomersUpdateSecurityTypedDict": ".customer_portal_customers_updateop",
    "CustomerPortalDownloadablesListQueryParamBenefitIDFilter": ".customer_portal_downloadables_listop",
    "CustomerPortalDownloadablesListQueryParamBenefitIDFilterTypedDict": ".customer_portal_downloadables_listop",
    "CustomerPortalDownloadablesListRequest": ".customer_portal_downloadables_listop",
    "CustomerPortalDownloadablesListRequestTypedDict": ".customer_portal_downloadables_listop",
    "CustomerPortalDownloadablesListResponse": ".customer_portal_downloadables_listop",
    "CustomerPortalDownloadablesListResponseTypedDict": ".customer_portal_downloadables_listop",
    "CustomerPortalDownloadablesListSecurity": ".customer_portal_downloadables_listop",
    "CustomerPortalDownloadablesListSecurityTypedDict": ".customer_portal_downloadables_listop",
    "CustomerPortalLicenseKeysGetRequest": ".customer_portal_license_keys_getop",
    "CustomerPortalLicenseKeysGetRequestTypedDict": ".customer_portal_license_keys_getop",
    "CustomerPortalLicenseKeysGetSecurity": ".customer_portal_license_keys_getop",
    "CustomerPortalLicenseKeysGetSecurityTypedDict": ".customer_portal_license_keys_getop",
    "CustomerPortalLicenseKeysListRequest": ".customer_portal_license_keys_listop",
    "CustomerPortalLicenseKeysListRequestTypedDict": ".customer_portal_license_keys_listop",
    "CustomerPortalLicenseKeysListResponse": ".customer_portal_license_keys_listop",
    "CustomerPortalLicenseKeysListResponseTypedDict": ".customer_portal_license_keys_listop",
    "CustomerPortalLicenseKeysListSecurity": ".customer_portal_license_keys_listop",
    "CustomerPortalLicenseKeysListSecurityTypedDict": ".customer_portal_license_keys_listop",
    "CustomerPortalOrdersConfirmRetryPaymentRequest": ".customer_portal_orders_confirm_retry_paymentop",
    "CustomerPortalOrdersConfirmRetryPaymentRequestTypedDict": ".customer_portal_orders_confirm_retry_paymentop",
    "CustomerPortalOrdersConfirmRetryPaymentSecurity": ".customer_portal_orders_confirm_retry_paymentop",
    "CustomerPortalOrdersConfirmRetryPaymentSecurityTypedDict": ".customer_portal_orders_confirm_retry_paymentop",
    "CustomerPortalOrdersGenerateInvoiceRequest": ".customer_portal_orders_generate_invoiceop",
    "CustomerPortalOrdersGenerateInvoiceRequestTypedDict": ".customer_portal_orders_generate_invoiceop",
    "CustomerPortalOrdersGenerateInvoiceResponse422CustomerPortalOrdersGenerateInvoice": ".customer_portal_orders_generate_invoiceop",
    "CustomerPortalOrdersGenerateInvoiceResponse422CustomerPortalOrdersGenerateInvoiceUnion": ".customer_portal_orders_generate_invoiceop",
    "CustomerPortalOrdersGenerateInvoiceSecurity": ".customer_portal_orders_generate_invoiceop",
    "CustomerPortalOrdersGenerateInvoiceSecurityTypedDict": ".customer_portal_orders_generate_invoiceop",
    "CustomerPortalOrdersGetPaymentStatusRequest": ".customer_portal_orders_get_payment_statusop",
    "CustomerPortalOrdersGetPaymentStatusRequestTypedDict": ".customer_portal_orders_get_payment_statusop",
    "CustomerPortalOrdersGetPaymentStatusSecurity": ".customer_portal_orders_get_payment_statusop",
    "CustomerPortalOrdersGetPaymentStatusSecurityTypedDict": ".customer_portal_orders_get_payment_statusop",
    "CustomerPortalOrdersGetRequest": ".customer_portal_orders_getop",
    "CustomerPortalOrdersGetRequestTypedDict": ".customer_portal_orders_getop",
    "CustomerPortalOrdersGetSecurity": ".customer_portal_orders_getop",
    "CustomerPortalOrdersGetSecurityTypedDict": ".customer_portal_orders_getop",
    "CustomerPortalOrdersInvoiceRequest": ".customer_portal_orders_invoiceop",
    "CustomerPortalOrdersInvoiceRequestTypedDict": ".customer_portal_orders_invoiceop",
    "CustomerPortalOrdersInvoiceSecurity": ".customer_portal_orders_invoiceop",
    "CustomerPortalOrdersInvoiceSecurityTypedDict": ".customer_portal_orders_invoiceop",
    "CustomerPortalOrdersListQueryParamProductBillingTypeFilter": ".customer_portal_orders_listop",
    "CustomerPortalOrdersListQueryParamProductBillingTypeFilterTypedDict": ".customer_portal_orders_listop",
    "CustomerPortalOrdersListQueryParamProductIDFilter": ".customer_portal_orders_listop",
    "CustomerPortalOrdersListQueryParamProductIDFilterTypedDict": ".customer_portal_orders_listop",
    "CustomerPortalOrdersListQueryParamSubscriptionIDFilter": ".customer_portal_orders_listop",
    "CustomerPortalOrdersListQueryParamSubscriptionIDFilterTypedDict": ".customer_portal_orders_listop",
    "CustomerPortalOrdersListRequest": ".customer_portal_orders_listop",
    "CustomerPortalOrdersListRequestTypedDict": ".customer_portal_orders_listop",
    "CustomerPortalOrdersListResponse": ".customer_portal_orders_listop",
    "CustomerPortalOrdersListResponseTypedDict": ".customer_portal_orders_listop",
    "CustomerPortalOrdersListSecurity": ".customer_portal_orders_listop",
    "CustomerPortalOrdersListSecurityTypedDict": ".customer_portal_orders_listop",
    "CustomerPortalOrdersUpdateRequest": ".customer_portal_orders_updateop",
    "CustomerPortalOrdersUpdateRequestTypedDict": ".customer_portal_orders_updateop",
    "CustomerPortalOrdersUpdateSecurity": ".customer_portal_orders_updateop",
    "CustomerPortalOrdersUpdateSecurityTypedDict": ".customer_portal_orders_updateop",
    "CustomerPortalOrganizationsGetRequest": ".customer_portal_organizations_getop",
    "CustomerPortalOrganizationsGetRequestTypedDict": ".customer_portal_organizations_getop",
    "CustomerPortalSeatsAssignSeatSecurity": ".customer_portal_seats_assign_seatop",
    "CustomerPortalSeatsAssignSeatSecurityTypedDict": ".customer_portal_seats_assign_seatop",
    "CustomerPortalSeatsListClaimedSubscriptionsSecurity": ".customer_portal_seats_list_claimed_subscriptionsop",
    "CustomerPortalSeatsListClaimedSubscriptionsSecurityTypedDict": ".customer_portal_seats_list_claimed_subscriptionsop",
    "CustomerPortalSeatsListSeatsRequest": ".customer_portal_seats_list_seatsop",
    "CustomerPortalSeatsListSeatsRequestTypedDict": ".customer_portal_seats_list_seatsop",
    "CustomerPortalSeatsListSeatsSecurity": ".customer_portal_seats_list_seatsop",
    "CustomerPortalSeatsListSeatsSecurityTypedDict": ".customer_portal_seats_list_seatsop",
    "CustomerPortalSeatsResendInvitationRequest": ".customer_portal_seats_resend_invitationop",
    "CustomerPortalSeatsResendInvitationRequestTypedDict": ".customer_portal_seats_resend_invitationop",
    "CustomerPortalSeatsResendInvitationSecurity": ".customer_portal_seats_resend_invitationop",
    "CustomerPortalSeatsResendInvitationSecurityTypedDict": ".customer_portal_seats_resend_invitationop",
    "CustomerPortalSeatsRevokeSeatRequest": ".customer_portal_seats_revoke_seatop",
    "CustomerPortalSeatsRevokeSeatRequestTypedDict": ".customer_portal_seats_revoke_seatop",
    "CustomerPortalSeatsRevokeSeatSecurity": ".customer_portal_seats_revoke_seatop",
    "CustomerPortalSeatsRevokeSeatSecurityTypedDict": ".customer_portal_seats_revoke_seatop",
    "CustomerPortalSubscriptionsCancelRequest": ".customer_portal_subscriptions_cancelop",
    "CustomerPortalSubscriptionsCancelRequestTypedDict": ".customer_portal_subscriptions_cancelop",
    "CustomerPortalSubscriptionsCancelSecurity": ".customer_portal_subscriptions_cancelop",
    "CustomerPortalSubscriptionsCancelSecurityTypedDict": ".customer_portal_subscriptions_cancelop",
    "CustomerPortalSubscriptionsGetRequest": ".customer_portal_subscriptions_getop",
    "CustomerPortalSubscriptionsGetRequestTypedDict": ".customer_portal_subscriptions_getop",
    "CustomerPortalSubscriptionsGetSecurity": ".customer_portal_subscriptions_getop",
    "CustomerPortalSubscriptionsGetSecurityTypedDict": ".customer_portal_subscriptions_getop",
    "CustomerPortalSubscriptionsListQueryParamProductIDFilter": ".customer_portal_subscriptions_listop",
    "CustomerPortalSubscriptionsListQueryParamProductIDFilterTypedDict": ".customer_portal_subscriptions_listop",
    "CustomerPortalSubscriptionsListRequest": ".customer_portal_subscriptions_listop",
    "CustomerPortalSubscriptionsListRequestTypedDict": ".customer_portal_subscriptions_listop",
    "CustomerPortalSubscriptionsListResponse": ".customer_portal_subscriptions_listop",
    "CustomerPortalSubscriptionsListResponseTypedDict": ".customer_portal_subscriptions_listop",
    "CustomerPortalSubscriptionsListSecurity": ".customer_portal_subscriptions_listop",
    "CustomerPortalSubscriptionsListSecurityTypedDict": ".customer_portal_subscriptions_listop",
    "CustomerPortalSubscriptionsUpdateRequest": ".customer_portal_subscriptions_updateop",
    "CustomerPortalSubscriptionsUpdateRequestTypedDict": ".customer_portal_subscriptions_updateop",
    "CustomerPortalSubscriptionsUpdateSecurity": ".customer_portal_subscriptions_updateop",
    "CustomerPortalSubscriptionsUpdateSecurityTypedDict": ".customer_portal_subscriptions_updateop",
    "CustomerPortalWalletsGetRequest": ".customer_portal_wallets_getop",
    "CustomerPortalWalletsGetRequestTypedDict": ".customer_portal_wallets_getop",
    "CustomerPortalWalletsGetSecurity": ".customer_portal_wallets_getop",
    "CustomerPortalWalletsGetSecurityTypedDict": ".customer_portal_wallets_getop",
    "CustomerPortalWalletsListRequest": ".customer_portal_wallets_listop",
    "CustomerPortalWalletsListRequestTypedDict": ".customer_portal_wallets_listop",
    "CustomerPortalWalletsListResponse": ".customer_portal_wallets_listop",
    "CustomerPortalWalletsListResponseTypedDict": ".customer_portal_wallets_listop",
    "CustomerPortalWalletsListSecurity": ".customer_portal_wallets_listop",
    "CustomerPortalWalletsListSecurityTypedDict": ".customer_portal_wallets_listop",
    "CustomerSeatsGetClaimInfoRequest": ".customer_seats_get_claim_infoop",
    "CustomerSeatsGetClaimInfoRequestTypedDict": ".customer_seats_get_claim_infoop",
    "CustomerSeatsListSeatsRequest": ".customer_seats_list_seatsop",
    "CustomerSeatsListSeatsRequestTypedDict": ".customer_seats_list_seatsop",
    "CustomerSeatsResendInvitationRequest": ".customer_seats_resend_invitationop",
    "CustomerSeatsResendInvitationRequestTypedDict": ".customer_seats_resend_invitationop",
    "CustomerSeatsRevokeSeatRequest": ".customer_seats_revoke_seatop",
    "CustomerSeatsRevokeSeatRequestTypedDict": ".customer_seats_revoke_seatop",
    "CustomerSessionsCreateCustomerSessionCreate": ".customer_sessions_createop",
    "CustomerSessionsCreateCustomerSessionCreateTypedDict": ".customer_sessions_createop",
    "CustomerBenefitGrant": ".customerbenefitgrant",
    "CustomerBenefitGrantTypedDict": ".customerbenefitgrant",
    "CustomerBenefitGrantCustom": ".customerbenefitgrantcustom",
    "CustomerBenefitGrantCustomTypedDict": ".customerbenefitgrantcustom",
    "CustomerBenefitGrantCustomUpdate": ".customerbenefitgrantcustomupdate",
    "CustomerBenefitGrantCustomUpdateTypedDict": ".customerbenefitgrantcustomupdate",
    "CustomerBenefitGrantDiscord": ".customerbenefitgrantdiscord",
    "CustomerBenefitGrantDiscordTypedDict": ".customerbenefitgrantdiscord",
    "CustomerBenefitGrantDiscordPropertiesUpdate": ".customerbenefitgrantdiscordpropertiesupdate",
    "CustomerBenefitGrantDiscordPropertiesUpdateTypedDict": ".customerbenefitgrantdiscordpropertiesupdate",
    "CustomerBenefitGrantDiscordUpdate": ".customerbenefitgrantdiscordupdate",
    "CustomerBenefitGrantDiscordUpdateTypedDict": ".customerbenefitgrantdiscordupdate",
    "CustomerBenefitGrantDownloadables": ".customerbenefitgrantdownloadables",
    "CustomerBenefitGrantDownloadablesTypedDict": ".customerbenefitgrantdownloadables",
    "CustomerBenefitGrantDownloadablesUpdate": ".customerbenefitgrantdownloadablesupdate",
    "CustomerBenefitGrantDownloadablesUpdateTypedDict": ".customerbenefitgrantdownloadablesupdate",
    "CustomerBenefitGrantGitHubRepository": ".customerbenefitgrantgithubrepository",
    "CustomerBenefitGrantGitHubRepositoryTypedDict": ".customerbenefitgrantgithubrepository",
    "CustomerBenefitGrantGitHubRepositoryPropertiesUpdate": ".customerbenefitgrantgithubrepositorypropertiesupdate",
    "CustomerBenefitGrantGitHubRepositoryPropertiesUpdateTypedDict": ".customerbenefitgrantgithubrepositorypropertiesupdate",
    "CustomerBenefitGrantGitHubRepositoryUpdate": ".customerbenefitgrantgithubrepositoryupdate",
    "CustomerBenefitGrantGitHubRepositoryUpdateTypedDict": ".customerbenefitgrantgithubrepositoryupdate",
    "CustomerBenefitGrantLicenseKeys": ".customerbenefitgrantlicensekeys",
    "CustomerBenefitGrantLicenseKeysTypedDict": ".customerbenefitgrantlicensekeys",
    "CustomerBenefitGrantLicenseKeysUpdate": ".customerbenefitgrantlicensekeysupdate",
    "CustomerBenefitGrantLicenseKeysUpdateTypedDict": ".customerbenefitgrantlicensekeysupdate",
    "CustomerBenefitGrantMeterCredit": ".customerbenefitgrantmetercredit",
    "CustomerBenefitGrantMeterCreditTypedDict": ".customerbenefitgrantmetercredit",
    "CustomerBenefitGrantMeterCreditUpdate": ".customerbenefitgrantmetercreditupdate",
    "CustomerBenefitGrantMeterCreditUpdateTypedDict": ".customerbenefitgrantmetercreditupdate",
    "CustomerBenefitGrantSortProperty": ".customerbenefitgrantsortproperty",
    "CustomerBenefitGrantUpdate": ".customerbenefitgrantupdate",
    "CustomerBenefitGrantUpdateTypedDict": ".customerbenefitgrantupdate",
    "CustomerCancellationReason": ".customercancellationreason",
    "CustomerCreate": ".customercreate",
    "CustomerCreateMetadata": ".customercreate",
    "CustomerCreateMetadataTypedDict": ".customercreate",
    "CustomerCreateTaxID": ".customercreate",
    "CustomerCreateTaxIDTypedDict": ".customercreate",
    "CustomerCreateTypedDict": ".customercreate",
    "CustomerCreatedEvent": ".customercreatedevent",
    "CustomerCreatedEventTypedDict": ".customercreatedevent",
    "CustomerCreatedMetadata": ".customercreatedmetadata",
    "CustomerCreatedMetadataTypedDict": ".customercreatedmetadata",
    "CustomerCustomerMeter": ".customercustomermeter",
    "CustomerCustomerMeterTypedDict": ".customercustomermeter",
    "CustomerCustomerMeterMeter": ".customercustomermetermeter",
    "CustomerCustomerMeterMeterTypedDict": ".customercustomermetermeter",
    "CustomerCustomerMeterSortProperty": ".customercustomermetersortproperty",
    "CustomerCustomerSession": ".customercustomersession",
    "CustomerCustomerSessionTypedDict": ".customercustomersession",
    "CustomerDeletedEvent": ".customerdeletedevent",
    "CustomerDeletedEventTypedDict": ".customerdeletedevent",
    "CustomerDeletedMetadata": ".customerdeletedmetadata",
    "CustomerDeletedMetadataTypedDict": ".customerdeletedmetadata",
    "CustomerMeter": ".customermeter",
    "CustomerMeterTypedDict": ".customermeter",
    "CustomerMeterSortProperty": ".customermetersortproperty",
    "CustomerNotReady": ".customernotready",
    "CustomerNotReadyData": ".customernotready",
    "CustomerOrder": ".customerorder",
    "CustomerOrderTypedDict": ".customerorder",
    "CustomerOrderConfirmPayment": ".customerorderconfirmpayment",
    "CustomerOrderConfirmPaymentTypedDict": ".customerorderconfirmpayment",
    "CustomerOrderInvoice": ".customerorderinvoice",
    "CustomerOrderInvoiceTypedDict": ".customerorderinvoice",
    "CustomerOrderPaymentConfirmation": ".customerorderpaymentconfirmation",
    "CustomerOrderPaymentConfirmationTypedDict": ".customerorderpaymentconfirmation",
    "CustomerOrderPaymentStatus": ".customerorderpaymentstatus",
    "CustomerOrderPaymentStatusTypedDict": ".customerorderpaymentstatus",
    "CustomerOrderProduct": ".customerorderproduct",
    "CustomerOrderProductPrices": ".customerorderproduct",
    "CustomerOrderProductPricesTypedDict": ".customerorderproduct",
    "CustomerOrderProductTypedDict": ".customerorderproduct",
    "CustomerOrderSortProperty": ".customerordersortproperty",
    "CustomerOrderSubscription": ".customerordersubscription",
    "CustomerOrderSubscriptionTypedDict": ".customerordersubscription",
    "CustomerOrderUpdate": ".customerorderupdate",
    "CustomerOrderUpdateTypedDict": ".customerorderupdate",
    "CustomerOrganization": ".customerorganization",
    "CustomerOrganizationTypedDict": ".customerorganization",
    "CustomerOrganizationData": ".customerorganizationdata",
    "CustomerOrganizationDataTypedDict": ".customerorganizationdata",
    "CustomerPaymentMethod": ".customerpaymentmethod",
    "CustomerPaymentMethodTypedDict": ".customerpaymentmethod",
    "CustomerPaymentMethodConfirm": ".customerpaymentmethodconfirm",
    "CustomerPaymentMethodConfirmTypedDict": ".customerpaymentmethodconfirm",
    "CustomerPaymentMethodCreate": ".customerpaymentmethodcreate",
    "CustomerPaymentMethodCreateTypedDict": ".customerpaymentmethodcreate",
    "CustomerPaymentMethodCreateRequiresActionResponse": ".customerpaymentmethodcreaterequiresactionresponse",
    "CustomerPaymentMethodCreateRequiresActionResponseTypedDict": ".customerpaymentmethodcreaterequiresactionresponse",
    "CustomerPaymentMethodCreateResponse": ".customerpaymentmethodcreateresponse",
    "CustomerPaymentMethodCreateResponseTypedDict": ".customerpaymentmethodcreateresponse",
    "CustomerPaymentMethodCreateSucceededResponse": ".customerpaymentmethodcreatesucceededresponse",
    "CustomerPaymentMethodCreateSucceededResponseTypedDict": ".customerpaymentmethodcreatesucceededresponse",
    "CustomerPortalCustomer": ".customerportalcustomer",
    "CustomerPortalCustomerTaxID": ".customerportalcustomer",
    "CustomerPortalCustomerTaxIDTypedDict": ".customerportalcustomer",
    "CustomerPortalCustomerTypedDict": ".customerportalcustomer",
    "CustomerPortalCustomerUpdate": ".customerportalcustomerupdate",
    "CustomerPortalCustomerUpdateTypedDict": ".customerportalcustomerupdate",
    "CustomerPortalOAuthAccount": ".customerportaloauthaccount",
    "CustomerPortalOAuthAccountTypedDict": ".customerportaloauthaccount",
    "CustomerPortalSubscriptionSettings": ".customerportalsubscriptionsettings",
    "CustomerPortalSubscriptionSettingsTypedDict": ".customerportalsubscriptionsettings",
    "CustomerPortalUsageSettings": ".customerportalusagesettings",
    "CustomerPortalUsageSettingsTypedDict": ".customerportalusagesettings",
    "CustomerProduct": ".customerproduct",
    "CustomerProductPrices": ".customerproduct",
    "CustomerProductPricesTypedDict": ".customerproduct",
    "CustomerProductTypedDict": ".customerproduct",
    "CustomersDeleteExternalRequest": ".customers_delete_externalop",
    "CustomersDeleteExternalRequestTypedDict": ".customers_delete_externalop",
    "CustomersDeleteRequest": ".customers_deleteop",
    "CustomersDeleteRequestTypedDict": ".customers_deleteop",
    "CustomersExportQueryParamOrganizationID": ".customers_exportop",
    "CustomersExportQueryParamOrganizationIDTypedDict": ".customers_exportop",
    "CustomersExportRequest": ".customers_exportop",
    "CustomersExportRequestTypedDict": ".customers_exportop",
    "CustomersGetExternalRequest": ".customers_get_externalop",
    "CustomersGetExternalRequestTypedDict": ".customers_get_externalop",
    "CustomersGetStateExternalRequest": ".customers_get_state_externalop",
    "CustomersGetStateExternalRequestTypedDict": ".customers_get_state_externalop",
    "CustomersGetStateRequest": ".customers_get_stateop",
    "CustomersGetStateRequestTypedDict": ".customers_get_stateop",
    "CustomersGetRequest": ".customers_getop",
    "CustomersGetRequestTypedDict": ".customers_getop",
    "CustomersUpdateExternalRequest": ".customers_update_externalop",
    "CustomersUpdateExternalRequestTypedDict": ".customers_update_externalop",
    "CustomersUpdateRequest": ".customers_updateop",
    "CustomersUpdateRequestTypedDict": ".customers_updateop",
    "CustomerSeat": ".customerseat",
    "CustomerSeatTypedDict": ".customerseat",
    "CustomerSeatClaimResponse": ".customerseatclaimresponse",
    "CustomerSeatClaimResponseTypedDict": ".customerseatclaimresponse",
    "CustomerSession": ".customersession",
    "CustomerSessionTypedDict": ".customersession",
    "CustomerSessionCustomerExternalIDCreate": ".customersessioncustomerexternalidcreate",
    "CustomerSessionCustomerExternalIDCreateTypedDict": ".customersessioncustomerexternalidcreate",
    "CustomerSessionCustomerIDCreate": ".customersessioncustomeridcreate",
    "CustomerSessionCustomerIDCreateTypedDict": ".customersessioncustomeridcreate",
    "CustomerSortProperty": ".customersortproperty",
    "CustomerState": ".customerstate",
    "CustomerStateTaxID": ".customerstate",
    "CustomerStateTaxIDTypedDict": ".customerstate",
    "CustomerStateTypedDict": ".customerstate",
    "CustomerStateBenefitGrant": ".customerstatebenefitgrant",
    "CustomerStateBenefitGrantProperties": ".customerstatebenefitgrant",
    "CustomerStateBenefitGrantPropertiesTypedDict": ".customerstatebenefitgrant",
    "CustomerStateBenefitGrantTypedDict": ".customerstatebenefitgrant",
    "CustomerStateMeter": ".customerstatemeter",
    "CustomerStateMeterTypedDict": ".customerstatemeter",
    "CustomerStateSubscription": ".customerstatesubscription",
    "CustomerStateSubscriptionCustomFieldData": ".customerstatesubscription",
    "CustomerStateSubscriptionCustomFieldDataTypedDict": ".customerstatesubscription",
    "CustomerStateSubscriptionTypedDict": ".customerstatesubscription",
    "Status": ".customerstatesubscription",
    "CustomerStateSubscriptionMeter": ".customerstatesubscriptionmeter",
    "CustomerStateSubscriptionMeterTypedDict": ".customerstatesubscriptionmeter",
    "CustomerSubscription": ".customersubscription",
    "CustomerSubscriptionPrices": ".customersubscription",
    "CustomerSubscriptionPricesTypedDict": ".customersubscription",
    "CustomerSubscriptionTypedDict": ".customersubscription",
    "CustomerSubscriptionCancel": ".customersubscriptioncancel",
    "CustomerSubscriptionCancelTypedDict": ".customersubscriptioncancel",
    "CustomerSubscriptionMeter": ".customersubscriptionmeter",
    "CustomerSubscriptionMeterTypedDict": ".customersubscriptionmeter",
    "CustomerSubscriptionMeterMeter": ".customersubscriptionmetermeter",
    "CustomerSubscriptionMeterMeterTypedDict": ".customersubscriptionmetermeter",
    "CustomerSubscriptionProduct": ".customersubscriptionproduct",
    "CustomerSubscriptionProductPrices": ".customersubscriptionproduct",
    "CustomerSubscriptionProductPricesTypedDict": ".customersubscriptionproduct",
    "CustomerSubscriptionProductTypedDict": ".customersubscriptionproduct",
    "CustomerSubscriptionSortProperty": ".customersubscriptionsortproperty",
    "CustomerSubscriptionUpdate": ".customersubscriptionupdate",
    "CustomerSubscriptionUpdateTypedDict": ".customersubscriptionupdate",
    "CustomerSubscriptionUpdateProduct": ".customersubscriptionupdateproduct",
    "CustomerSubscriptionUpdateProductTypedDict": ".customersubscriptionupdateproduct",
    "CustomerSubscriptionUpdateSeats": ".customersubscriptionupdateseats",
    "CustomerSubscriptionUpdateSeatsTypedDict": ".customersubscriptionupdateseats",
    "CustomerUpdate": ".customerupdate",
    "CustomerUpdateMetadata": ".customerupdate",
    "CustomerUpdateMetadataTypedDict": ".customerupdate",
    "CustomerUpdateTaxID": ".customerupdate",
    "CustomerUpdateTaxIDTypedDict": ".customerupdate",
    "CustomerUpdateTypedDict": ".customerupdate",
    "CustomerUpdatedEvent": ".customerupdatedevent",
    "CustomerUpdatedEventTypedDict": ".customerupdatedevent",
    "CustomerUpdatedFields": ".customerupdatedfields",
    "CustomerUpdatedFieldsMetadata": ".customerupdatedfields",
    "CustomerUpdatedFieldsMetadataTypedDict": ".customerupdatedfields",
    "CustomerUpdatedFieldsTypedDict": ".customerupdatedfields",
    "CustomerUpdatedMetadata": ".customerupdatedmetadata",
    "CustomerUpdatedMetadataTypedDict": ".customerupdatedmetadata",
    "CustomerUpdateExternalID": ".customerupdateexternalid",
    "CustomerUpdateExternalIDMetadata": ".customerupdateexternalid",
    "CustomerUpdateExternalIDMetadataTypedDict": ".customerupdateexternalid",
    "CustomerUpdateExternalIDTaxID": ".customerupdateexternalid",
    "CustomerUpdateExternalIDTaxIDTypedDict": ".customerupdateexternalid",
    "CustomerUpdateExternalIDTypedDict": ".customerupdateexternalid",
    "CustomerWallet": ".customerwallet",
    "CustomerWalletTypedDict": ".customerwallet",
    "CustomerWalletSortProperty": ".customerwalletsortproperty",
    "CustomerWithMembers": ".customerwithmembers",
    "CustomerWithMembersTaxID": ".customerwithmembers",
    "CustomerWithMembersTaxIDTypedDict": ".customerwithmembers",
    "CustomerWithMembersTypedDict": ".customerwithmembers",
    "CustomField": ".customfield",
    "CustomFieldTypedDict": ".customfield",
    "CustomFieldCheckbox": ".customfieldcheckbox",
    "CustomFieldCheckboxTypedDict": ".customfieldcheckbox",
    "CustomFieldCheckboxProperties": ".customfieldcheckboxproperties",
    "CustomFieldCheckboxPropertiesTypedDict": ".customfieldcheckboxproperties",
    "CustomFieldCreate": ".customfieldcreate",
    "CustomFieldCreateTypedDict": ".customfieldcreate",
    "CustomFieldCreateCheckbox": ".customfieldcreatecheckbox",
    "CustomFieldCreateCheckboxMetadata": ".customfieldcreatecheckbox",
    "CustomFieldCreateCheckboxMetadataTypedDict": ".customfieldcreatecheckbox",
    "CustomFieldCreateCheckboxTypedDict": ".customfieldcreatecheckbox",
    "CustomFieldCreateDate": ".customfieldcreatedate",
    "CustomFieldCreateDateMetadata": ".customfieldcreatedate",
    "CustomFieldCreateDateMetadataTypedDict": ".customfieldcreatedate",
    "CustomFieldCreateDateTypedDict": ".customfieldcreatedate",
    "CustomFieldCreateNumber": ".customfieldcreatenumber",
    "CustomFieldCreateNumberMetadata": ".customfieldcreatenumber",
    "CustomFieldCreateNumberMetadataTypedDict": ".customfieldcreatenumber",
    "CustomFieldCreateNumberTypedDict": ".customfieldcreatenumber",
    "CustomFieldCreateSelect": ".customfieldcreateselect",
    "CustomFieldCreateSelectMetadata": ".customfieldcreateselect",
    "CustomFieldCreateSelectMetadataTypedDict": ".customfieldcreateselect",
    "CustomFieldCreateSelectTypedDict": ".customfieldcreateselect",
    "CustomFieldCreateText": ".customfieldcreatetext",
    "CustomFieldCreateTextMetadata": ".customfieldcreatetext",
    "CustomFieldCreateTextMetadataTypedDict": ".customfieldcreatetext",
    "CustomFieldCreateTextTypedDict": ".customfieldcreatetext",
    "CustomFieldDate": ".customfielddate",
    "CustomFieldDateTypedDict": ".customfielddate",
    "CustomFieldDateProperties": ".customfielddateproperties",
    "CustomFieldDatePropertiesTypedDict": ".customfielddateproperties",
    "CustomFieldNumber": ".customfieldnumber",
    "CustomFieldNumberTypedDict": ".customfieldnumber",
    "CustomFieldNumberProperties": ".customfieldnumberproperties",
    "CustomFieldNumberPropertiesTypedDict": ".customfieldnumberproperties",
    "CustomFieldSelect": ".customfieldselect",
    "CustomFieldSelectTypedDict": ".customfieldselect",
    "CustomFieldSelectOption": ".customfieldselectoption",
    "CustomFieldSelectOptionTypedDict": ".customfieldselectoption",
    "CustomFieldSelectProperties": ".customfieldselectproperties",
    "CustomFieldSelectPropertiesTypedDict": ".customfieldselectproperties",
    "CustomFieldSortProperty": ".customfieldsortproperty",
    "CustomFieldText": ".customfieldtext",
    "CustomFieldTextTypedDict": ".customfieldtext",
    "CustomFieldTextProperties": ".customfieldtextproperties",
    "CustomFieldTextPropertiesTypedDict": ".customfieldtextproperties",
    "CustomFieldType": ".customfieldtype",
    "CustomFieldUpdate": ".customfieldupdate",
    "CustomFieldUpdateTypedDict": ".customfieldupdate",
    "CustomFieldUpdateCheckbox": ".customfieldupdatecheckbox",
    "CustomFieldUpdateCheckboxMetadata": ".customfieldupdatecheckbox",
    "CustomFieldUpdateCheckboxMetadataTypedDict": ".customfieldupdatecheckbox",
    "CustomFieldUpdateCheckboxTypedDict": ".customfieldupdatecheckbox",
    "CustomFieldUpdateDate": ".customfieldupdatedate",
    "CustomFieldUpdateDateMetadata": ".customfieldupdatedate",
    "CustomFieldUpdateDateMetadataTypedDict": ".customfieldupdatedate",
    "CustomFieldUpdateDateTypedDict": ".customfieldupdatedate",
    "CustomFieldUpdateNumber": ".customfieldupdatenumber",
    "CustomFieldUpdateNumberMetadata": ".customfieldupdatenumber",
    "CustomFieldUpdateNumberMetadataTypedDict": ".customfieldupdatenumber",
    "CustomFieldUpdateNumberTypedDict": ".customfieldupdatenumber",
    "CustomFieldUpdateSelect": ".customfieldupdateselect",
    "CustomFieldUpdateSelectMetadata": ".customfieldupdateselect",
    "CustomFieldUpdateSelectMetadataTypedDict": ".customfieldupdateselect",
    "CustomFieldUpdateSelectTypedDict": ".customfieldupdateselect",
    "CustomFieldUpdateText": ".customfieldupdatetext",
    "CustomFieldUpdateTextMetadata": ".customfieldupdatetext",
    "CustomFieldUpdateTextMetadataTypedDict": ".customfieldupdatetext",
    "CustomFieldUpdateTextTypedDict": ".customfieldupdatetext",
    "Discount": ".discount",
    "DiscountTypedDict": ".discount",
    "DiscountCreate": ".discountcreate",
    "DiscountCreateTypedDict": ".discountcreate",
    "DiscountDuration": ".discountduration",
    "DiscountFixedOnceForeverDuration": ".discountfixedonceforeverduration",
    "DiscountFixedOnceForeverDurationTypedDict": ".discountfixedonceforeverduration",
    "DiscountFixedOnceForeverDurationBase": ".discountfixedonceforeverdurationbase",
    "DiscountFixedOnceForeverDurationBaseTypedDict": ".discountfixedonceforeverdurationbase",
    "DiscountFixedOnceForeverDurationCreate": ".discountfixedonceforeverdurationcreate",
    "DiscountFixedOnceForeverDurationCreateMetadata": ".discountfixedonceforeverdurationcreate",
    "DiscountFixedOnceForeverDurationCreateMetadataTypedDict": ".discountfixedonceforeverdurationcreate",
    "DiscountFixedOnceForeverDurationCreateTypedDict": ".discountfixedonceforeverdurationcreate",
    "DiscountFixedRepeatDuration": ".discountfixedrepeatduration",
    "DiscountFixedRepeatDurationTypedDict": ".discountfixedrepeatduration",
    "DiscountFixedRepeatDurationBase": ".discountfixedrepeatdurationbase",
    "DiscountFixedRepeatDurationBaseTypedDict": ".discountfixedrepeatdurationbase",
    "DiscountFixedRepeatDurationCreate": ".discountfixedrepeatdurationcreate",
    "DiscountFixedRepeatDurationCreateMetadata": ".discountfixedrepeatdurationcreate",
    "DiscountFixedRepeatDurationCreateMetadataTypedDict": ".discountfixedrepeatdurationcreate",
    "DiscountFixedRepeatDurationCreateTypedDict": ".discountfixedrepeatdurationcreate",
    "DiscountPercentageOnceForeverDuration": ".discountpercentageonceforeverduration",
    "DiscountPercentageOnceForeverDurationTypedDict": ".discountpercentageonceforeverduration",
    "DiscountPercentageOnceForeverDurationBase": ".discountpercentageonceforeverdurationbase",
    "DiscountPercentageOnceForeverDurationBaseTypedDict": ".discountpercentageonceforeverdurationbase",
    "DiscountPercentageOnceForeverDurationCreate": ".discountpercentageonceforeverdurationcreate",
    "DiscountPercentageOnceForeverDurationCreateMetadata": ".discountpercentageonceforeverdurationcreate",
    "DiscountPercentageOnceForeverDurationCreateMetadataTypedDict": ".discountpercentageonceforeverdurationcreate",
    "DiscountPercentageOnceForeverDurationCreateTypedDict": ".discountpercentageonceforeverdurationcreate",
    "DiscountPercentageRepeatDuration": ".discountpercentagerepeatduration",
    "DiscountPercentageRepeatDurationTypedDict": ".discountpercentagerepeatduration",
    "DiscountPercentageRepeatDurationBase": ".discountpercentagerepeatdurationbase",
    "DiscountPercentageRepeatDurationBaseTypedDict": ".discountpercentagerepeatdurationbase",
    "DiscountPercentageRepeatDurationCreate": ".discountpercentagerepeatdurationcreate",
    "DiscountPercentageRepeatDurationCreateMetadata": ".discountpercentagerepeatdurationcreate",
    "DiscountPercentageRepeatDurationCreateMetadataTypedDict": ".discountpercentagerepeatdurationcreate",
    "DiscountPercentageRepeatDurationCreateTypedDict": ".discountpercentagerepeatdurationcreate",
    "DiscountProduct": ".discountproduct",
    "DiscountProductTypedDict": ".discountproduct",
    "DiscountsDeleteRequest": ".discounts_deleteop",
    "DiscountsDeleteRequestTypedDict": ".discounts_deleteop",
    "DiscountsGetRequest": ".discounts_getop",
    "DiscountsGetRequestTypedDict": ".discounts_getop",
    "DiscountsListQueryParamOrganizationIDFilter": ".discounts_listop",
    "DiscountsListQueryParamOrganizationIDFilterTypedDict": ".discounts_listop",
    "DiscountsListRequest": ".discounts_listop",
    "DiscountsListRequestTypedDict": ".discounts_listop",
    "DiscountsListResponse": ".discounts_listop",
    "DiscountsListResponseTypedDict": ".discounts_listop",
    "DiscountsUpdateRequest": ".discounts_updateop",
    "DiscountsUpdateRequestTypedDict": ".discounts_updateop",
    "DiscountSortProperty": ".discountsortproperty",
    "DiscountType": ".discounttype",
    "DiscountUpdate": ".discountupdate",
    "DiscountUpdateMetadata": ".discountupdate",
    "DiscountUpdateMetadataTypedDict": ".discountupdate",
    "DiscountUpdateTypedDict": ".discountupdate",
    "Dispute": ".dispute",
    "DisputeTypedDict": ".dispute",
    "DisputesGetRequest": ".disputes_getop",
    "DisputesGetRequestTypedDict": ".disputes_getop",
    "DisputesListQueryParamOrganizationIDFilter": ".disputes_listop",
    "DisputesListQueryParamOrganizationIDFilterTypedDict": ".disputes_listop",
    "DisputesListRequest": ".disputes_listop",
    "DisputesListRequestTypedDict": ".disputes_listop",
    "DisputesListResponse": ".disputes_listop",
    "DisputesListResponseTypedDict": ".disputes_listop",
    "QueryParamOrderIDFilter": ".disputes_listop",
    "QueryParamOrderIDFilterTypedDict": ".disputes_listop",
    "StatusFilter": ".disputes_listop",
    "StatusFilterTypedDict": ".disputes_listop",
    "DisputeSortProperty": ".disputesortproperty",
    "DisputeStatus": ".disputestatus",
    "DownloadableFileCreate": ".downloadablefilecreate",
    "DownloadableFileCreateTypedDict": ".downloadablefilecreate",
    "DownloadableFileRead": ".downloadablefileread",
    "DownloadableFileReadTypedDict": ".downloadablefileread",
    "DownloadableRead": ".downloadableread",
    "DownloadableReadTypedDict": ".downloadableread",
    "Event": ".event",
    "EventTypedDict": ".event",
    "EventTypesListQueryParamCustomerIDFilter": ".event_types_listop",
    "EventTypesListQueryParamCustomerIDFilterTypedDict": ".event_types_listop",
    "EventTypesListQueryParamExternalCustomerIDFilter": ".event_types_listop",
    "EventTypesListQueryParamExternalCustomerIDFilterTypedDict": ".event_types_listop",
    "EventTypesListQueryParamOrganizationIDFilter": ".event_types_listop",
    "EventTypesListQueryParamOrganizationIDFilterTypedDict": ".event_types_listop",
    "EventTypesListRequest": ".event_types_listop",
    "EventTypesListRequestTypedDict": ".event_types_listop",
    "EventTypesListResponse": ".event_types_listop",
    "EventTypesListResponseTypedDict": ".event_types_listop",
    "EventTypesUpdateRequest": ".event_types_updateop",
    "EventTypesUpdateRequestTypedDict": ".event_types_updateop",
    "EventCreateCustomer": ".eventcreatecustomer",
    "EventCreateCustomerTypedDict": ".eventcreatecustomer",
    "EventCreateExternalCustomer": ".eventcreateexternalcustomer",
    "EventCreateExternalCustomerTypedDict": ".eventcreateexternalcustomer",
    "EventMetadataInput": ".eventmetadatainput",
    "EventMetadataInputTypedDict": ".eventmetadatainput",
    "EventMetadataOutput": ".eventmetadataoutput",
    "EventMetadataOutputTypedDict": ".eventmetadataoutput",
    "EventName": ".eventname",
    "EventNameTypedDict": ".eventname",
    "EventNamesSortProperty": ".eventnamessortproperty",
    "EventsGetRequest": ".events_getop",
    "EventsGetRequestTypedDict": ".events_getop",
    "EventsListNamesQueryParamCustomerIDFilter": ".events_list_namesop",
    "EventsListNamesQueryParamCustomerIDFilterTypedDict": ".events_list_namesop",
    "EventsListNamesQueryParamExternalCustomerIDFilter": ".events_list_namesop",
    "EventsListNamesQueryParamExternalCustomerIDFilterTypedDict": ".events_list_namesop",
    "EventsListNamesQueryParamOrganizationIDFilter": ".events_list_namesop",
    "EventsListNamesQueryParamOrganizationIDFilterTypedDict": ".events_list_namesop",
    "EventsListNamesRequest": ".events_list_namesop",
    "EventsListNamesRequestTypedDict": ".events_list_namesop",
    "EventsListNamesResponse": ".events_list_namesop",
    "EventsListNamesResponseTypedDict": ".events_list_namesop",
    "QueryParamSourceFilter": ".events_list_namesop",
    "QueryParamSourceFilterTypedDict": ".events_list_namesop",
    "EventsIngest": ".eventsingest",
    "EventsIngestTypedDict": ".eventsingest",
    "EventsModel": ".eventsingest",
    "EventsModelTypedDict": ".eventsingest",
    "EventsIngestResponse": ".eventsingestresponse",
    "EventsIngestResponseTypedDict": ".eventsingestresponse",
    "EventSortProperty": ".eventsortproperty",
    "EventSource": ".eventsource",
    "EventType": ".eventtype",
    "EventTypeTypedDict": ".eventtype",
    "EventTypesSortProperty": ".eventtypessortproperty",
    "EventTypeUpdate": ".eventtypeupdate",
    "EventTypeUpdateTypedDict": ".eventtypeupdate",
    "EventTypeWithStats": ".eventtypewithstats",
    "EventTypeWithStatsTypedDict": ".eventtypewithstats",
    "ExistingProductPrice": ".existingproductprice",
    "ExistingProductPriceTypedDict": ".existingproductprice",
    "ExpiredCheckoutError": ".expiredcheckouterror",
    "ExpiredCheckoutErrorData": ".expiredcheckouterror",
    "FileCreate": ".filecreate",
    "FileCreateTypedDict": ".filecreate",
    "FileDownload": ".filedownload",
    "FileDownloadTypedDict": ".filedownload",
    "FilePatch": ".filepatch",
    "FilePatchTypedDict": ".filepatch",
    "FilesDeleteRequest": ".files_deleteop",
    "FilesDeleteRequestTypedDict": ".files_deleteop",
    "FileIDFilter": ".files_listop",
    "FileIDFilterTypedDict": ".files_listop",
    "FilesListQueryParamOrganizationIDFilter": ".files_listop",
    "FilesListQueryParamOrganizationIDFilterTypedDict": ".files_listop",
    "FilesListRequest": ".files_listop",
    "FilesListRequestTypedDict": ".files_listop",
    "FilesListResponse": ".files_listop",
    "FilesListResponseTypedDict": ".files_listop",
    "FilesUpdateRequest": ".files_updateop",
    "FilesUpdateRequestTypedDict": ".files_updateop",
    "FilesUpdateResponseFilesUpdate": ".files_updateop",
    "FilesUpdateResponseFilesUpdateTypedDict": ".files_updateop",
    "FilesUploadedRequest": ".files_uploadedop",
    "FilesUploadedRequestTypedDict": ".files_uploadedop",
    "FilesUploadedResponseFilesUploaded": ".files_uploadedop",
    "FilesUploadedResponseFilesUploadedTypedDict": ".files_uploadedop",
    "FileServiceTypes": ".fileservicetypes",
    "FileUpload": ".fileupload",
    "FileUploadTypedDict": ".fileupload",
    "FileUploadCompleted": ".fileuploadcompleted",
    "FileUploadCompletedTypedDict": ".fileuploadcompleted",
    "FilterClause": ".filterclause",
    "FilterClauseTypedDict": ".filterclause",
    "Value": ".filterclause",
    "ValueTypedDict": ".filterclause",
    "FilterConjunction": ".filterconjunction",
    "FilterOperator": ".filteroperator",
    "GenericPayment": ".genericpayment",
    "GenericPaymentTypedDict": ".genericpayment",
    "HTTPValidationError": ".httpvalidationerror",
    "HTTPValidationErrorData": ".httpvalidationerror",
    "IntrospectTokenRequest": ".introspecttokenrequest",
    "IntrospectTokenRequestTokenTypeHint": ".introspecttokenrequest",
    "IntrospectTokenRequestTypedDict": ".introspecttokenrequest",
    "IntrospectTokenResponse": ".introspecttokenresponse",
    "IntrospectTokenResponseTypedDict": ".introspecttokenresponse",
    "TokenType": ".introspecttokenresponse",
    "LegacyRecurringProductPrice": ".legacyrecurringproductprice",
    "LegacyRecurringProductPriceTypedDict": ".legacyrecurringproductprice",
    "LegacyRecurringProductPriceCustom": ".legacyrecurringproductpricecustom",
    "LegacyRecurringProductPriceCustomTypedDict": ".legacyrecurringproductpricecustom",
    "LegacyRecurringProductPriceFixed": ".legacyrecurringproductpricefixed",
    "LegacyRecurringProductPriceFixedTypedDict": ".legacyrecurringproductpricefixed",
    "LegacyRecurringProductPriceFree": ".legacyrecurringproductpricefree",
    "LegacyRecurringProductPriceFreeTypedDict": ".legacyrecurringproductpricefree",
    "LicenseKeysGetActivationRequest": ".license_keys_get_activationop",
    "LicenseKeysGetActivationRequestTypedDict": ".license_keys_get_activationop",
    "LicenseKeysGetRequest": ".license_keys_getop",
    "LicenseKeysGetRequestTypedDict": ".license_keys_getop",
    "LicenseKeysListQueryParamOrganizationIDFilter": ".license_keys_listop",
    "LicenseKeysListQueryParamOrganizationIDFilterTypedDict": ".license_keys_listop",
    "LicenseKeysListRequest": ".license_keys_listop",
    "LicenseKeysListRequestTypedDict": ".license_keys_listop",
    "LicenseKeysListResponse": ".license_keys_listop",
    "LicenseKeysListResponseTypedDict": ".license_keys_listop",
    "QueryParamBenefitIDFilter": ".license_keys_listop",
    "QueryParamBenefitIDFilterTypedDict": ".license_keys_listop",
    "LicenseKeysUpdateRequest": ".license_keys_updateop",
    "LicenseKeysUpdateRequestTypedDict": ".license_keys_updateop",
    "LicenseKeyActivate": ".licensekeyactivate",
    "LicenseKeyActivateConditions": ".licensekeyactivate",
    "LicenseKeyActivateConditionsTypedDict": ".licensekeyactivate",
    "LicenseKeyActivateMeta": ".licensekeyactivate",
    "LicenseKeyActivateMetaTypedDict": ".licensekeyactivate",
    "LicenseKeyActivateTypedDict": ".licensekeyactivate",
    "LicenseKeyActivationBase": ".licensekeyactivationbase",
    "LicenseKeyActivationBaseTypedDict": ".licensekeyactivationbase",
    "Meta": ".licensekeyactivationbase",
    "MetaTypedDict": ".licensekeyactivationbase",
    "LicenseKeyActivationRead": ".licensekeyactivationread",
    "LicenseKeyActivationReadMeta": ".licensekeyactivationread",
    "LicenseKeyActivationReadMetaTypedDict": ".licensekeyactivationread",
    "LicenseKeyActivationReadTypedDict": ".licensekeyactivationread",
    "LicenseKeyCustomer": ".licensekeycustomer",
    "LicenseKeyCustomerTaxID": ".licensekeycustomer",
    "LicenseKeyCustomerTaxIDTypedDict": ".licensekeycustomer",
    "LicenseKeyCustomerTypedDict": ".licensekeycustomer",
    "LicenseKeyDeactivate": ".licensekeydeactivate",
    "LicenseKeyDeactivateTypedDict": ".licensekeydeactivate",
    "LicenseKeyRead": ".licensekeyread",
    "LicenseKeyReadTypedDict": ".licensekeyread",
    "LicenseKeyStatus": ".licensekeystatus",
    "LicenseKeyUpdate": ".licensekeyupdate",
    "LicenseKeyUpdateTypedDict": ".licensekeyupdate",
    "Conditions": ".licensekeyvalidate",
    "ConditionsTypedDict": ".licensekeyvalidate",
    "LicenseKeyValidate": ".licensekeyvalidate",
    "LicenseKeyValidateTypedDict": ".licensekeyvalidate",
    "LicenseKeyWithActivations": ".licensekeywithactivations",
    "LicenseKeyWithActivationsTypedDict": ".licensekeywithactivations",
    "ListResource": ".listresource_",
    "ListResourceTypedDict": ".listresource_",
    "ListResourceBenefit": ".listresource_benefit_",
    "ListResourceBenefitTypedDict": ".listresource_benefit_",
    "ListResourceBenefitGrant": ".listresource_benefitgrant_",
    "ListResourceBenefitGrantTypedDict": ".listresource_benefitgrant_",
    "ListResourceCheckout": ".listresource_checkout_",
    "ListResourceCheckoutTypedDict": ".listresource_checkout_",
    "ListResourceCheckoutLink": ".listresource_checkoutlink_",
    "ListResourceCheckoutLinkTypedDict": ".listresource_checkoutlink_",
    "ListResourceCustomerBenefitGrant": ".listresource_customerbenefitgrant_",
    "ListResourceCustomerBenefitGrantTypedDict": ".listresource_customerbenefitgrant_",
    "ListResourceCustomerCustomerMeter": ".listresource_customercustomermeter_",
    "ListResourceCustomerCustomerMeterTypedDict": ".listresource_customercustomermeter_",
    "ListResourceCustomerMeter": ".listresource_customermeter_",
    "ListResourceCustomerMeterTypedDict": ".listresource_customermeter_",
    "ListResourceCustomerOrder": ".listresource_customerorder_",
    "ListResourceCustomerOrderTypedDict": ".listresource_customerorder_",
    "ListResourceCustomerPaymentMethod": ".listresource_customerpaymentmethod_",
    "ListResourceCustomerPaymentMethodTypedDict": ".listresource_customerpaymentmethod_",
    "ListResourceCustomerSubscription": ".listresource_customersubscription_",
    "ListResourceCustomerSubscriptionTypedDict": ".listresource_customersubscription_",
    "ListResourceCustomerWallet": ".listresource_customerwallet_",
    "ListResourceCustomerWalletTypedDict": ".listresource_customerwallet_",
    "ListResourceCustomerWithMembers": ".listresource_customerwithmembers_",
    "ListResourceCustomerWithMembersTypedDict": ".listresource_customerwithmembers_",
    "ListResourceCustomField": ".listresource_customfield_",
    "ListResourceCustomFieldTypedDict": ".listresource_customfield_",
    "ListResourceDiscount": ".listresource_discount_",
    "ListResourceDiscountTypedDict": ".listresource_discount_",
    "ListResourceDispute": ".listresource_dispute_",
    "ListResourceDisputeTypedDict": ".listresource_dispute_",
    "ListResourceDownloadableRead": ".listresource_downloadableread_",
    "ListResourceDownloadableReadTypedDict": ".listresource_downloadableread_",
    "ListResourceEvent": ".listresource_event_",
    "ListResourceEventTypedDict": ".listresource_event_",
    "ListResourceEventName": ".listresource_eventname_",
    "ListResourceEventNameTypedDict": ".listresource_eventname_",
    "ListResourceEventTypeWithStats": ".listresource_eventtypewithstats_",
    "ListResourceEventTypeWithStatsTypedDict": ".listresource_eventtypewithstats_",
    "FileRead": ".listresource_fileread_",
    "FileReadTypedDict": ".listresource_fileread_",
    "ListResourceFileRead": ".listresource_fileread_",
    "ListResourceFileReadTypedDict": ".listresource_fileread_",
    "ListResourceLicenseKeyRead": ".listresource_licensekeyread_",
    "ListResourceLicenseKeyReadTypedDict": ".listresource_licensekeyread_",
    "ListResourceMember": ".listresource_member_",
    "ListResourceMemberTypedDict": ".listresource_member_",
    "ListResourceMeter": ".listresource_meter_",
    "ListResourceMeterTypedDict": ".listresource_meter_",
    "ListResourceOrder": ".listresource_order_",
    "ListResourceOrderTypedDict": ".listresource_order_",
    "ListResourceOrganization": ".listresource_organization_",
    "ListResourceOrganizationTypedDict": ".listresource_organization_",
    "ListResourceProduct": ".listresource_product_",
    "ListResourceProductTypedDict": ".listresource_product_",
    "ListResourceRefund": ".listresource_refund_",
    "ListResourceRefundTypedDict": ".listresource_refund_",
    "ListResourceSubscription": ".listresource_subscription_",
    "ListResourceSubscriptionTypedDict": ".listresource_subscription_",
    "ListResourceWebhookDelivery": ".listresource_webhookdelivery_",
    "ListResourceWebhookDeliveryTypedDict": ".listresource_webhookdelivery_",
    "ListResourceWebhookEndpoint": ".listresource_webhookendpoint_",
    "ListResourceWebhookEndpointTypedDict": ".listresource_webhookendpoint_",
    "ListResourceWithCursorPaginationEvent": ".listresourcewithcursorpagination_event_",
    "ListResourceWithCursorPaginationEventTypedDict": ".listresourcewithcursorpagination_event_",
    "LLMMetadata": ".llmmetadata",
    "LLMMetadataTypedDict": ".llmmetadata",
    "Member": ".member",
    "MemberTypedDict": ".member",
    "MemberCreate": ".membercreate",
    "MemberCreateTypedDict": ".membercreate",
    "MemberRole": ".memberrole",
    "MembersDeleteMemberRequest": ".members_delete_memberop",
    "MembersDeleteMemberRequestTypedDict": ".members_delete_memberop",
    "MembersGetMemberRequest": ".members_get_memberop",
    "MembersGetMemberRequestTypedDict": ".members_get_memberop",
    "MembersListMembersRequest": ".members_list_membersop",
    "MembersListMembersRequestTypedDict": ".members_list_membersop",
    "MembersListMembersResponse": ".members_list_membersop",
    "MembersListMembersResponseTypedDict": ".members_list_membersop",
    "MembersUpdateMemberRequest": ".members_update_memberop",
    "MembersUpdateMemberRequestTypedDict": ".members_update_memberop",
    "MemberSession": ".membersession",
    "MemberSessionTypedDict": ".membersession",
    "MemberSessionCreate": ".membersessioncreate",
    "MemberSessionCreateTypedDict": ".membersessioncreate",
    "MemberSortProperty": ".membersortproperty",
    "MemberUpdate": ".memberupdate",
    "MemberUpdateTypedDict": ".memberupdate",
    "MetadataOutputType": ".metadataoutputtype",
    "MetadataOutputTypeTypedDict": ".metadataoutputtype",
    "Meter": ".meter",
    "MeterAggregation": ".meter",
    "MeterAggregationTypedDict": ".meter",
    "MeterTypedDict": ".meter",
    "MeterCreate": ".metercreate",
    "MeterCreateAggregation": ".metercreate",
    "MeterCreateAggregationTypedDict": ".metercreate",
    "MeterCreateMetadata": ".metercreate",
    "MeterCreateMetadataTypedDict": ".metercreate",
    "MeterCreateTypedDict": ".metercreate",
    "MeterCreditedMetadata": ".metercreditedmetadata",
    "MeterCreditedMetadataTypedDict": ".metercreditedmetadata",
    "MeterCreditEvent": ".metercreditevent",
    "MeterCreditEventTypedDict": ".metercreditevent",
    "MeterQuantities": ".meterquantities",
    "MeterQuantitiesTypedDict": ".meterquantities",
    "MeterQuantity": ".meterquantity",
    "MeterQuantityTypedDict": ".meterquantity",
    "MeterResetEvent": ".meterresetevent",
    "MeterResetEventTypedDict": ".meterresetevent",
    "MeterResetMetadata": ".meterresetmetadata",
    "MeterResetMetadataTypedDict": ".meterresetmetadata",
    "MetersGetRequest": ".meters_getop",
    "MetersGetRequestTypedDict": ".meters_getop",
    "MetersUpdateRequest": ".meters_updateop",
    "MetersUpdateRequestTypedDict": ".meters_updateop",
    "MeterSortProperty": ".metersortproperty",
    "Aggregation": ".meterupdate",
    "AggregationTypedDict": ".meterupdate",
    "MeterUpdate": ".meterupdate",
    "MeterUpdateMetadata": ".meterupdate",
    "MeterUpdateMetadataTypedDict": ".meterupdate",
    "MeterUpdateTypedDict": ".meterupdate",
    "Metric": ".metric",
    "MetricTypedDict": ".metric",
    "ActiveSubscriptions": ".metricperiod",
    "ActiveSubscriptionsTypedDict": ".metricperiod",
    "ActiveUserByEvent": ".metricperiod",
    "ActiveUserByEventTypedDict": ".metricperiod",
    "AverageOrderValue": ".metricperiod",
    "AverageOrderValueTypedDict": ".metricperiod",
    "AverageRevenuePerUser": ".metricperiod",
    "AverageRevenuePerUserTypedDict": ".metricperiod",
    "CanceledSubscriptions": ".metricperiod",
    "CanceledSubscriptionsCustomerService": ".metricperiod",
    "CanceledSubscriptionsCustomerServiceTypedDict": ".metricperiod",
    "CanceledSubscriptionsLowQuality": ".metricperiod",
    "CanceledSubscriptionsLowQualityTypedDict": ".metricperiod",
    "CanceledSubscriptionsMissingFeatures": ".metricperiod",
    "CanceledSubscriptionsMissingFeaturesTypedDict": ".metricperiod",
    "CanceledSubscriptionsOther": ".metricperiod",
    "CanceledSubscriptionsOtherTypedDict": ".metricperiod",
    "CanceledSubscriptionsSwitchedService": ".metricperiod",
    "CanceledSubscriptionsSwitchedServiceTypedDict": ".metricperiod",
    "CanceledSubscriptionsTooComplex": ".metricperiod",
    "CanceledSubscriptionsTooComplexTypedDict": ".metricperiod",
    "CanceledSubscriptionsTooExpensive": ".metricperiod",
    "CanceledSubscriptionsTooExpensiveTypedDict": ".metricperiod",
    "CanceledSubscriptionsTypedDict": ".metricperiod",
    "CanceledSubscriptionsUnused": ".metricperiod",
    "CanceledSubscriptionsUnusedTypedDict": ".metricperiod",
    "Cashflow": ".metricperiod",
    "CashflowTypedDict": ".metricperiod",
    "CheckoutsConversion": ".metricperiod",
    "CheckoutsConversionTypedDict": ".metricperiod",
    "CheckoutsModel": ".metricperiod",
    "CheckoutsModelTypedDict": ".metricperiod",
    "ChurnRate": ".metricperiod",
    "ChurnRateTypedDict": ".metricperiod",
    "ChurnedSubscriptions": ".metricperiod",
    "ChurnedSubscriptionsTypedDict": ".metricperiod",
    "CommittedMonthlyRecurringRevenue": ".metricperiod",
    "CommittedMonthlyRecurringRevenueTypedDict": ".metricperiod",
    "CommittedSubscriptions": ".metricperiod",
    "CommittedSubscriptionsTypedDict": ".metricperiod",
    "CostPerUser": ".metricperiod",
    "CostPerUserTypedDict": ".metricperiod",
    "Costs": ".metricperiod",
    "CostsTypedDict": ".metricperiod",
    "CumulativeCosts": ".metricperiod",
    "CumulativeCostsTypedDict": ".metricperiod",
    "CumulativeRevenue": ".metricperiod",
    "CumulativeRevenueTypedDict": ".metricperiod",
    "GrossMargin": ".metricperiod",
    "GrossMarginPercentage": ".metricperiod",
    "GrossMarginPercentageTypedDict": ".metricperiod",
    "GrossMarginTypedDict": ".metricperiod",
    "Ltv": ".metricperiod",
    "LtvTypedDict": ".metricperiod",
    "MetricPeriod": ".metricperiod",
    "MetricPeriodTypedDict": ".metricperiod",
    "MonthlyRecurringRevenue": ".metricperiod",
    "MonthlyRecurringRevenueTypedDict": ".metricperiod",
    "NetAverageOrderValue": ".metricperiod",
    "NetAverageOrderValueTypedDict": ".metricperiod",
    "NetCumulativeRevenue": ".metricperiod",
    "NetCumulativeRevenueTypedDict": ".metricperiod",
    "NetRevenue": ".metricperiod",
    "NetRevenueTypedDict": ".metricperiod",
    "NewSubscriptions": ".metricperiod",
    "NewSubscriptionsNetRevenue": ".metricperiod",
    "NewSubscriptionsNetRevenueTypedDict": ".metricperiod",
    "NewSubscriptionsRevenue": ".metricperiod",
    "NewSubscriptionsRevenueTypedDict": ".metricperiod",
    "NewSubscriptionsTypedDict": ".metricperiod",
    "OneTimeProducts": ".metricperiod",
    "OneTimeProductsNetRevenue": ".metricperiod",
    "OneTimeProductsNetRevenueTypedDict": ".metricperiod",
    "OneTimeProductsRevenue": ".metricperiod",
    "OneTimeProductsRevenueTypedDict": ".metricperiod",
    "OneTimeProductsTypedDict": ".metricperiod",
    "OrdersModel": ".metricperiod",
    "OrdersModelTypedDict": ".metricperiod",
    "RenewedSubscriptions": ".metricperiod",
    "RenewedSubscriptionsNetRevenue": ".metricperiod",
    "RenewedSubscriptionsNetRevenueTypedDict": ".metricperiod",
    "RenewedSubscriptionsRevenue": ".metricperiod",
    "RenewedSubscriptionsRevenueTypedDict": ".metricperiod",
    "RenewedSubscriptionsTypedDict": ".metricperiod",
    "Revenue": ".metricperiod",
    "RevenueTypedDict": ".metricperiod",
    "SucceededCheckouts": ".metricperiod",
    "SucceededCheckoutsTypedDict": ".metricperiod",
    "Metrics": ".metrics",
    "MetricsTypedDict": ".metrics",
    "MetricsGetQueryParamCustomerIDFilter": ".metrics_getop",
    "MetricsGetQueryParamCustomerIDFilterTypedDict": ".metrics_getop",
    "MetricsGetQueryParamOrganizationIDFilter": ".metrics_getop",
    "MetricsGetQueryParamOrganizationIDFilterTypedDict": ".metrics_getop",
    "MetricsGetQueryParamProductIDFilter": ".metrics_getop",
    "MetricsGetQueryParamProductIDFilterTypedDict": ".metrics_getop",
    "MetricsGetRequest": ".metrics_getop",
    "MetricsGetRequestTypedDict": ".metrics_getop",
    "QueryParamProductBillingTypeFilter": ".metrics_getop",
    "QueryParamProductBillingTypeFilterTypedDict": ".metrics_getop",
    "MetricsIntervalLimit": ".metricsintervallimit",
    "MetricsIntervalLimitTypedDict": ".metricsintervallimit",
    "MetricsIntervalsLimits": ".metricsintervalslimits",
    "MetricsIntervalsLimitsTypedDict": ".metricsintervalslimits",
    "MetricsLimits": ".metricslimits",
    "MetricsLimitsTypedDict": ".metricslimits",
    "MetricsResponse": ".metricsresponse",
    "MetricsResponseTypedDict": ".metricsresponse",
    "MetricsTotals": ".metricstotals",
    "MetricsTotalsActiveSubscriptions": ".metricstotals",
    "MetricsTotalsActiveSubscriptionsTypedDict": ".metricstotals",
    "MetricsTotalsActiveUserByEvent": ".metricstotals",
    "MetricsTotalsActiveUserByEventTypedDict": ".metricstotals",
    "MetricsTotalsAverageOrderValue": ".metricstotals",
    "MetricsTotalsAverageOrderValueTypedDict": ".metricstotals",
    "MetricsTotalsAverageRevenuePerUser": ".metricstotals",
    "MetricsTotalsAverageRevenuePerUserTypedDict": ".metricstotals",
    "MetricsTotalsCanceledSubscriptions": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsCustomerService": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsCustomerServiceTypedDict": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsLowQuality": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsLowQualityTypedDict": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsMissingFeatures": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsMissingFeaturesTypedDict": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsOther": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsOtherTypedDict": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsSwitchedService": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsSwitchedServiceTypedDict": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsTooComplex": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsTooComplexTypedDict": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsTooExpensive": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsTooExpensiveTypedDict": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsTypedDict": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsUnused": ".metricstotals",
    "MetricsTotalsCanceledSubscriptionsUnusedTypedDict": ".metricstotals",
    "MetricsTotalsCashflow": ".metricstotals",
    "MetricsTotalsCashflowTypedDict": ".metricstotals",
    "MetricsTotalsCheckouts": ".metricstotals",
    "MetricsTotalsCheckoutsConversion": ".metricstotals",
    "MetricsTotalsCheckoutsConversionTypedDict": ".metricstotals",
    "MetricsTotalsCheckoutsTypedDict": ".metricstotals",
    "MetricsTotalsChurnRate": ".metricstotals",
    "MetricsTotalsChurnRateTypedDict": ".metricstotals",
    "MetricsTotalsChurnedSubscriptions": ".metricstotals",
    "MetricsTotalsChurnedSubscriptionsTypedDict": ".metricstotals",
    "MetricsTotalsCommittedMonthlyRecurringRevenue": ".metricstotals",
    "MetricsTotalsCommittedMonthlyRecurringRevenueTypedDict": ".metricstotals",
    "MetricsTotalsCommittedSubscriptions": ".metricstotals",
    "MetricsTotalsCommittedSubscriptionsTypedDict": ".metricstotals",
    "MetricsTotalsCostPerUser": ".metricstotals",
    "MetricsTotalsCostPerUserTypedDict": ".metricstotals",
    "MetricsTotalsCosts": ".metricstotals",
    "MetricsTotalsCostsTypedDict": ".metricstotals",
    "MetricsTotalsCumulativeCosts": ".metricstotals",
    "MetricsTotalsCumulativeCostsTypedDict": ".metricstotals",
    "MetricsTotalsCumulativeRevenue": ".metricstotals",
    "MetricsTotalsCumulativeRevenueTypedDict": ".metricstotals",
    "MetricsTotalsGrossMargin": ".metricstotals",
    "MetricsTotalsGrossMarginPercentage": ".metricstotals",
    "MetricsTotalsGrossMarginPercentageTypedDict": ".metricstotals",
    "MetricsTotalsGrossMarginTypedDict": ".metricstotals",
    "MetricsTotalsLtv": ".metricstotals",
    "MetricsTotalsLtvTypedDict": ".metricstotals",
    "MetricsTotalsMonthlyRecurringRevenue": ".metricstotals",
    "MetricsTotalsMonthlyRecurringRevenueTypedDict": ".metricstotals",
    "MetricsTotalsNetAverageOrderValue": ".metricstotals",
    "MetricsTotalsNetAverageOrderValueTypedDict": ".metricstotals",
    "MetricsTotalsNetCumulativeRevenue": ".metricstotals",
    "MetricsTotalsNetCumulativeRevenueTypedDict": ".metricstotals",
    "MetricsTotalsNetRevenue": ".metricstotals",
    "MetricsTotalsNetRevenueTypedDict": ".metricstotals",
    "MetricsTotalsNewSubscriptions": ".metricstotals",
    "MetricsTotalsNewSubscriptionsNetRevenue": ".metricstotals",
    "MetricsTotalsNewSubscriptionsNetRevenueTypedDict": ".metricstotals",
    "MetricsTotalsNewSubscriptionsRevenue": ".metricstotals",
    "MetricsTotalsNewSubscriptionsRevenueTypedDict": ".metricstotals",
    "MetricsTotalsNewSubscriptionsTypedDict": ".metricstotals",
    "MetricsTotalsOneTimeProducts": ".metricstotals",
    "MetricsTotalsOneTimeProductsNetRevenue": ".metricstotals",
    "MetricsTotalsOneTimeProductsNetRevenueTypedDict": ".metricstotals",
    "MetricsTotalsOneTimeProductsRevenue": ".metricstotals",
    "MetricsTotalsOneTimeProductsRevenueTypedDict": ".metricstotals",
    "MetricsTotalsOneTimeProductsTypedDict": ".metricstotals",
    "MetricsTotalsOrders": ".metricstotals",
    "MetricsTotalsOrdersTypedDict": ".metricstotals",
    "MetricsTotalsRenewedSubscriptions": ".metricstotals",
    "MetricsTotalsRenewedSubscriptionsNetRevenue": ".metricstotals",
    "MetricsTotalsRenewedSubscriptionsNetRevenueTypedDict": ".metricstotals",
    "MetricsTotalsRenewedSubscriptionsRevenue": ".metricstotals",
    "MetricsTotalsRenewedSubscriptionsRevenueTypedDict": ".metricstotals",
    "MetricsTotalsRenewedSubscriptionsTypedDict": ".metricstotals",
    "MetricsTotalsRevenue": ".metricstotals",
    "MetricsTotalsRevenueTypedDict": ".metricstotals",
    "MetricsTotalsSucceededCheckouts": ".metricstotals",
    "MetricsTotalsSucceededCheckoutsTypedDict": ".metricstotals",
    "MetricsTotalsTypedDict": ".metricstotals",
    "MetricType": ".metrictype",
    "MissingInvoiceBillingDetails": ".missinginvoicebillingdetails",
    "MissingInvoiceBillingDetailsData": ".missinginvoicebillingdetails",
    "NoResponseError": ".no_response_error",
    "NotOpenCheckout": ".notopencheckout",
    "NotOpenCheckoutData": ".notopencheckout",
    "NotPaidOrder": ".notpaidorder",
    "NotPaidOrderData": ".notpaidorder",
    "NotPermitted": ".notpermitted",
    "NotPermittedData": ".notpermitted",
    "Oauth2AuthorizeResponseOauth2Authorize": ".oauth2_authorizeop",
    "Oauth2AuthorizeResponseOauth2AuthorizeTypedDict": ".oauth2_authorizeop",
    "Oauth2ClientsOauth2DeleteClientRequest": ".oauth2_clients_oauth2_delete_clientop",
    "Oauth2ClientsOauth2DeleteClientRequestTypedDict": ".oauth2_clients_oauth2_delete_clientop",
    "Oauth2ClientsOauth2GetClientRequest": ".oauth2_clients_oauth2_get_clientop",
    "Oauth2ClientsOauth2GetClientRequestTypedDict": ".oauth2_clients_oauth2_get_clientop",
    "Oauth2ClientsOauth2UpdateClientRequest": ".oauth2_clients_oauth2_update_clientop",
    "Oauth2ClientsOauth2UpdateClientRequestTypedDict": ".oauth2_clients_oauth2_update_clientop",
    "Oauth2RequestTokenRequestBody": ".oauth2_request_tokenop",
    "Oauth2RequestTokenRequestBodyTypedDict": ".oauth2_request_tokenop",
    "Oauth2UserinfoResponseOauth2Userinfo": ".oauth2_userinfoop",
    "Oauth2UserinfoResponseOauth2UserinfoTypedDict": ".oauth2_userinfoop",
    "GrantTypes": ".oauth2clientconfiguration",
    "OAuth2ClientConfiguration": ".oauth2clientconfiguration",
    "OAuth2ClientConfigurationTypedDict": ".oauth2clientconfiguration",
    "TokenEndpointAuthMethod": ".oauth2clientconfiguration",
    "OAuth2ClientConfigurationUpdate": ".oauth2clientconfigurationupdate",
    "OAuth2ClientConfigurationUpdateGrantTypes": ".oauth2clientconfigurationupdate",
    "OAuth2ClientConfigurationUpdateTokenEndpointAuthMethod": ".oauth2clientconfigurationupdate",
    "OAuth2ClientConfigurationUpdateTypedDict": ".oauth2clientconfigurationupdate",
    "OAuth2ClientPublic": ".oauth2clientpublic",
    "OAuth2ClientPublicTypedDict": ".oauth2clientpublic",
    "Order": ".order",
    "OrderCustomFieldData": ".order",
    "OrderCustomFieldDataTypedDict": ".order",
    "OrderDiscount": ".order",
    "OrderDiscountTypedDict": ".order",
    "OrderTypedDict": ".order",
    "OrderBillingReason": ".orderbillingreason",
    "OrderCustomer": ".ordercustomer",
    "OrderCustomerTaxID": ".ordercustomer",
    "OrderCustomerTaxIDTypedDict": ".ordercustomer",
    "OrderCustomerTypedDict": ".ordercustomer",
    "OrderInvoice": ".orderinvoice",
    "OrderInvoiceTypedDict": ".orderinvoice",
    "OrderItemSchema": ".orderitemschema",
    "OrderItemSchemaTypedDict": ".orderitemschema",
    "OrderNotEligibleForRetry": ".ordernoteligibleforretry",
    "OrderNotEligibleForRetryData": ".ordernoteligibleforretry",
    "OrderPaidEvent": ".orderpaidevent",
    "OrderPaidEventTypedDict": ".orderpaidevent",
    "OrderPaidMetadata": ".orderpaidmetadata",
    "OrderPaidMetadataTypedDict": ".orderpaidmetadata",
    "OrderProduct": ".orderproduct",
    "OrderProductTypedDict": ".orderproduct",
    "OrderRefundedEvent": ".orderrefundedevent",
    "OrderRefundedEventTypedDict": ".orderrefundedevent",
    "OrderRefundedMetadata": ".orderrefundedmetadata",
    "OrderRefundedMetadataTypedDict": ".orderrefundedmetadata",
    "OrdersExportQueryParamOrganizationIDFilter": ".orders_exportop",
    "OrdersExportQueryParamOrganizationIDFilterTypedDict": ".orders_exportop",
    "OrdersExportQueryParamProductIDFilter": ".orders_exportop",
    "OrdersExportQueryParamProductIDFilterTypedDict": ".orders_exportop",
    "OrdersExportRequest": ".orders_exportop",
    "OrdersExportRequestTypedDict": ".orders_exportop",
    "OrdersGenerateInvoiceRequest": ".orders_generate_invoiceop",
    "OrdersGenerateInvoiceRequestTypedDict": ".orders_generate_invoiceop",
    "OrdersGenerateInvoiceResponse422OrdersGenerateInvoice": ".orders_generate_invoiceop",
    "OrdersGenerateInvoiceResponse422OrdersGenerateInvoiceUnion": ".orders_generate_invoiceop",
    "OrdersGetRequest": ".orders_getop",
    "OrdersGetRequestTypedDict": ".orders_getop",
    "OrdersInvoiceRequest": ".orders_invoiceop",
    "OrdersInvoiceRequestTypedDict": ".orders_invoiceop",
    "OrdersUpdateRequest": ".orders_updateop",
    "OrdersUpdateRequestTypedDict": ".orders_updateop",
    "OrderSortProperty": ".ordersortproperty",
    "OrderStatus": ".orderstatus",
    "OrderSubscription": ".ordersubscription",
    "OrderSubscriptionTypedDict": ".ordersubscription",
    "OrderUpdate": ".orderupdate",
    "OrderUpdateTypedDict": ".orderupdate",
    "Organization": ".organization",
    "OrganizationTypedDict": ".organization",
    "OrganizationAvatarFileCreate": ".organizationavatarfilecreate",
    "OrganizationAvatarFileCreateTypedDict": ".organizationavatarfilecreate",
    "OrganizationAvatarFileRead": ".organizationavatarfileread",
    "OrganizationAvatarFileReadTypedDict": ".organizationavatarfileread",
    "OrganizationCreate": ".organizationcreate",
    "OrganizationCreateTypedDict": ".organizationcreate",
    "OrganizationCustomerEmailSettings": ".organizationcustomeremailsettings",
    "OrganizationCustomerEmailSettingsTypedDict": ".organizationcustomeremailsettings",
    "OrganizationCustomerPortalSettings": ".organizationcustomerportalsettings",
    "OrganizationCustomerPortalSettingsTypedDict": ".organizationcustomerportalsettings",
    "OrganizationDetails": ".organizationdetails",
    "OrganizationDetailsTypedDict": ".organizationdetails",
    "SwitchingFrom": ".organizationdetails",
    "OrganizationFeatureSettings": ".organizationfeaturesettings",
    "OrganizationFeatureSettingsTypedDict": ".organizationfeaturesettings",
    "OrganizationNotificationSettings": ".organizationnotificationsettings",
    "OrganizationNotificationSettingsTypedDict": ".organizationnotificationsettings",
    "OrganizationsGetRequest": ".organizations_getop",
    "OrganizationsGetRequestTypedDict": ".organizations_getop",
    "OrganizationsListRequest": ".organizations_listop",
    "OrganizationsListRequestTypedDict": ".organizations_listop",
    "OrganizationsListResponse": ".organizations_listop",
    "OrganizationsListResponseTypedDict": ".organizations_listop",
    "OrganizationsUpdateRequest": ".organizations_updateop",
    "OrganizationsUpdateRequestTypedDict": ".organizations_updateop",
    "OrganizationSocialLink": ".organizationsociallink",
    "OrganizationSocialLinkTypedDict": ".organizationsociallink",
    "OrganizationSocialPlatforms": ".organizationsocialplatforms",
    "OrganizationSortProperty": ".organizationsortproperty",
    "OrganizationStatus": ".organizationstatus",
    "OrganizationSubscriptionSettings": ".organizationsubscriptionsettings",
    "OrganizationSubscriptionSettingsTypedDict": ".organizationsubscriptionsettings",
    "OrganizationUpdate": ".organizationupdate",
    "OrganizationUpdateTypedDict": ".organizationupdate",
    "OwnerCreate": ".ownercreate",
    "OwnerCreateTypedDict": ".ownercreate",
    "Pagination": ".pagination",
    "PaginationTypedDict": ".pagination",
    "Payment": ".payment",
    "PaymentTypedDict": ".payment",
    "PaymentAlreadyInProgress": ".paymentalreadyinprogress",
    "PaymentAlreadyInProgressData": ".paymentalreadyinprogress",
    "PaymentError": ".paymenterror",
    "PaymentErrorData": ".paymenterror",
    "PaymentMethodCard": ".paymentmethodcard",
    "PaymentMethodCardTypedDict": ".paymentmethodcard",
    "PaymentMethodCardMetadata": ".paymentmethodcardmetadata",
    "PaymentMethodCardMetadataTypedDict": ".paymentmethodcardmetadata",
    "PaymentMethodGeneric": ".paymentmethodgeneric",
    "PaymentMethodGenericTypedDict": ".paymentmethodgeneric",
    "PaymentMethodInUseByActiveSubscription": ".paymentmethodinusebyactivesubscription",
    "PaymentMethodInUseByActiveSubscriptionData": ".paymentmethodinusebyactivesubscription",
    "PaymentNotReady": ".paymentnotready",
    "PaymentNotReadyData": ".paymentnotready",
    "PaymentProcessor": ".paymentprocessor",
    "PaymentsGetRequest": ".payments_getop",
    "PaymentsGetRequestTypedDict": ".payments_getop",
    "CustomerEmailFilter": ".payments_listop",
    "CustomerEmailFilterTypedDict": ".payments_listop",
    "MethodFilter": ".payments_listop",
    "MethodFilterTypedDict": ".payments_listop",
    "PaymentsListQueryParamCheckoutIDFilter": ".payments_listop",
    "PaymentsListQueryParamCheckoutIDFilterTypedDict": ".payments_listop",
    "PaymentsListQueryParamOrderIDFilter": ".payments_listop",
    "PaymentsListQueryParamOrderIDFilterTypedDict": ".payments_listop",
    "PaymentsListQueryParamOrganizationIDFilter": ".payments_listop",
    "PaymentsListQueryParamOrganizationIDFilterTypedDict": ".payments_listop",
    "PaymentsListQueryParamStatusFilter": ".payments_listop",
    "PaymentsListQueryParamStatusFilterTypedDict": ".payments_listop",
    "PaymentsListRequest": ".payments_listop",
    "PaymentsListRequestTypedDict": ".payments_listop",
    "PaymentsListResponse": ".payments_listop",
    "PaymentsListResponseTypedDict": ".payments_listop",
    "PaymentSortProperty": ".paymentsortproperty",
    "PaymentStatus": ".paymentstatus",
    "PortalAuthenticatedUser": ".portalauthenticateduser",
    "PortalAuthenticatedUserTypedDict": ".portalauthenticateduser",
    "PresentmentCurrency": ".presentmentcurrency",
    "Prices": ".product",
    "PricesTypedDict": ".product",
    "Product": ".product",
    "ProductTypedDict": ".product",
    "ProductBenefitsUpdate": ".productbenefitsupdate",
    "ProductBenefitsUpdateTypedDict": ".productbenefitsupdate",
    "ProductBillingType": ".productbillingtype",
    "ProductCreate": ".productcreate",
    "ProductCreateTypedDict": ".productcreate",
    "ProductCreateOneTime": ".productcreateonetime",
    "ProductCreateOneTimeMetadata": ".productcreateonetime",
    "ProductCreateOneTimeMetadataTypedDict": ".productcreateonetime",
    "ProductCreateOneTimePrices": ".productcreateonetime",
    "ProductCreateOneTimePricesTypedDict": ".productcreateonetime",
    "ProductCreateOneTimeTypedDict": ".productcreateonetime",
    "ProductCreateRecurring": ".productcreaterecurring",
    "ProductCreateRecurringMetadata": ".productcreaterecurring",
    "ProductCreateRecurringMetadataTypedDict": ".productcreaterecurring",
    "ProductCreateRecurringPrices": ".productcreaterecurring",
    "ProductCreateRecurringPricesTypedDict": ".productcreaterecurring",
    "ProductCreateRecurringTypedDict": ".productcreaterecurring",
    "ProductMediaFileCreate": ".productmediafilecreate",
    "ProductMediaFileCreateTypedDict": ".productmediafilecreate",
    "ProductMediaFileRead": ".productmediafileread",
    "ProductMediaFileReadTypedDict": ".productmediafileread",
    "ProductPrice": ".productprice",
    "ProductPriceTypedDict": ".productprice",
    "ProductPriceCustom": ".productpricecustom",
    "ProductPriceCustomTypedDict": ".productpricecustom",
    "ProductPriceCustomCreate": ".productpricecustomcreate",
    "ProductPriceCustomCreateTypedDict": ".productpricecustomcreate",
    "ProductPriceFixed": ".productpricefixed",
    "ProductPriceFixedTypedDict": ".productpricefixed",
    "ProductPriceFixedCreate": ".productpricefixedcreate",
    "ProductPriceFixedCreateTypedDict": ".productpricefixedcreate",
    "ProductPriceFree": ".productpricefree",
    "ProductPriceFreeTypedDict": ".productpricefree",
    "ProductPriceFreeCreate": ".productpricefreecreate",
    "ProductPriceFreeCreateTypedDict": ".productpricefreecreate",
    "ProductPriceMeter": ".productpricemeter",
    "ProductPriceMeterTypedDict": ".productpricemeter",
    "ProductPriceMeteredUnit": ".productpricemeteredunit",
    "ProductPriceMeteredUnitTypedDict": ".productpricemeteredunit",
    "ProductPriceMeteredUnitCreate": ".productpricemeteredunitcreate",
    "ProductPriceMeteredUnitCreateTypedDict": ".productpricemeteredunitcreate",
    "UnitAmount": ".productpricemeteredunitcreate",
    "UnitAmountTypedDict": ".productpricemeteredunitcreate",
    "ProductPriceSeatBased": ".productpriceseatbased",
    "ProductPriceSeatBasedTypedDict": ".productpriceseatbased",
    "ProductPriceSeatBasedCreate": ".productpriceseatbasedcreate",
    "ProductPriceSeatBasedCreateTypedDict": ".productpriceseatbasedcreate",
    "ProductPriceSeatTier": ".productpriceseattier",
    "ProductPriceSeatTierTypedDict": ".productpriceseattier",
    "ProductPriceSeatTiersInput": ".productpriceseattiers_input",
    "ProductPriceSeatTiersInputTypedDict": ".productpriceseattiers_input",
    "ProductPriceSeatTiersOutput": ".productpriceseattiers_output",
    "ProductPriceSeatTiersOutputTypedDict": ".productpriceseattiers_output",
    "ProductPriceSource": ".productpricesource",
    "ProductPriceType": ".productpricetype",
    "ProductsGetRequest": ".products_getop",
    "ProductsGetRequestTypedDict": ".products_getop",
    "ProductsUpdateBenefitsRequest": ".products_update_benefitsop",
    "ProductsUpdateBenefitsRequestTypedDict": ".products_update_benefitsop",
    "ProductsUpdateRequest": ".products_updateop",
    "ProductsUpdateRequestTypedDict": ".products_updateop",
    "ProductSortProperty": ".productsortproperty",
    "ProductUpdate": ".productupdate",
    "ProductUpdateMetadata": ".productupdate",
    "ProductUpdateMetadataTypedDict": ".productupdate",
    "ProductUpdatePrices": ".productupdate",
    "ProductUpdatePricesTypedDict": ".productupdate",
    "ProductUpdateTypedDict": ".productupdate",
    "Two": ".productupdate",
    "TwoTypedDict": ".productupdate",
    "Func": ".propertyaggregation",
    "PropertyAggregation": ".propertyaggregation",
    "PropertyAggregationTypedDict": ".propertyaggregation",
    "RefreshTokenRequest": ".refreshtokenrequest",
    "RefreshTokenRequestTypedDict": ".refreshtokenrequest",
    "Refund": ".refund",
    "RefundTypedDict": ".refund",
    "RefundCreate": ".refundcreate",
    "RefundCreateMetadata": ".refundcreate",
    "RefundCreateMetadataTypedDict": ".refundcreate",
    "RefundCreateTypedDict": ".refundcreate",
    "RefundDispute": ".refunddispute",
    "RefundDisputeTypedDict": ".refunddispute",
    "RefundedAlready": ".refundedalready",
    "RefundedAlreadyData": ".refundedalready",
    "RefundReason": ".refundreason",
    "OrderIDFilter": ".refunds_listop",
    "OrderIDFilterTypedDict": ".refunds_listop",
    "RefundIDFilter": ".refunds_listop",
    "RefundIDFilterTypedDict": ".refunds_listop",
    "RefundsListQueryParamCustomerIDFilter": ".refunds_listop",
    "RefundsListQueryParamCustomerIDFilterTypedDict": ".refunds_listop",
    "RefundsListQueryParamOrganizationIDFilter": ".refunds_listop",
    "RefundsListQueryParamOrganizationIDFilterTypedDict": ".refunds_listop",
    "RefundsListRequest": ".refunds_listop",
    "RefundsListRequestTypedDict": ".refunds_listop",
    "RefundsListResponse": ".refunds_listop",
    "RefundsListResponseTypedDict": ".refunds_listop",
    "SubscriptionIDFilter": ".refunds_listop",
    "SubscriptionIDFilterTypedDict": ".refunds_listop",
    "RefundSortProperty": ".refundsortproperty",
    "RefundStatus": ".refundstatus",
    "ResourceNotFound": ".resourcenotfound",
    "ResourceNotFoundData": ".resourcenotfound",
    "ResponseValidationError": ".responsevalidationerror",
    "RevokeTokenRequest": ".revoketokenrequest",
    "RevokeTokenRequestTypedDict": ".revoketokenrequest",
    "TokenTypeHint": ".revoketokenrequest",
    "RevokeTokenResponse": ".revoketokenresponse",
    "RevokeTokenResponseTypedDict": ".revoketokenresponse",
    "S3DownloadURL": ".s3downloadurl",
    "S3DownloadURLTypedDict": ".s3downloadurl",
    "S3FileCreateMultipart": ".s3filecreatemultipart",
    "S3FileCreateMultipartTypedDict": ".s3filecreatemultipart",
    "S3FileCreatePart": ".s3filecreatepart",
    "S3FileCreatePartTypedDict": ".s3filecreatepart",
    "S3FileUploadCompletedPart": ".s3fileuploadcompletedpart",
    "S3FileUploadCompletedPartTypedDict": ".s3fileuploadcompletedpart",
    "S3FileUploadMultipart": ".s3fileuploadmultipart",
    "S3FileUploadMultipartTypedDict": ".s3fileuploadmultipart",
    "S3FileUploadPart": ".s3fileuploadpart",
    "S3FileUploadPartTypedDict": ".s3fileuploadpart",
    "Scope": ".scope",
    "SDKError": ".sdkerror",
    "SeatAssign": ".seatassign",
    "SeatAssignTypedDict": ".seatassign",
    "SeatClaim": ".seatclaim",
    "SeatClaimTypedDict": ".seatclaim",
    "SeatClaimInfo": ".seatclaiminfo",
    "SeatClaimInfoTypedDict": ".seatclaiminfo",
    "SeatsList": ".seatslist",
    "SeatsListTypedDict": ".seatslist",
    "SeatStatus": ".seatstatus",
    "Security": ".security",
    "SecurityTypedDict": ".security",
    "CustomFieldData": ".subscription",
    "CustomFieldDataTypedDict": ".subscription",
    "Subscription": ".subscription",
    "SubscriptionDiscount": ".subscription",
    "SubscriptionDiscountTypedDict": ".subscription",
    "SubscriptionPrices": ".subscription",
    "SubscriptionPricesTypedDict": ".subscription",
    "SubscriptionTypedDict": ".subscription",
    "SubscriptionBillingPeriodUpdatedEvent": ".subscriptionbillingperiodupdatedevent",
    "SubscriptionBillingPeriodUpdatedEventTypedDict": ".subscriptionbillingperiodupdatedevent",
    "SubscriptionBillingPeriodUpdatedMetadata": ".subscriptionbillingperiodupdatedmetadata",
    "SubscriptionBillingPeriodUpdatedMetadataTypedDict": ".subscriptionbillingperiodupdatedmetadata",
    "SubscriptionCancel": ".subscriptioncancel",
    "SubscriptionCancelTypedDict": ".subscriptioncancel",
    "SubscriptionCanceledEvent": ".subscriptioncanceledevent",
    "SubscriptionCanceledEventTypedDict": ".subscriptioncanceledevent",
    "SubscriptionCanceledMetadata": ".subscriptioncanceledmetadata",
    "SubscriptionCanceledMetadataTypedDict": ".subscriptioncanceledmetadata",
    "Metadata": ".subscriptioncreatecustomer",
    "MetadataTypedDict": ".subscriptioncreatecustomer",
    "SubscriptionCreateCustomer": ".subscriptioncreatecustomer",
    "SubscriptionCreateCustomerTypedDict": ".subscriptioncreatecustomer",
    "SubscriptionCreatedEvent": ".subscriptioncreatedevent",
    "SubscriptionCreatedEventTypedDict": ".subscriptioncreatedevent",
    "SubscriptionCreatedMetadata": ".subscriptioncreatedmetadata",
    "SubscriptionCreatedMetadataTypedDict": ".subscriptioncreatedmetadata",
    "SubscriptionCreateExternalCustomer": ".subscriptioncreateexternalcustomer",
    "SubscriptionCreateExternalCustomerMetadata": ".subscriptioncreateexternalcustomer",
    "SubscriptionCreateExternalCustomerMetadataTypedDict": ".subscriptioncreateexternalcustomer",
    "SubscriptionCreateExternalCustomerTypedDict": ".subscriptioncreateexternalcustomer",
    "SubscriptionCustomer": ".subscriptioncustomer",
    "SubscriptionCustomerTypedDict": ".subscriptioncustomer",
    "TaxID": ".subscriptioncustomer",
    "TaxIDTypedDict": ".subscriptioncustomer",
    "SubscriptionCycledEvent": ".subscriptioncycledevent",
    "SubscriptionCycledEventTypedDict": ".subscriptioncycledevent",
    "SubscriptionCycledMetadata": ".subscriptioncycledmetadata",
    "SubscriptionCycledMetadataTypedDict": ".subscriptioncycledmetadata",
    "SubscriptionLocked": ".subscriptionlocked",
    "SubscriptionLockedData": ".subscriptionlocked",
    "SubscriptionMeter": ".subscriptionmeter",
    "SubscriptionMeterTypedDict": ".subscriptionmeter",
    "SubscriptionProductUpdatedEvent": ".subscriptionproductupdatedevent",
    "SubscriptionProductUpdatedEventTypedDict": ".subscriptionproductupdatedevent",
    "SubscriptionProductUpdatedMetadata": ".subscriptionproductupdatedmetadata",
    "SubscriptionProductUpdatedMetadataTypedDict": ".subscriptionproductupdatedmetadata",
    "SubscriptionProrationBehavior": ".subscriptionprorationbehavior",
    "SubscriptionRecurringInterval": ".subscriptionrecurringinterval",
    "SubscriptionRevoke": ".subscriptionrevoke",
    "SubscriptionRevokeTypedDict": ".subscriptionrevoke",
    "SubscriptionRevokedEvent": ".subscriptionrevokedevent",
    "SubscriptionRevokedEventTypedDict": ".subscriptionrevokedevent",
    "SubscriptionRevokedMetadata": ".subscriptionrevokedmetadata",
    "SubscriptionRevokedMetadataTypedDict": ".subscriptionrevokedmetadata",
    "SubscriptionsCreateSubscriptionCreate": ".subscriptions_createop",
    "SubscriptionsCreateSubscriptionCreateTypedDict": ".subscriptions_createop",
    "OrganizationID": ".subscriptions_exportop",
    "OrganizationIDTypedDict": ".subscriptions_exportop",
    "SubscriptionsExportRequest": ".subscriptions_exportop",
    "SubscriptionsExportRequestTypedDict": ".subscriptions_exportop",
    "SubscriptionsGetRequest": ".subscriptions_getop",
    "SubscriptionsGetRequestTypedDict": ".subscriptions_getop",
    "BenefitIDFilter": ".subscriptions_listop",
    "BenefitIDFilterTypedDict": ".subscriptions_listop",
    "BenefitTypeFilter": ".subscriptions_listop",
    "BenefitTypeFilterTypedDict": ".subscriptions_listop",
    "BenefitsListRequest": ".subscriptions_listop",
    "BenefitsListRequestTypedDict": ".subscriptions_listop",
    "BenefitsListResponse": ".subscriptions_listop",
    "BenefitsListResponseTypedDict": ".subscriptions_listop",
    "CheckoutIDFilter": ".subscriptions_listop",
    "CheckoutIDFilterTypedDict": ".subscriptions_listop",
    "CustomerIDFilter": ".subscriptions_listop",
    "CustomerIDFilterTypedDict": ".subscriptions_listop",
    "CustomersListQueryParamOrganizationIDFilter": ".subscriptions_listop",
    "CustomersListQueryParamOrganizationIDFilterTypedDict": ".subscriptions_listop",
    "CustomersListRequest": ".subscriptions_listop",
    "CustomersListRequestTypedDict": ".subscriptions_listop",
    "CustomersListResponse": ".subscriptions_listop",
    "CustomersListResponseTypedDict": ".subscriptions_listop",
    "DiscountIDFilter": ".subscriptions_listop",
    "DiscountIDFilterTypedDict": ".subscriptions_listop",
    "EventsListQueryParamCustomerIDFilter": ".subscriptions_listop",
    "EventsListQueryParamCustomerIDFilterTypedDict": ".subscriptions_listop",
    "EventsListQueryParamOrganizationIDFilter": ".subscriptions_listop",
    "EventsListQueryParamOrganizationIDFilterTypedDict": ".subscriptions_listop",
    "EventsListRequest": ".subscriptions_listop",
    "EventsListRequestTypedDict": ".subscriptions_listop",
    "EventsListResponseEventsList": ".subscriptions_listop",
    "EventsListResponseEventsListTypedDict": ".subscriptions_listop",
    "ExcludeIDs": ".subscriptions_listop",
    "ExcludeIDsTypedDict": ".subscriptions_listop",
    "ExternalCustomerIDFilter": ".subscriptions_listop",
    "ExternalCustomerIDFilterTypedDict": ".subscriptions_listop",
    "FilterIDs": ".subscriptions_listop",
    "FilterIDsTypedDict": ".subscriptions_listop",
    "MetadataQuery": ".subscriptions_listop",
    "MetadataQueryTypedDict": ".subscriptions_listop",
    "MetersListQueryParamOrganizationIDFilter": ".subscriptions_listop",
    "MetersListQueryParamOrganizationIDFilterTypedDict": ".subscriptions_listop",
    "MetersListRequest": ".subscriptions_listop",
    "MetersListRequestTypedDict": ".subscriptions_listop",
    "MetersListResponse": ".subscriptions_listop",
    "MetersListResponseTypedDict": ".subscriptions_listop",
    "MetersQuantitiesQueryParamCustomerIDFilter": ".subscriptions_listop",
    "MetersQuantitiesQueryParamCustomerIDFilterTypedDict": ".subscriptions_listop",
    "MetersQuantitiesQueryParamExternalCustomerIDFilter": ".subscriptions_listop",
    "MetersQuantitiesQueryParamExternalCustomerIDFilterTypedDict": ".subscriptions_listop",
    "MetersQuantitiesRequest": ".subscriptions_listop",
    "MetersQuantitiesRequestTypedDict": ".subscriptions_listop",
    "NameFilter": ".subscriptions_listop",
    "NameFilterTypedDict": ".subscriptions_listop",
    "OrdersListQueryParamCustomerIDFilter": ".subscriptions_listop",
    "OrdersListQueryParamCustomerIDFilterTypedDict": ".subscriptions_listop",
    "OrdersListQueryParamOrganizationIDFilter": ".subscriptions_listop",
    "OrdersListQueryParamOrganizationIDFilterTypedDict": ".subscriptions_listop",
    "OrdersListQueryParamProductIDFilter": ".subscriptions_listop",
    "OrdersListQueryParamProductIDFilterTypedDict": ".subscriptions_listop",
    "OrdersListRequest": ".subscriptions_listop",
    "OrdersListRequestTypedDict": ".subscriptions_listop",
    "OrdersListResponse": ".subscriptions_listop",
    "OrdersListResponseTypedDict": ".subscriptions_listop",
    "OrganizationIDFilter": ".subscriptions_listop",
    "OrganizationIDFilterTypedDict": ".subscriptions_listop",
    "ProductBillingTypeFilter": ".subscriptions_listop",
    "ProductBillingTypeFilterTypedDict": ".subscriptions_listop",
    "ProductIDFilter": ".subscriptions_listop",
    "ProductIDFilterTypedDict": ".subscriptions_listop",
    "ProductsListQueryParamOrganizationIDFilter": ".subscriptions_listop",
    "ProductsListQueryParamOrganizationIDFilterTypedDict": ".subscriptions_listop",
    "ProductsListRequest": ".subscriptions_listop",
    "ProductsListRequestTypedDict": ".subscriptions_listop",
    "ProductsListResponse": ".subscriptions_listop",
    "ProductsListResponseTypedDict": ".subscriptions_listop",
    "QueryParamDiscountIDFilter": ".subscriptions_listop",
    "QueryParamDiscountIDFilterTypedDict": ".subscriptions_listop",
    "QueryParamExternalCustomerIDFilter": ".subscriptions_listop",
    "QueryParamExternalCustomerIDFilterTypedDict": ".subscriptions_listop",
    "QueryParamOrganizationIDFilter": ".subscriptions_listop",
    "QueryParamOrganizationIDFilterTypedDict": ".subscriptions_listop",
    "QueryParamProductIDFilter": ".subscriptions_listop",
    "QueryParamProductIDFilterTypedDict": ".subscriptions_listop",
    "SourceFilter": ".subscriptions_listop",
    "SourceFilterTypedDict": ".subscriptions_listop",
    "SubscriptionsListRequest": ".subscriptions_listop",
    "SubscriptionsListRequestTypedDict": ".subscriptions_listop",
    "SubscriptionsListResponse": ".subscriptions_listop",
    "SubscriptionsListResponseTypedDict": ".subscriptions_listop",
    "SubscriptionsRevokeRequest": ".subscriptions_revokeop",
    "SubscriptionsRevokeRequestTypedDict": ".subscriptions_revokeop",
    "SubscriptionsUpdateRequest": ".subscriptions_updateop",
    "SubscriptionsUpdateRequestTypedDict": ".subscriptions_updateop",
    "SubscriptionSeatsUpdatedEvent": ".subscriptionseatsupdatedevent",
    "SubscriptionSeatsUpdatedEventTypedDict": ".subscriptionseatsupdatedevent",
    "SubscriptionSeatsUpdatedMetadata": ".subscriptionseatsupdatedmetadata",
    "SubscriptionSeatsUpdatedMetadataTypedDict": ".subscriptionseatsupdatedmetadata",
    "SubscriptionSortProperty": ".subscriptionsortproperty",
    "SubscriptionStatus": ".subscriptionstatus",
    "SubscriptionUncanceledEvent": ".subscriptionuncanceledevent",
    "SubscriptionUncanceledEventTypedDict": ".subscriptionuncanceledevent",
    "SubscriptionUncanceledMetadata": ".subscriptionuncanceledmetadata",
    "SubscriptionUncanceledMetadataTypedDict": ".subscriptionuncanceledmetadata",
    "SubscriptionUpdate": ".subscriptionupdate",
    "SubscriptionUpdateTypedDict": ".subscriptionupdate",
    "SubscriptionUpdateBillingPeriod": ".subscriptionupdatebillingperiod",
    "SubscriptionUpdateBillingPeriodTypedDict": ".subscriptionupdatebillingperiod",
    "SubscriptionUpdateDiscount": ".subscriptionupdatediscount",
    "SubscriptionUpdateDiscountTypedDict": ".subscriptionupdatediscount",
    "SubscriptionUpdateProduct": ".subscriptionupdateproduct",
    "SubscriptionUpdateProductTypedDict": ".subscriptionupdateproduct",
    "SubscriptionUpdateSeats": ".subscriptionupdateseats",
    "SubscriptionUpdateSeatsTypedDict": ".subscriptionupdateseats",
    "SubscriptionUpdateTrial": ".subscriptionupdatetrial",
    "SubscriptionUpdateTrialTypedDict": ".subscriptionupdatetrial",
    "SubType": ".subtype",
    "SystemEvent": ".systemevent",
    "SystemEventTypedDict": ".systemevent",
    "TaxIDFormat": ".taxidformat",
    "TimeInterval": ".timeinterval",
    "TokenResponse": ".tokenresponse",
    "TokenResponseTypedDict": ".tokenresponse",
    "TrialAlreadyRedeemed": ".trialalreadyredeemed",
    "TrialAlreadyRedeemedData": ".trialalreadyredeemed",
    "TrialInterval": ".trialinterval",
    "Unauthorized": ".unauthorized",
    "UnauthorizedData": ".unauthorized",
    "UniqueAggregation": ".uniqueaggregation",
    "UniqueAggregationTypedDict": ".uniqueaggregation",
    "UserEvent": ".userevent",
    "UserEventTypedDict": ".userevent",
    "UserInfoOrganization": ".userinfoorganization",
    "UserInfoOrganizationTypedDict": ".userinfoorganization",
    "UserInfoUser": ".userinfouser",
    "UserInfoUserTypedDict": ".userinfouser",
    "ValidatedLicenseKey": ".validatedlicensekey",
    "ValidatedLicenseKeyTypedDict": ".validatedlicensekey",
    "Loc": ".validationerror",
    "LocTypedDict": ".validationerror",
    "ValidationError": ".validationerror",
    "ValidationErrorTypedDict": ".validationerror",
    "WebhookBenefitCreatedPayload": ".webhookbenefitcreatedpayload",
    "WebhookBenefitCreatedPayloadTypedDict": ".webhookbenefitcreatedpayload",
    "WebhookBenefitGrantCreatedPayload": ".webhookbenefitgrantcreatedpayload",
    "WebhookBenefitGrantCreatedPayloadTypedDict": ".webhookbenefitgrantcreatedpayload",
    "WebhookBenefitGrantCycledPayload": ".webhookbenefitgrantcycledpayload",
    "WebhookBenefitGrantCycledPayloadTypedDict": ".webhookbenefitgrantcycledpayload",
    "WebhookBenefitGrantRevokedPayload": ".webhookbenefitgrantrevokedpayload",
    "WebhookBenefitGrantRevokedPayloadTypedDict": ".webhookbenefitgrantrevokedpayload",
    "WebhookBenefitGrantUpdatedPayload": ".webhookbenefitgrantupdatedpayload",
    "WebhookBenefitGrantUpdatedPayloadTypedDict": ".webhookbenefitgrantupdatedpayload",
    "WebhookBenefitUpdatedPayload": ".webhookbenefitupdatedpayload",
    "WebhookBenefitUpdatedPayloadTypedDict": ".webhookbenefitupdatedpayload",
    "WebhookCheckoutCreatedPayload": ".webhookcheckoutcreatedpayload",
    "WebhookCheckoutCreatedPayloadTypedDict": ".webhookcheckoutcreatedpayload",
    "WebhookCheckoutUpdatedPayload": ".webhookcheckoutupdatedpayload",
    "WebhookCheckoutUpdatedPayloadTypedDict": ".webhookcheckoutupdatedpayload",
    "WebhookCustomerCreatedPayload": ".webhookcustomercreatedpayload",
    "WebhookCustomerCreatedPayloadTypedDict": ".webhookcustomercreatedpayload",
    "WebhookCustomerDeletedPayload": ".webhookcustomerdeletedpayload",
    "WebhookCustomerDeletedPayloadTypedDict": ".webhookcustomerdeletedpayload",
    "WebhookCustomerSeatAssignedPayload": ".webhookcustomerseatassignedpayload",
    "WebhookCustomerSeatAssignedPayloadTypedDict": ".webhookcustomerseatassignedpayload",
    "WebhookCustomerSeatClaimedPayload": ".webhookcustomerseatclaimedpayload",
    "WebhookCustomerSeatClaimedPayloadTypedDict": ".webhookcustomerseatclaimedpayload",
    "WebhookCustomerSeatRevokedPayload": ".webhookcustomerseatrevokedpayload",
    "WebhookCustomerSeatRevokedPayloadTypedDict": ".webhookcustomerseatrevokedpayload",
    "WebhookCustomerStateChangedPayload": ".webhookcustomerstatechangedpayload",
    "WebhookCustomerStateChangedPayloadTypedDict": ".webhookcustomerstatechangedpayload",
    "WebhookCustomerUpdatedPayload": ".webhookcustomerupdatedpayload",
    "WebhookCustomerUpdatedPayloadTypedDict": ".webhookcustomerupdatedpayload",
    "WebhookDelivery": ".webhookdelivery",
    "WebhookDeliveryTypedDict": ".webhookdelivery",
    "WebhookEndpoint": ".webhookendpoint",
    "WebhookEndpointTypedDict": ".webhookendpoint",
    "WebhookEndpointCreate": ".webhookendpointcreate",
    "WebhookEndpointCreateTypedDict": ".webhookendpointcreate",
    "WebhookEndpointUpdate": ".webhookendpointupdate",
    "WebhookEndpointUpdateTypedDict": ".webhookendpointupdate",
    "WebhookEvent": ".webhookevent",
    "WebhookEventTypedDict": ".webhookevent",
    "WebhookEventType": ".webhookeventtype",
    "WebhookFormat": ".webhookformat",
    "WebhookOrderCreatedPayload": ".webhookordercreatedpayload",
    "WebhookOrderCreatedPayloadTypedDict": ".webhookordercreatedpayload",
    "WebhookOrderPaidPayload": ".webhookorderpaidpayload",
    "WebhookOrderPaidPayloadTypedDict": ".webhookorderpaidpayload",
    "WebhookOrderRefundedPayload": ".webhookorderrefundedpayload",
    "WebhookOrderRefundedPayloadTypedDict": ".webhookorderrefundedpayload",
    "WebhookOrderUpdatedPayload": ".webhookorderupdatedpayload",
    "WebhookOrderUpdatedPayloadTypedDict": ".webhookorderupdatedpayload",
    "WebhookOrganizationUpdatedPayload": ".webhookorganizationupdatedpayload",
    "WebhookOrganizationUpdatedPayloadTypedDict": ".webhookorganizationupdatedpayload",
    "WebhookProductCreatedPayload": ".webhookproductcreatedpayload",
    "WebhookProductCreatedPayloadTypedDict": ".webhookproductcreatedpayload",
    "WebhookProductUpdatedPayload": ".webhookproductupdatedpayload",
    "WebhookProductUpdatedPayloadTypedDict": ".webhookproductupdatedpayload",
    "WebhookRefundCreatedPayload": ".webhookrefundcreatedpayload",
    "WebhookRefundCreatedPayloadTypedDict": ".webhookrefundcreatedpayload",
    "WebhookRefundUpdatedPayload": ".webhookrefundupdatedpayload",
    "WebhookRefundUpdatedPayloadTypedDict": ".webhookrefundupdatedpayload",
    "WebhooksDeleteWebhookEndpointRequest": ".webhooks_delete_webhook_endpointop",
    "WebhooksDeleteWebhookEndpointRequestTypedDict": ".webhooks_delete_webhook_endpointop",
    "WebhooksGetWebhookEndpointRequest": ".webhooks_get_webhook_endpointop",
    "WebhooksGetWebhookEndpointRequestTypedDict": ".webhooks_get_webhook_endpointop",
    "EndpointID": ".webhooks_list_webhook_deliveriesop",
    "EndpointIDTypedDict": ".webhooks_list_webhook_deliveriesop",
    "WebhooksListWebhookDeliveriesRequest": ".webhooks_list_webhook_deliveriesop",
    "WebhooksListWebhookDeliveriesRequestTypedDict": ".webhooks_list_webhook_deliveriesop",
    "WebhooksListWebhookDeliveriesResponse": ".webhooks_list_webhook_deliveriesop",
    "WebhooksListWebhookDeliveriesResponseTypedDict": ".webhooks_list_webhook_deliveriesop",
    "QueryParamOrganizationID": ".webhooks_list_webhook_endpointsop",
    "QueryParamOrganizationIDTypedDict": ".webhooks_list_webhook_endpointsop",
    "WebhooksListWebhookEndpointsRequest": ".webhooks_list_webhook_endpointsop",
    "WebhooksListWebhookEndpointsRequestTypedDict": ".webhooks_list_webhook_endpointsop",
    "WebhooksListWebhookEndpointsResponse": ".webhooks_list_webhook_endpointsop",
    "WebhooksListWebhookEndpointsResponseTypedDict": ".webhooks_list_webhook_endpointsop",
    "WebhooksRedeliverWebhookEventRequest": ".webhooks_redeliver_webhook_eventop",
    "WebhooksRedeliverWebhookEventRequestTypedDict": ".webhooks_redeliver_webhook_eventop",
    "WebhooksResetWebhookEndpointSecretRequest": ".webhooks_reset_webhook_endpoint_secretop",
    "WebhooksResetWebhookEndpointSecretRequestTypedDict": ".webhooks_reset_webhook_endpoint_secretop",
    "WebhooksUpdateWebhookEndpointRequest": ".webhooks_update_webhook_endpointop",
    "WebhooksUpdateWebhookEndpointRequestTypedDict": ".webhooks_update_webhook_endpointop",
    "WebhookSubscriptionActivePayload": ".webhooksubscriptionactivepayload",
    "WebhookSubscriptionActivePayloadTypedDict": ".webhooksubscriptionactivepayload",
    "WebhookSubscriptionCanceledPayload": ".webhooksubscriptioncanceledpayload",
    "WebhookSubscriptionCanceledPayloadTypedDict": ".webhooksubscriptioncanceledpayload",
    "WebhookSubscriptionCreatedPayload": ".webhooksubscriptioncreatedpayload",
    "WebhookSubscriptionCreatedPayloadTypedDict": ".webhooksubscriptioncreatedpayload",
    "WebhookSubscriptionPastDuePayload": ".webhooksubscriptionpastduepayload",
    "WebhookSubscriptionPastDuePayloadTypedDict": ".webhooksubscriptionpastduepayload",
    "WebhookSubscriptionRevokedPayload": ".webhooksubscriptionrevokedpayload",
    "WebhookSubscriptionRevokedPayloadTypedDict": ".webhooksubscriptionrevokedpayload",
    "WebhookSubscriptionUncanceledPayload": ".webhooksubscriptionuncanceledpayload",
    "WebhookSubscriptionUncanceledPayloadTypedDict": ".webhooksubscriptionuncanceledpayload",
    "WebhookSubscriptionUpdatedPayload": ".webhooksubscriptionupdatedpayload",
    "WebhookSubscriptionUpdatedPayloadTypedDict": ".webhooksubscriptionupdatedpayload",
    "WebTokenRequest": ".webtokenrequest",
    "WebTokenRequestSubType": ".webtokenrequest",
    "WebTokenRequestTypedDict": ".webtokenrequest",
}


def dynamic_import(modname, retries=3):
    for attempt in range(retries):
        try:
            return import_module(modname, __package__)
        except KeyError:
            # Clear any half-initialized module and retry
            sys.modules.pop(modname, None)
            if attempt == retries - 1:
                break
    raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")


def __getattr__(attr_name: str) -> object:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(
            f"No {attr_name} found in _dynamic_imports for module name -> {__name__} "
        )

    try:
        module = dynamic_import(module_name)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(
            f"Failed to import {attr_name} from {module_name}: {e}"
        ) from e
    except AttributeError as e:
        raise AttributeError(
            f"Failed to get {attr_name} from {module_name}: {e}"
        ) from e


def __dir__():
    lazy_attrs = builtins.list(_dynamic_imports.keys())
    return builtins.sorted(lazy_attrs)
